# -*- coding: utf-8 -*-

import datetime
import autocomplete_light
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from jmb.core.admin import ConstrainedModelForm
from jmb.core.admin.options import JumboActionForm
from publieditor.event.models import ContactGroup, DelegatedContact, Event, Newspaper, Membership, Contact
from publieditor.event.validators import validate_excel_extensions


class ContactActionForm(JumboActionForm):
    """
    Action form used in ContactAdmin to perform available actions
    """
    today = datetime.date.today
    contact_group = forms.ModelChoiceField(
        queryset=ContactGroup.objects.filter(status=1),
        label=_('Group'),
        required=False,
    )
    event = forms.ModelChoiceField(
        queryset=Event.objects.filter(
            status__exact=1, event_adhesion_deadline__gte=today,
        ),
        label=_('Event'),
        required=False,
    )


class ContactGroupActionForm(JumboActionForm):
    """
    Action form used in ContactGroupAdmin to perform available actions
    """
    today = datetime.date.today
    event = forms.ModelChoiceField(
        queryset=Event.objects.filter(
            status__exact=1, event_adhesion_deadline__gte=today,
        ),
        label=_('Event'),
        required=False,
    )


class MailingListActionForm(JumboActionForm):
    """
    This is no longer used because mailing list support was dropped
    Please check before deleting
    """
    today = datetime.date.today
    event = forms.ModelChoiceField(
        queryset=Event.objects.filter(
            status__exact=1, event_adhesion_deadline__gte=today,
        ),
        label=_('Event'),
        required=False,
    )


class InviteActionForm(JumboActionForm):
    """
    Action form used in InviteAdmin to perform available actions
    """
    test = forms.BooleanField(
        label=_("Test"),
        required=False,
    )
    status = forms.ChoiceField(
        choices=settings.PUBLIEDITOR_EVENT_INVITE_STATUS_CODES,
        label=_("Status"),
        required=False,
    )

    call_status = forms.ChoiceField(
        choices=settings.PUBLIEDITOR_EVENT_PHONE_CONTACT_RESULT,
        label=_("Status"),
        required=False,
    )


class EventActionForm(JumboActionForm):
    """
    Action form used in EventAdmin to perform available actions
    """
    test = forms.BooleanField(
        required=False,
        label=_("Test"),
    )


class MembershipForm(ConstrainedModelForm):
    """
    Form used in MembershipAdmin to hide contact field and filter newspaper choices
    """
    newspaper = forms.ModelChoiceField(
        queryset=Newspaper.objects.filter(status=True), required=True,
        widget=autocomplete_light.ChoiceWidget('NewspaperAutocomplete'),
        label=_("Newspaper"),
    )
    hidden_fields = ('contact',)

    def clean(self):
        default = self.cleaned_data['default']
        contact = self.cleaned_data['contact']
        if not default and contact.memberships.count() == 0:
            raise forms.ValidationError("No default role set for current contact")
        return self.cleaned_data

    class Meta:
        model = Membership


class DelegatedContactActionForm(JumboActionForm):
    """
    Action form used in DelegatedContactAdmin to perform available actions
    """
    test = forms.BooleanField(
        required=False,
        label=_("Test"),
    )


class InviteSetDelegatedAdminForm(forms.Form):
    """
    Form used in invite_set_delegated admin_view
    """
    contact = forms.ModelChoiceField(
        queryset=Contact.objects, required=True,
        widget=autocomplete_light.ChoiceWidget('ContactAutocomplete'),
        label=_("Contact"),
    )
    has_to_add = forms.BooleanField(
        label=_("Add to event invited list"),
        initial=True,
        required=False,
    )

    def __init__(self, *args, **kwargs):
        self.invite_id = kwargs.pop('invite_id')
        super(InviteSetDelegatedAdminForm, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        """
        Sets desired invite in deleagated status and manages delegated invite creation
        :return: TUple containing result boolean and delegated contact
        """
        from publieditor.event.models import Invite
        invite = Invite.objects.get(id=self.invite_id)
        invite.status = settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
        invite.save()
        invite.delete_all_answers()
        contact = self.cleaned_data['contact']
        # Salvo DelegatedContact
        delegated_contact = DelegatedContact.objects.create(
            invited=contact, invite=invite,
            title=contact.title, first_name=contact.first_name, last_name=contact.last_name,
            email=contact.email, phone_number="",
        )
        if "has_to_add" in self.cleaned_data and self.cleaned_data["has_to_add"]:
            Invite.objects.get_or_create(
                contact=contact, event=invite.event,
                defaults={'status': settings.PUBLIEDITOR_EVENT_INVITE_PENDING}
            )
        return True, delegated_contact


class ContactImportDataForm(forms.Form):
    """
    Form used in ContactImportView admin view to manage contacts import
    """
    file_to_import = forms.FileField(
        required=True,
        label=_('Select a file'),
        validators=[validate_excel_extensions],
    )
    do_overwrite = forms.BooleanField(
        required=False,
        label=_('Overwrite contacts with email found in file')
    )
