#! -*- coding: utf-8 -*-

import ast, datetime
from django.conf import settings
from django.db import transaction
from django.utils.translation import ugettext_lazy as _

from jmb.address.models import AddressScope
from jmb.core.utils.data_import import XlsFile
from jmb.i18n.models import Language
from jmb.l10n.models import Country
from publieditor.event.models import (
    Contact, ContactEmailCc, ContactGroup, Airline, Card, License, Newspaper, Role, Membership, Phone, AreaCode,
    ContactAddress, ContactSocialMedia
)


class ContactImportData(XlsFile):
    """
    Contacts import data for contacts admin import from xls file
    """
    fields_map = {
        # id
        "title": "title",
        # language
        "first_name": "first_name",
        "last_name": "last_name",
        "company": "company",
        "birth_place": "birth_place",
        "birthday": "birthday",
        "email": "email",
        # email_cc
        # groups
        # phone_area_code
        # phone_number
        # mobile_area_code
        # mobile_number
        # fax_area_code
        # fax_number
        # default_address_type
        # default_address_name
        # default_address_number
        # default_address_locality
        # default_address_zip_code
        # default_address_province
        # default_address_country
        # default_address_scopes
        # memberships
        # frequent_flyer_cards
        "privacy_acceptance_date": "privacy_acceptance_date",
        "fca_privacy_acceptance_date": "fca_privacy_acceptance_date",
        "smoking": "smoking",
        "food": "food",
        "shirt_size": "shirt_size",
        "height": "height",
        "weight": "weight",
        "sweater_size": "sweater_size",
        "shoes_size": "shoes_size",
        "ski": "ski",
        "ski_level": "ski_level",
        "ski_equipment": "ski_equipment",
        "ski_boots_size": "ski_boots_size",
        "id_card_number": "id_card_number",
        "id_card_issue_date": "id_card_issue_date",
        "id_card_expiration_date": "id_card_expiration_date",
        "passport_number": "passport_number",
        "passport_issue_date": "passport_issue_date",
        "passport_expiration_date": "passport_expiration_date",
        "license_number": "license_number",
        "license_issue_date": "license_issue_date",
        "license_expiration_date": "license_expiration_date",
        "license_emitted_by": "license_emitted_by",
        # license_types
        "press_card_number": "press_card_number",
        "elenco": "elenco",
        "food_note": "food_note",
        "note": "note",
        "internal_note": "internal_note",
    }
    dates_field = [
        "birthday", "privacy_acceptance_date", "id_card_issue_date", "id_card_expiration_date",
        "passport_issue_date", "passport_expiration_date", "license_issue_date", "license_expiration_date"
    ]

    def __init__(self, filename=None, file_contents=None, auto=False, sheet_index=0, opts=None, model=None):
        """
        Just redefined to set instance property do_overwrite based on user form selection
        """
        super(ContactImportData, self).__init__(filename, file_contents, auto, sheet_index, opts, model)
        self.do_overwrite = self.opts.get('do_overwrite', False)

    def append_exception_error_messages(self, exception, field_name, row_index, contact_created=False):
        if not contact_created:
            self.messages['errors'].append(_(
                u'Contatto non creato: problema alla riga "{row_index}" colonna "{field_name}": {exception}'.format(
                    exception=exception,
                    row_index=row_index+1,
                    field_name=field_name,
                )))
        else:
            self.messages['errors'].append(_(
                u'Contatto creato: problema alla riga "{row_index}" colonna "{field_name}": {exception}'.format(
                    exception=exception,
                    row_index=row_index+1,
                    field_name=field_name,
                )))

    def do(self, row, kw, j):
        try:
            with transaction.atomic():
                kw['email'] = kw['email'].lower()
                language_code = row.language if hasattr(row, 'language') else 'it'
                try:
                    kw['language'] = Language.objects.get(code__iexact=language_code)
                except Exception as e:
                    self.append_exception_error_messages(e, 'language', j, contact_created=False)
                    return

                for date_field in self.dates_field:
                    if date_field in kw and kw[date_field]:
                        try:
                            kw[date_field] = datetime.datetime.strptime(kw[date_field], '%d/%m/%Y')
                        except Exception as e:
                            self.append_exception_error_messages(e, date_field, j, contact_created=False)
                            return
                if "press_card_number" in kw and kw["press_card_number"]:
                    kw["press_card_number"] = "{}".format(int(kw["press_card_number"]))
                can_proceed, contact = self.manage_contact(kw)
                if can_proceed:
                    try:
                        self.manage_email_cc(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'email_cc', j, contact_created=True)

                    try:
                        self.manage_groups(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'groups', j, contact_created=True)

                    try:
                        self.manage_phones(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'phone_number, mobile_number, fax_number', j, contact_created=True)

                    try:
                        self.manage_other_phones(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'other_phones', j, contact_created=True)

                    try:
                        self.manage_addresses(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'default_address_name, default_address_type, default_address_number, default_address_locality, default_address_zip_code, default_address_province, default_address_country, default_address_scopes', j, contact_created=True)

                    try:
                        self.manage_social_media_links(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'social_media', j, contact_created=True)

                    try:
                        self.manage_memberships(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'memberships', j, contact_created=True)

                    try:
                        self.manage_cards(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'frequent_flyers', j, contact_created=True)

                    try:
                        self.manage_licenses(contact, row)
                    except Exception as e:
                        self.append_exception_error_messages(e, 'license_types', j, contact_created=True)

                else:
                    self.messages['errors'].append(
                        _("Email {} already found, Contact not updated".format(kw['email']))
                    )
        except Exception as e:
            self.messages['errors'].append(_("Errore generico alla riga {}: {}".format(j, e)))

    def manage_contact(self, kw):
        """
        Always create contact if not found.
        If contact exists saves file values only if do_overwrite is True
        :param kw:
        :return:
        """
        email = kw['email']
        contact_exists = bool(Contact.objects.filter(email__iexact=email).count())
        if not contact_exists:
            contact = Contact.objects.create(**kw)
            self.messages['inserted'] += 1
            return True, contact
        elif contact_exists and self.do_overwrite:
            contact = Contact.objects.filter(email__iexact=email)[0]
            if "privacy_acceptance_date" in kw:
                del kw["privacy_acceptance_date"]
            if contact.language and "language" in kw:
                del kw["language"]
            contact.__dict__.update(kw)
            contact.save()
            contact.cc_emails.all().delete()
            contact.groups.all().delete()
            contact.phones.all().delete()
            contact.addresses.all().delete()
            contact.memberships.all().delete()
            contact.social_media_links.all().delete()
            contact.cards.all().delete()
            contact.license_types.all().delete()
            contact.social_media_links.all().delete()
            self.messages['modified'] += 1
            return True, contact
        return False, None

    @staticmethod
    def manage_email_cc(contact, row):
        if hasattr(row, 'email_cc'):
            for email_cc in [email_cc.strip() for email_cc in row.email_cc.split(",") if email_cc.strip()]:
                ContactEmailCc.objects.create(contact=contact, email=email_cc)

    @staticmethod
    def manage_groups(contact, row):
        if hasattr(row, 'groups'):
            for group in [group.strip() for group in row.groups.split(",") if group.strip()]:
                contact_group, created = ContactGroup.objects.get_or_create(name=group)
                contact.groups.add(contact_group)

    @staticmethod
    def manage_phones(contact, row):
        phone_key_map = {
            "phone_number": {"phone_type": "P", "area_code_field": "phone_area_code"},
            "mobile_number": {"phone_type": "M", "area_code_field": "mobile_area_code"},
            "fax_number": {"phone_type": "F", "area_code_field": "fax_area_code"},
        }
        for phone_key in phone_key_map.keys():
            if hasattr(row, phone_key):
                number = getattr(row, phone_key).strip()
                default = True if phone_key == "mobile_number" else False
                area_code_number = getattr(
                    row, phone_key_map[phone_key]["area_code_field"]
                ).strip().replace(" ", "") if hasattr(
                    row, phone_key_map[phone_key]["area_code_field"]
                ) else None
                area_code = AreaCode.objects.filter(code__iexact=area_code_number)[0] \
                    if area_code_number and AreaCode.objects.filter(code__iexact=area_code_number).count() \
                    else None
                if number:
                    Phone.objects.create(
                        contact=contact, area_code=area_code, number=number, default=default,
                        phone_type=phone_key_map[phone_key]["phone_type"]
                    )

    @staticmethod
    def manage_other_phones(contact, row):
        if hasattr(row, 'other_phones'):
            phone_maps = None
            if "]," in row.other_phones:
                phone_maps = [p.rstrip("],") for p in row.other_phones.split("[") if p.strip()]
            elif row.other_phones:
                app = row.other_phones
                phone_maps = [app[app.index("[")+1:app.index("]")]]
            if phone_maps:
                for phone_map in phone_maps:
                    phone_type, area_code_number, phone_number = phone_map.split(":")
                    area_code_number = None if area_code_number.strip() == "" else area_code_number.replace(" ", "")
                    if phone_number:
                        try:
                            area_code = AreaCode.objects.get(code__iexact=area_code_number)
                        except AreaCode.DoesNotExist:
                            area_code = None
                        Phone.objects.create(
                            contact=contact, area_code=area_code, number=phone_number,
                            phone_type=phone_type
                        )

    @staticmethod
    def manage_addresses(contact, row):
        if hasattr(row, 'default_address_name'):
            contact_address = ContactAddress(
                contact=contact, name=row.default_address_name.strip(), default=True
            )
            # Type
            if hasattr(row, 'default_address_type') and row.default_address_type.strip():
                for type_code, type_name in settings.ADDRESS_ADDRESS_TYPE_CHOICES:
                    if type_name == row.default_address_type:
                        contact_address.address_type = type_code
            # Number
            if hasattr(row, 'default_address_number') and row.default_address_number.strip():
                contact_address.number = row.default_address_number.strip()[:20]
            # Locality
            if hasattr(row, 'default_address_locality') and row.default_address_locality.strip():
                contact_address.locality = row.default_address_locality.strip()[:100]
            # ZIp code
            if hasattr(row, 'default_address_zip_code') and row.default_address_zip_code.strip():
                contact_address.zip_code = row.default_address_zip_code.strip()[:20]
            # Province
            if hasattr(row, 'default_address_province') and row.default_address_province.strip():
                contact_address.province = row.default_address_province.strip()[:2]
            # Country
            if hasattr(row, 'default_address_country') and row.default_address_country.strip():
                country_code = row.default_address_country.strip()
                if Country.objects.filter(iso2_code__iexact=country_code).count():
                    contact_address.country = Country.objects.filter(iso2_code__iexact=country_code)[0]
            # Address save and scopes management
            contact_address.save()
            if hasattr(row, 'default_address_scopes') and row.default_address_scopes.strip():
                for scope in [sc.strip() for sc in row.default_address_scopes.split(".") if sc.strip()]:
                    address_scope, created = AddressScope.objects.get_or_create(name=scope)
                    contact_address.address_scopes.add(address_scope)

    @staticmethod
    def manage_memberships(contact, row):
        if hasattr(row, 'memberships'):
            membership_maps = None
            if "]," in row.memberships:
                membership_maps = [m.rstrip("],") for m in row.memberships.split("[") if m.strip()]
            elif row.memberships:
                app = row.memberships
                membership_maps = [app[app.index("[")+1:app.index("]")]]
            if membership_maps:
                for membership_map in membership_maps:
                    has_default_membership = bool(contact.memberships.filter(default=True))
                    if ":" in membership_map:
                        newspaper_name, role_name = membership_map.split(":")
                    else:
                        newspaper_name, role_name = (membership_map, None)
                    newspaper, created = Newspaper.objects.get_or_create(name=newspaper_name)
                    membership = Membership(contact=contact, newspaper=newspaper)
                    if role_name:
                        role, created = Role.objects.get_or_create(name=role_name)
                        membership.role = role
                    if not has_default_membership:
                        membership.default = True
                    membership.save()

    @staticmethod
    def manage_social_media_links(contact, row):
        if hasattr(row, 'social_media'):
            maps = None
            if "]," in row.social_media:
                maps = [m.rstrip("],") for m in row.social_media.split("[") if m.strip()]
            elif row.social_media:
                app = row.social_media
                maps = [app[app.index("[")+1:app.index("]")]]
            if maps:
                for social_media_map in maps:
                    index = social_media_map.index(":")
                    social, link = social_media_map[:index],  social_media_map[index+1:]
                    social_media_link = ContactSocialMedia(
                        contact=contact, social=social, link=link
                    )
                    social_media_link.save()

    @staticmethod
    def manage_cards(contact, row):
        if hasattr(row, 'frequent_flyers'):
            for card_number in [card.strip() for card in row.frequent_flyers.split(",") if card.strip()]:
                card = Card(contact=contact)
                if bool(Airline.objects.filter(iata_code__iexact=card_number[:2]).count()):
                    card.airline = Airline.objects.filter(iata_code__iexact=card_number[:2])[0]
                    card.number = card_number[2:]
                else:
                    card.number = card_number
                card.save()

    @staticmethod
    def manage_licenses(contact, row):
        if hasattr(row, 'license_types'):
            for license_type in [lt.strip() for lt in row.license_types.split(",") if lt.strip()]:
                license, created = License.objects.get_or_create(name=license_type)
                contact.license_types.add(license)
