#! -*- coding: utf-8 -*-
import os
import xlwt
import datetime
import json
import logging

from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, Http404
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, get_object_or_404, render_to_response
from django.template import RequestContext
from django.template.base import Context
from django.template import loader
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugettext
from django.views.generic import TemplateView, View
from django.utils.text import slugify
from django.utils.safestring import mark_safe

from jmb.i18n.models import Language
from publieditor.event.models import Event, Contact, Invite, EventStatusDateStatistic,\
    EventStatusStatistic, EventLinkDateStatistic, EventLinkStatistic, LastMailingStatusProcessed
from publieditor.event.admin_forms import InviteSetDelegatedAdminForm, ContactImportDataForm
from publieditor.event.admin_imports import ContactImportData
from jmb.core.views.import_data import ImportDataView
from publieditor.event.public_forms import AdhesionForm


def _get_status(status_ind):
    """
    Simple util used to get correct readable value for a mailing status option
    :param status_ind: db mailing status option
    :return: Correct status readable value or 404 error
    """
    mailer_status = settings.MAILER_MAILINGSTATUS_STATUS_CHOICES
    for status in mailer_status:
        if status[0] == int(status_ind):
            return status
    raise Http404('Stato non presente')


def get_privacy_default_text(request):
    """
    Simple util used to get correct privacy for a given language.
    I this this in no longer used, event privacy ar now used
    :param request: Current request
    :return: Json HttpResponse empty in case of error or correct message on success
    """
    language = Language.objects.get(id=request.GET['language_id'])
    if language.code in settings.PUBLIEDITOR_EVENT_PRIVACY_DEFAULT_TEXT:
        return HttpResponse(
            json.dumps(dict(msg=settings.PUBLIEDITOR_EVENT_PRIVACY_DEFAULT_TEXT[language.code])),
            content_type="application/json"
        )
    else:
        return HttpResponse(json.dumps(dict()), content_type="application/json")


def export_contact_vcard(request, contact_id):
    """
    Generic view used to get a vcard for a selected contact
    """
    from publieditor.event.utils.views_utils import vcard_string
    contact = Contact.objects.get(pk=contact_id)
    output = vcard_string(contact)
    filename = "%s%s.vcf" % (
        contact.first_name, contact.last_name
    )
    response = HttpResponse(output, mimetype="text/x-vCard")
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return response


def get_adhesion_default_text(request):
    """
    Get adhesion text based on request language
    :param request: current request
    :return: Json response with text on success, empty on error
    """
    language = Language.objects.get(id=request.GET['language_id'])
    if language.code in settings.PUBLIEDITOR_EVENT_ADHESION_DEFAULT_TEXT:
        return HttpResponse(
            json.dumps(dict(msg=settings.PUBLIEDITOR_EVENT_ADHESION_DEFAULT_TEXT[language.code])),
            content_type="application/json"
        )
    else:
        return HttpResponse(json.dumps(dict(msg=None)), content_type="application/json")


def initialize_event(request):
    """
    Used to create a new event starting from the default one, which is defined in settings
    :param request: Current request
    :return: Redirect to new event changelist
    """
    from jmb.core.utils import models_utils
    logger = logging.getLogger('jmb')

    event = Event.objects.get(id=settings.DEFAULT_EVENT_ID)
    model_object_copier = models_utils.ModelObjectCopier(
        event,
        related_parent_name='event',
        related_managers_excludes=['invites', 'travel_options', 'event_statistic', 'event_status_data_email_statistic'],
        kwargs_excludes=['date_create', 'date_last_modify', 'creator', 'last_modifier'],
    )
    done, msg, new_event = model_object_copier.bulk_copy()

    logger.debug('bulk_copy:\n\tdone:{done}\n\tmsg:{msg}\n\tnew_event:{new_event}'.format(
        done=done,
        msg=msg,
        new_event=new_event,
    ))

    url = reverse("admin:event_event_change", args=[new_event.id])
    return redirect(url)


def delegatedcontact_create_invite(request, delegatedcontact_id):
    """
    Creates a new invite for a delegated contact
    :param request: Current request
    :param delegatedcontact_id: Id of delegated contact, which was already created
    :return: Redirect to new invite changelist
    """
    from publieditor.event.models import DelegatedContact, Invite
    delegated_contact = DelegatedContact.objects.get(id=delegatedcontact_id)
    invite, invite_created = Invite.objects.get_or_create(
        contact=delegated_contact.invited,
        event=delegated_contact.invite.event,
        defaults={'status': settings.PUBLIEDITOR_EVENT_INVITE_PENDING}
    )
    if invite_created:
        messages.info(request, _('Invite created'))
    else:
        messages.info(request, _('Invite already exists'))
    url = reverse("admin:event_invite_change", args=[delegated_contact.invite.id])
    return redirect(url)


def delegatedcontact_send_invite(request, delegatedcontact_id):
    """
    Createse and sends an invite for a delegated contact
    :param request: Current request
    :param delegatedcontact_id:  Id of delegated contact, which was already created
    :return: Redirect to new invite changelist
    """
    from publieditor.event.models import DelegatedContact, Invite
    delegated_contact = DelegatedContact.objects.get(id=delegatedcontact_id)
    invite, invite_created = Invite.objects.get_or_create(
        contact=delegated_contact.invited,
        event=delegated_contact.invite.event,
        defaults={'status': settings.PUBLIEDITOR_EVENT_INVITE_PENDING}
    )
    result, msg = invite.send()
    if result:
        delegated_contact.managed = True
        delegated_contact.save()
        messages.info(request, _('Invite sent'))
    else:
        messages.info(request, _('Invite not sent'))
    url = reverse("admin:event_invite_change", args=[delegated_contact.invite.id])
    return redirect(url)


def event_statistics(request, event_id, status, year, month, day):
    """
    This view is used to create real time event statistics.
    :param request: Current request
    :param event_id: Id of desired event
    :param status: Optional status used to filter invite emails
    :param year: Optional year used to filter invite emails
    :param month: Optional month used to filter invite emails
    :param day: Optional day used to filter invite emails
    :return: Redirects to view page
    """
    event = get_object_or_404(Event, id=event_id)

    send_date_string = '%s-%s-%s' % (year, month, day)
    send_date = None

    status_name = ''

    # Ordino gli stati
    status_order = [
        settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND_IN_TEST,
        settings.MAILER_MAILINGSTATUS_STATUS_SENT_TEST,
        settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND,
        settings.MAILER_MAILINGSTATUS_STATUS_SENT,
        settings.MAILER_MAILINGSTATUS_STATUS_BOUNCED,
        # settings.MAILER_MAILINGSTATUS_STATUS_ERROR,
        # settings.MAILER_MAILINGSTATUS_STATUS_INVALID,
        settings.MAILER_MAILINGSTATUS_STATUS_OPENED,
        settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE,
        settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED,
    ]

    # Filtri
    mailer_status = []
    for status_position in status_order:
        status_settings = _get_status(status_position)
        mailer_status.append([status_settings[0], status_settings[1]])

    # lista delle date
    send_dates = EventStatusDateStatistic.objects.filter(
            event_id=int(event_id)
        ).dates('date_invite', 'day')

    link_totals = {}
    totals = []
    link_rows = None
    if send_date_string == '9999-99-99':
        # filtra per stato
        if status != '99':
            mailer_status = [_get_status(status)]
            status_name = mailer_status[0][1]
            stat_rows = EventStatusStatistic.objects.filter(event_id=int(event_id), status=mailer_status[0][0])
            if status == str(settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED):
                link_rows = EventLinkStatistic.objects.filter(event_id=int(event_id))
        else:
            stat_rows = EventStatusStatistic.objects.filter(event_id=int(event_id))
    else:
        send_date = datetime.datetime.strptime(send_date_string, '%Y-%m-%d').date()
        if status != '99':
            mailer_status = [_get_status(status)]
            status_name = mailer_status[0][1]
            stat_rows = EventStatusDateStatistic.objects.filter(
                event_id=int(event_id),
                status=mailer_status[0][0],
                date_invite=datetime.datetime.strptime(send_date_string, "%Y-%m-%d")
            )
            if status == str(settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED):
                link_rows = EventLinkDateStatistic.objects.filter(
                    event_id=int(event_id),
                    date_invite=datetime.datetime.strptime(send_date_string, "%Y-%m-%d")
                )
        else:
            stat_rows = EventStatusDateStatistic.objects.filter(
                event_id=int(event_id),
                date_invite=datetime.datetime.strptime(send_date_string, "%Y-%m-%d")
            )

    graph_data = []
    for status_settings in mailer_status:
        for row in stat_rows:
            if row.status == status_settings[0]:
                # table data
                status_total_dict = dict(
                    status=status_settings[1],
                    totals=row.total,
                    uniques=row.unique_email_address
                )
                totals.append(status_total_dict)

                # graph data
                graph_data_dict = {}
                graph_data_dict[status_settings[1]] = {}
                graph_data_dict[status_settings[1]]['data'] = []
                graph_data_dict[status_settings[1]]['data'].append(
                    {
                        'y': row.unique_email_address,
                    })
                graph_data_dict[status_settings[1]]['data'].append(
                    {
                        'y': row.total - row.unique_email_address,
                    })
                graph_data_dict[status_settings[1]]['data'].append(
                    {
                        'y': str(row.total),
                    })
                graph_data.append(graph_data_dict)

    if link_rows:
        for row in link_rows:
            link_totals[row.link_url] = {}
            link_totals[row.link_url]['title'] = row.link_title
            link_totals[row.link_url]['totals'] = row.total
            link_totals[row.link_url]['uniques'] = row.unique_email_address

    last_p = LastMailingStatusProcessed.objects.get()
    is_updating = last_p.is_script_running
    last_update = last_p.date_last_modify

    context = {
        'event_id': event_id,
        'status': status,
        'event': event,
        'mailer_status': mailer_status,
        'send_dates': send_dates,
        'send_date': send_date,
        'year': year,
        'month': month,
        'day': day,
        'status_name': status_name,
        'send_date_string': send_date_string,
        'totals': totals,
        'link_totals': link_totals,
        'is_updating': is_updating,
        'last_update': last_update,
        'graph_data': graph_data
    }
    return render_to_response(
        'admin/event/event/event_statistics.html',
        context, context_instance=RequestContext(request)
    )


# def event_statistics_update(request):
#     """
#         Update statistic data
#         """
#     #call_update_statistic_cmd.delay()
#     import ipdb; ipdb.set_trace()
#     #call_command('update_event_statistics')
#     return redirect(reverse("admin:event_event_changelist") + "?{}".format(request.GET.urlencode()))

def event_statistics_update(request):
    """
    Send invites to delegated contacts
    """
    from publieditor.event.tasks import EventUpdateStatisticsAsyncTask
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    EventUpdateStatisticsAsyncTask.delay_or_fail(
        app_label='event',
    )
    return redirect(reverse(
        "admin:event_statistics",
        kwargs={
            'event_id': int(request.GET['event_id']),
            'status': int(request.GET['status']),
            'year': int(request.GET['year']),
            'month': int(request.GET['month']),
            'day': int(request.GET['day'])
        }
    ))
event_statistics_update.short_description = _('Statistics')


@login_required
def event_landing_page_presentation_preview(request, event_id):
    """
    View used to preview the web landing page of an event if the first step is presentation
    :param request: Current request
    :param event_id: Desired event
    :return: Redirect to view page
    """
    try:
        event = Event.objects.get(id=event_id)
    except Exception, ex:
        messages.info(request, _('Event does not exists'))
        url = reverse("admin:event_event_changelist")
        return redirect(url)
    fake_contact = Contact(
        first_name="Nome",
        last_name="Cognome",
        email="nome.cognome@provaemail.it",
        language=Language.objects.get(code="en"),
        title='mr',
    )
    presentation_translation = event.get_step_translation(
        "translations",
        user=request.user, is_preview=True,
    )
    rendered_body = None
    if presentation_translation:
        body = presentation_translation.web_body
        body_context = Context({"contact": fake_contact})
        template = loader.get_template_from_string(body)
        rendered_body = template.render(body_context)
    schedule_url = None
    if event.show_schedule:
        schedule_url = reverse("admin:show_event_schedule_preview", args=[event.id])
    context = dict(
        header_image=presentation_translation.header_image,
        site_protocol=settings.SITE_PROTOCOL,
        site_domain=settings.SITE_DOMAIN,
        event=event,
        preview_step="presentation",
        presentation_translation=presentation_translation,
        external_images=presentation_translation.get_bmw_external_images(),
        web_body=rendered_body,
        schedule_url=schedule_url,
        required_step=None,
        breadcrumbs=None,
        button_text="Aderisci online" if not event.is_privacy else "Registrati",
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_PREVIEW_PAGE_TEMPLATE_CHOICES[event.landing_page_template],
        context,
        context_instance=RequestContext(request)
    )


@login_required
def event_landing_page_schedule_preview(request, event_id):
    """
    View used to preview the web landing page of an event if the first step is schedule
    :param request: Current request
    :param event_id: Desired event
    :return: Redirect to view page
    """
    try:
        event = Event.objects.get(id=event_id)
    except Exception, ex:
        messages.info(request, _('Event does not exists'))
        url = reverse("admin:event_event_changelist")
        return redirect(url)
    fake_contact = Contact(
        first_name="Nome",
        last_name="Cognome",
        email="nome.cognome@provaemail.it",
        language=Language.objects.get(code="en"),
        title='mr',
    )
    presentation_translation = event.get_step_translation(
        "translations",
        user=request.user, is_preview=True,
    )
    schedule_translation = event.get_step_translation(
        "schedule_translations",
        user=request.user, is_preview=True,
    )
    rendered_body = None
    if schedule_translation:
        body = schedule_translation.body
        body_context = Context({"contact": fake_contact})
        template = loader.get_template_from_string(body)
        rendered_body = template.render(body_context)
    presentation_url = None
    if event.show_presentation:
        presentation_url = reverse("admin:show_event_presentation_preview", args=[event.id])
    context = dict(
        event=event,
        preview_step="schedule",
        presentation_translation=presentation_translation,
        external_images=presentation_translation.get_bmw_external_images(),
        presentation_url=presentation_url,
        schedule_translation=schedule_translation,
        web_body=rendered_body,
        required_step=None,
        breadcrumbs=None,
        button_text="Aderisci online" if not event.is_privacy else "Registrati",
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_PREVIEW_PAGE_TEMPLATE_CHOICES[event.landing_page_template],
        context,
        context_instance=RequestContext(request)
    )


@login_required
def event_landing_page_adhesion_preview(request, event_id):
    """
    View used to preview the web landing page of an event if the first step is adhesion
    :param request: Current request
    :param event_id: Desired event
    :return: Redirect to view page
    """
    try:
        event = Event.objects.get(id=event_id)
    except Exception, ex:
        messages.info(request, _('Event does not exists'))
        url = reverse("admin:event_event_changelist")
        return redirect(url)
    fake_contact = Contact(
        first_name="Nome",
        last_name="Cognome",
        email="nome.cognome@provaemail.it",
        language=Language.objects.get(code="en"),
        title='mr',
    )
    presentation_translation = event.get_step_translation(
        "translations",
        user=request.user, is_preview=True,
    )
    schedule_translation = event.get_step_translation(
        "schedule_translations",
        user=request.user, is_preview=True,
    )
    adhesion_translation = event.get_step_translation(
        "adhesion_translations",
        user=request.user, is_preview=True,
    )
    rendered_body = None
    if adhesion_translation:
        body = adhesion_translation.body
        body_context = Context({"contact": fake_contact})
        template = loader.get_template_from_string(body)
        rendered_body = template.render(body_context)
    presentation_url = None
    if event.show_presentation:
        presentation_url = reverse("admin:show_event_presentation_preview", args=[event.id])
    fake_invite = Invite.objects.filter(event__enable_delegate=event.enable_delegate)[0]
    form = AdhesionForm(invite=fake_invite)
    context = dict(
        event=event,
        preview_step="adhesion",
        presentation_translation=presentation_translation,
        external_images=presentation_translation.get_bmw_external_images(),
        presentation_url=presentation_url,
        schedule_translation=schedule_translation,
        web_body=rendered_body,
        required_step="adhesion",
        breadcrumbs=None,
        form=form,
        button_text="Aderisci online" if not event.is_privacy else "Registrati",
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_PREVIEW_PAGE_TEMPLATE_CHOICES[event.landing_page_template],
        context,
        context_instance=RequestContext(request)
    )


def event_email_invite_preview(request, event_id):
    """
    View used to preview the email of an event
    :param request: Current request
    :param event_id: Desired event
    :return: Redirect to view page
    """
    try:
        event = Event.objects.get(id=event_id)
    except Exception, ex:
        messages.info(request, _('Event does not exists'))
        url = reverse("admin:event_event_changelist")
        return redirect(url)

    fake_contact = Contact(
        first_name="Nome",
        last_name="Cognome",
        email="nome.cognome@provaemail.it",
        language=Language.objects.get(code="en"),
        title='mr',
    )

    translation = None
    try:
        translation = getattr(event, "translations").get(
            status=1, language=event.default_language
        )
    except ObjectDoesNotExist:
        if getattr(event, "translations").count():
            translation = getattr(event, "translations").all()[0]

    rendered_body = None
    if translation:
        body = translation.email_body
        body_context = Context({"contact": fake_contact})
        template = loader.get_template_from_string(body)
        rendered_body = template.render(body_context)

    context = dict(
        body=rendered_body,
        translation=translation,
        left_signature=translation.left_signature,
        right_signature=translation.right_signature,
        site_protocol=settings.SITE_PROTOCOL,
        site_domain=settings.SITE_DOMAIN,
        url_landing_page="#",
        button_text="Aderisci online" if not event.is_privacy else "Registrati",
        header_image=translation.header_image,
        footer_image=translation.footer_image,
        external_images=translation.get_bmw_external_images(),
        event=event,
        preview=True
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_INVITE_TEMPLATES[event.landing_page_template],
        context,
        context_instance=RequestContext(request)
    )


def show_txt_event_invite(request, event_id):
    """
    View used to preview the text landing page of an event.
    This view should not be already used but please check before deleting
    :param request: Current request
    :param event_id: Desired event
    :return: Redirect to view page
    """
    try:
        event = Event.objects.get(id=event_id)
    except Exception, ex:
        messages.info(request, _('Event does not exists'))
        url = reverse("admin:event_event_changelist")
        return redirect(url)

    translation = None
    if event:
        try:
            translation = getattr(event, "translations").get(
                status=1, language=event.default_language
            )
        except ObjectDoesNotExist:
            if getattr(event, "translations").count():
                translation = getattr(event, "translations").all()[0]

    body = None
    if translation:
        body = translation.web_body

    context = dict(
        body=body,
        translation=translation,
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_TXT_PREVIEW_PAGE_TEMPLATE,
        context,
        context_instance=RequestContext(request)
    )


@login_required
def invite_set_delegated(request, invite_id):
    """
    View used to delegate a contact from an invite from admin
    :param request: Current request
    :param invite_id: Desired invite id
    :return: View page
    """

    invite = get_object_or_404(Invite, pk=invite_id)

    disabled_st = [
        settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
        settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
        settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
    ]
    if invite.status in disabled_st:
        msg = _('Operation not permitted')
        messages.error(request, msg)
        return redirect(reverse("admin:event_invite_changelist")+"?event={0}".format(invite.event.id))

    opts = Invite._meta
    app_label = opts.app_label
    has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())
    template = 'admin/{0}/{1}/set_delegated.html'.format(app_label, opts.object_name.lower())

    if request.method == 'POST':
        form = InviteSetDelegatedAdminForm(request.POST, invite_id=invite_id)
        if form.is_valid():
            form.save()
            msg = _('Delegated contact correctly saved')
            messages.success(request, msg)
            return redirect(reverse("admin:event_invite_changelist")+"?event={0}".format(invite.event.id))
    else:
        form = InviteSetDelegatedAdminForm(invite_id=invite_id)

    context = {
        'invite': invite,
        'opts': opts,
        'form': form,
        'app_label': opts.app_label,
        'has_change_permission': has_perm
    }

    return render_to_response(
        template,
        context, context_instance=RequestContext(request)
    )


# TRAVEL AGENCY
class EventTravelAgencyMixin(object):
    """
    Mixin used in travel agency views
    """
    groups = [
        {
            'name': ugettext("Confirmed"),
            'status': [
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
            ]
        },
        {
            'name': ugettext("To be confirmed"),
            'status': [
                settings.PUBLIEDITOR_EVENT_INVITE_PENDING,
                settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND,
                settings.PUBLIEDITOR_EVENT_INVITE_SENT
            ]
        },
        {
            'name': ugettext("Declined"),
            'status': [settings.PUBLIEDITOR_EVENT_INVITE_DECLINED]
        },
        {
            'name': ugettext("Delegated"),
            'status': [settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED]
        }
    ]

    def get_elements_by_turn(self, event, turns):
        elements_by_turn = []
        done_ids = []
        for turn in turns:
            turn_elements = {'turn': turn, 'elements': []}
            for group in self.groups:
                group_name, status = group['name'], group['status']
                invites = event.invites.filter(turn_preference=turn, status__in=status).order_by(
                    'contact__last_name'
                )
                group_invites_count = invites.count()
                if group_invites_count:
                    turn_group_elements = {'name': group_name, 'count': group_invites_count, 'invites': []}
                    for invite in invites:
                        if invite.id not in done_ids:
                            activity_answer_queryset = invite.activity_answers.all()
                            activity_list = [
                                activity.question.get_short_name() + ': ' + activity.answer for activity in activity_answer_queryset
                            ]
                            activities = '<br><br>'.join(activity_list)
                            newspapers = ", ".join([m.newspaper.name for m in invite.contact.memberships.all()])
                            last_name, first_name = invite.contact.last_name, invite.contact.first_name
                            phone = invite.contact.default_phone.full_number if invite.contact.default_phone else ''
                            note = u"{}".format(invite.note) if invite.note else u""
                            internal_note = u"{}".format(invite.internal_note) if invite.internal_note else u""
                            food = u"{}".format(invite.contact.get_food_display())
                            food_note = u"{}".format(invite.contact.food_note) if invite.contact.food_note else u""
                            cards = u",<br>".join([u'{}-{}'.format(card.airline.iata_code, card.number) if card.airline else u'{}'.format(card.number) for card in invite.contact.cards.all()])
                            departures = invite.get_departure_details()
                            returns = invite.get_return_details()
                            invite_dict = dict(
                                newspapers=newspapers,
                                last_name=last_name,
                                first_name=first_name,
                                phone=phone,
                                note=note,
                                food_note=food_note,
                                internal_note=internal_note,
                                food=food,
                                cards=cards,
                                departures=departures,
                                returns=returns,
                                activities=activities,
                                instance=invite,
                            )
                            turn_group_elements['invites'].append(invite_dict)
                            done_ids.append(invite.id)
                    turn_elements['elements'].append(turn_group_elements)
            elements_by_turn.append(turn_elements)
        return elements_by_turn


class EventTravelAgencyView(TemplateView, EventTravelAgencyMixin):
    """
    Recap view used to monitor event adhesions for a travel agency
    """
    template_name = "admin/event/event/travel_agency.html"

    def __init__(self, **kwargs):
        self.event_id = None
        super(EventTravelAgencyView, self).__init__(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        self.event_id = kwargs.get('event_id', None)
        event = self.get_object()
        if not event:
            messages.error(request, _('Forbidden'))
            return HttpResponseRedirect('/')
        return super(EventTravelAgencyView, self).dispatch(request, *args, **kwargs)

    def get_object(self):
        event = None
        if self.event_id:
            try:
                event = Event.objects.get(
                    pk=self.event_id,
                )
                if not self.request.user.is_superuser:
                    if self.request.user not in event.travel_agencies.all():
                        event = None
            except Event.DoesNotExist as e:
                event = None
        return event

    def get_context_data(self, **kwargs):
        context = super(EventTravelAgencyView, self).get_context_data(**kwargs)
        try:
            event = Event.objects.get(id=kwargs.get('event_id'))
            turns = event.turns.all().order_by('turn_start', 'turn_end')
            elements_by_turn = self.get_elements_by_turn(event, turns)
            context.update({
                'event': event,
                'turns': turns,
                'elements_by_turn': elements_by_turn,
            })
        except Event.DoesNotExist:
            return redirect(reverse('admin:event_event_changelist'))
        return context


class EventPollAnswersView(TemplateView, EventTravelAgencyMixin):
    """
    View used to display event invites answers for a travel agency
    """
    template_name = "admin/event/event/poll_answers.html"

    def get_context_data(self, **kwargs):
        from publieditor.event.models import PollAnswer
        context = super(EventPollAnswersView, self).get_context_data(**kwargs)
        try:
            questions_groups = []
            event = Event.objects.get(id=kwargs.get('event_id'))
            for event_question in event.questions.filter(question__typology='P').order_by('ordering'):
                event_question_answers = []
                eq_answers = PollAnswer.objects.filter(invite__event=event, question=event_question.question)
                eq_answers_count = eq_answers.count()
                for answer in eq_answers.values_list('answer', flat=True).distinct():
                    answers_count = eq_answers.filter(answer=answer).count()
                    event_question_answers.append({
                        'answer': answer,
                        'this_answer_count': answers_count,
                        'total_answers_count': eq_answers_count,
                        'this_answer_percentage': "{0:.0f}%".format(float(answers_count)/eq_answers_count * 100),
                    })
                if not len(event_question_answers):
                    event_question_answers.append({
                        'answer': ugettext("No answers"),
                        'this_answer_count': "",
                        'total_answers_count': "",
                        'this_answer_percentage': "",
                    })
                questions_groups.append({
                    'question': event_question.question,
                    'answers': event_question_answers
                })
            context.update({
                'event': event,
                'questions_groups': questions_groups,
            })
        except Event.DoesNotExist:
            return redirect(reverse('admin:event_event_changelist'))
        return context


class EventTravelAgencyExportView(View, EventTravelAgencyMixin):
    """
    Excel view used to create an excel file containing event partecipations details
    """
    def get(self, request, **kwargs):
        font = xlwt.Font()
        font.bold = True
        green_style = xlwt.easyxf('pattern: pattern solid, fore_colour light_green;')
        gray_style = xlwt.easyxf('pattern: pattern solid, fore_colour gray25;')
        violet_style = xlwt.easyxf('pattern: pattern solid, fore_colour rose;')
        green_style.font = font
        gray_style.font = font
        violet_style.font = font
        event = Event.objects.get(id=kwargs.get('event_id'))
        turns = event.turns.all().order_by('turn_start', 'turn_end')
        elements_by_turn = self.get_elements_by_turn(event, turns)
        wb = xlwt.Workbook()
        ws = wb.add_sheet('Travel Agency')
        rowPos, colPos = 0, 0
        ws.write_merge(rowPos, rowPos, colPos, 9, event.name, style=green_style)
        for turn_elements in elements_by_turn:
            rowPos += 1
            ws.write_merge(rowPos, rowPos, colPos, 9, turn_elements['turn'].title, style=green_style)
            rowPos += 1
            ws.write(rowPos, colPos, ugettext('Presenze'), style=gray_style)
            ws.write(rowPos, colPos+1, ugettext('Testate'), style=gray_style)
            ws.write(rowPos, colPos+2, ugettext('Cognome'), style=gray_style)
            ws.write(rowPos, colPos+3, ugettext('Nome'), style=gray_style)
            ws.write(rowPos, colPos+4, ugettext('Phone'), style=gray_style)
            ws.write(rowPos, colPos+5, ugettext('Note'), style=gray_style)
            ws.write(rowPos, colPos+6, ugettext('Intolleranze'), style=gray_style)
            ws.write(rowPos, colPos+7, ugettext('Frequent Flyer'), style=gray_style)
            ws.write(rowPos, colPos+8, ugettext('Andata'), style=gray_style)
            ws.write(rowPos, colPos+9, ugettext('Ritorno'), style=gray_style)
            for turn_element in turn_elements['elements']:
                rowPos += 1
                ws.write(rowPos, colPos, turn_element['count'], style=violet_style)
                ws.write_merge(rowPos, rowPos, colPos+1, 9, turn_element['name'], style=violet_style)
                for invite in turn_element['invites']:
                    food_note = u"{}".format(invite['food']) if not invite['food_note'] else u"{}, Note: {}".format(
                        invite['food'], invite['food_note']
                    )
                    rowPos += 1
                    style = xlwt.XFStyle()
                    style.alignment.wrap = 1
                    departures_length = len(invite['departures'].split("\n"))
                    returns_length = len(invite['returns'].split("\n"))
                    max_length = max(departures_length, returns_length)
                    if max_length > 1:
                        ws.row(rowPos).height_mismatch = True
                        ws.row(rowPos).height = max_length*280
                    ws.write(rowPos, colPos, turn_element['name'])
                    ws.write(rowPos, colPos+1, invite['newspapers'])
                    ws.write(rowPos, colPos+2, u"{}".format(invite['last_name']))
                    ws.write(rowPos, colPos+3, u"{}".format(invite['first_name']))
                    ws.write(rowPos, colPos+4, u"{}".format(invite['phone']))
                    ws.write(rowPos, colPos+5, u"{}".format(invite['internal_note']))
                    ws.write(rowPos, colPos+6, u"{}".format(food_note if not food_note == 'None' else u''))
                    ws.write(rowPos, colPos+7, u"{}".format(invite['cards']))
                    ws.write(rowPos, colPos+8, invite['departures'], style)
                    ws.write(rowPos, colPos+9, invite['returns'], style)
        ws.col(0).width = 2500
        ws.col(1).width = 15000
        ws.col(2).width = 3000
        ws.col(3).width = 3000
        ws.col(4).width = 5000
        ws.col(5).width = 3000
        ws.col(6).width = 3000
        ws.col(7).width = 10000
        ws.col(8).width = 20000
        ws.col(9).width = 20000
        response = HttpResponse(mimetype='application/vnd.ms-excel')
        response['Content-Disposition'] = 'attachment; filename={}.xls'.format(slugify(event.name))
        wb.save(response)
        return response

    def join_transports(self, transports):
        return u"\n".join(transports)


# LOGISTIC AGENCY
class EventLogisticAgencyMixin(object):
    """
    Mixin used in travel agency views
    """
    groups = [
        {
            'name': ugettext("Confirmed"),
            'status': [
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
            ]
        },
        {
            'name': ugettext("To be confirmed"),
            'status': [
                settings.PUBLIEDITOR_EVENT_INVITE_PENDING,
                settings.PUBLIEDITOR_EVENT_INVITE_TO_SEND,
                settings.PUBLIEDITOR_EVENT_INVITE_SENT
            ]
        },
        {
            'name': ugettext("Declined"),
            'status': [settings.PUBLIEDITOR_EVENT_INVITE_DECLINED]
        },
        {
            'name': ugettext("Delegated"),
            'status': [settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED]
        }
    ]

    def get_turn_elements(self, turn, event, done_ids):
        turn_elements = {'turn': turn, 'elements': []}
        for group in self.groups:
            group_name, status = group['name'], group['status']

            if turn:
                # invites = event.invites.filter(turn_preference=turn, status__in=status).order_by(
                #     'contact__last_name'
                # )
                invites = Invite.objects.select_related(
                    'contact', 'event',
                ).prefetch_related(
                  'contact__phones'
                ).filter(event_id=event.id, turn_preference=turn, status__in=status).order_by(
                    'event_place__date', 'contact__last_name'
                )
            else:
                # invites = event.invites.filter(turn_preference__isnull=True, status__in=status).order_by(
                #     'contact__last_name'
                # )
                invites = Invite.objects.select_related(
                    'contact', 'event',
                ).prefetch_related(
                  'contact__phones'
                ).filter(event_id=event.id, turn_preference__isnull=True, status__in=status).order_by(
                    'event_place__date', 'contact__last_name'
                )
            group_invites_count = invites.count()
            if group_invites_count:
                turn_group_elements = {'name': group_name, 'count': group_invites_count, 'invites': []}
                for invite in invites:
                    if invite.id not in done_ids:
                        activities = invite.get_activities_cl_short_name_list()
                        # newspapers = ", ".join([m.newspaper.name for m in invite.contact.memberships.all()])
                        newspapers = ", ".join(invite.contact.memberships.all().values_list('newspaper__name', flat=True))
                        last_name, first_name = invite.contact.last_name, invite.contact.first_name
                        # email = invite.contact.email
                        phone = invite.contact.default_phone.full_number if invite.contact.default_phone else ''
                        note = u"{}".format(invite.note) if invite.note else u""
                        internal_note = u"{}".format(invite.internal_note) if invite.internal_note else u""
                        food = u"{}".format(invite.contact.get_food_display())
                        food_note = u"{}".format(invite.contact.food_note) if invite.contact.food_note else u""
                        # cards = u", ".join([card.number for card in invite.contact.cards.all()])
                        cards = u", ".join(invite.contact.cards.all().values_list('number', flat=True))
                        departures = invite.get_departure_details()
                        returns = invite.get_return_details()

                        invite_dict = dict(
                            newspapers=newspapers,
                            last_name=last_name,
                            first_name=first_name,
                            phone=phone,
                            note=note,
                            food_note=food_note,
                            internal_note=internal_note,
                            food=food,
                            cards=cards,
                            departures=departures,
                            returns=returns,
                            activities=activities,
                            instance=invite,
                        )
                        turn_group_elements['invites'].append(invite_dict)
                        done_ids.append(invite.id)
                turn_elements['elements'].append(turn_group_elements)
        return turn_elements, done_ids

    def get_elements_by_turn(self, event, turns):
        elements_by_turn = []
        done_ids = []
        for turn in turns:
            turn_elements, done_ids = self.get_turn_elements(turn, event, done_ids)
            elements_by_turn.append(turn_elements)

        turn_elements, done_ids = self.get_turn_elements(None, event, done_ids)
        elements_by_turn.append(turn_elements)
        return elements_by_turn


class EventLogisticAgencyView(TemplateView, EventLogisticAgencyMixin):
    """
    Recap view used to monitor event adhesions for a logistic agency
    """

    def get_template_names(self):
        if self.get_object().slug == u'triumph-tour-2018':
            return "admin/event/event/logistic_agency_triumph.html"
        return "admin/event/event/logistic_agency.html"

    def __init__(self, **kwargs):
        self.event_id = None
        super(EventLogisticAgencyView, self).__init__(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        self.event_id = kwargs.get('event_id', None)
        event = self.get_object()
        if not event:
            messages.error(request, _('Forbidden'))
            return HttpResponseRedirect('/')
        return super(EventLogisticAgencyView, self).dispatch(request, *args, **kwargs)

    def get_object(self):
        event = None
        if self.event_id:
            try:
                event = Event.objects.get(
                    pk=self.event_id,
                )
                if not self.request.user.is_superuser:
                    if self.request.user not in event.logistic_agencies.all():
                        event = None
            except Event.DoesNotExist as e:
                event = None
        return event

    def get_context_data(self, **kwargs):
        context = super(EventLogisticAgencyView, self).get_context_data(**kwargs)
        try:
            event = Event.objects.get(id=kwargs.get('event_id'))
            turns = event.turns.all().order_by('turn_start', 'turn_end')
            elements_by_turn = self.get_elements_by_turn(event, turns)
            context.update({
                'event': event,
                'turns': turns,
                'elements_by_turn': elements_by_turn,
                'event_questions': event.questions.filter(question__typology="A").order_by('ordering')
            })
        except Event.DoesNotExist:
            return redirect(reverse('admin:event_event_changelist'))
        return context


class EventHostessView(TemplateView, EventLogisticAgencyMixin):
    pass


class EventLogisticAgencyExportView(View, EventLogisticAgencyMixin):
    """
    Excel view used to create an excel file containing event partecipations details
    """
    def get(self, request, **kwargs):
        font = xlwt.Font()
        font.bold = True
        green_style = xlwt.easyxf('pattern: pattern solid, fore_colour light_green;')
        gray_style = xlwt.easyxf('pattern: pattern solid, fore_colour gray25;')
        violet_style = xlwt.easyxf('pattern: pattern solid, fore_colour rose;')
        green_style.font = font
        gray_style.font = font
        violet_style.font = font
        event = Event.objects.get(id=kwargs.get('event_id'))
        questions = event.questions.all().order_by('ordering')
        turns = event.turns.all().order_by('turn_start', 'turn_end')
        elements_by_turn = self.get_elements_by_turn(event, turns)
        wb = xlwt.Workbook()
        ws = wb.add_sheet('Travel Agency')
        rowPos, colPos = 0, 0
        columns_count = + 9 + questions.all().count()
        ws.write_merge(rowPos, rowPos, colPos, columns_count, event.name, style=green_style)
        for turn_elements in elements_by_turn:
            colPos = 0
            if turn_elements['turn']:
                turn_title = turn_elements['turn'].title
            else:
                turn_title = 'without a turn'
            
            rowPos += 1
            ws.write_merge(rowPos, rowPos, colPos, columns_count, turn_title, style=green_style)
            rowPos += 1
            ws.write(rowPos, colPos, ugettext(u'Presenze'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Testate'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Cognome'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Nome'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Phone'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Delegato'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Note'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Note Interne'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Intolleranze'), style=gray_style)

            # Question column header
            for question in questions:
                colPos += 1
                ws.write(rowPos, colPos, ugettext(question.get_short_name()), style=gray_style)

            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Andata'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(u'Ritorno'), style=gray_style)
            for turn_element in turn_elements['elements']:
                colPos = 0
                rowPos += 1
                ws.write(rowPos, colPos, turn_element['count'], style=violet_style)
                ws.write_merge(rowPos, rowPos, colPos+1, columns_count, turn_element['name'], style=violet_style)
                for invite in turn_element['invites']:
                    colPos = 0
                    food_note = u"{}".format(invite['food']) if not invite['food_note'] else u"{}, Note: {}".format(
                        invite['food'], invite['food_note']
                    )
                    rowPos += 1
                    style = xlwt.XFStyle()
                    style.alignment.wrap = 1
                    departures_length = len(invite['departures'].split("\n"))
                    returns_length = len(invite['returns'].split("\n"))
                    max_length = max(departures_length, returns_length)
                    if max_length > 1:
                        ws.row(rowPos).height_mismatch = True
                        ws.row(rowPos).height = max_length*280
                    ws.write(rowPos, colPos, turn_element['name'])
                    colPos += 1
                    ws.write(rowPos, colPos, invite['newspapers'])
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(invite['last_name']))
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(invite['first_name']))
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(invite['phone']))
                    colPos += 1
                    if invite['instance'].status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED and invite['instance'].delegated_invites.all():
                        ws.write(rowPos, colPos, u"{}".format(invite['instance'].delegated_invites.all()[0].invited))
                    else:
                        ws.write(rowPos, colPos, u"")
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(invite['note']))
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(invite['internal_note']))
                    colPos += 1
                    ws.write(rowPos, colPos, u"{}".format(food_note if not food_note == 'None' else u''))

                    for question in questions:
                        colPos += 1
                        try:
                            event_question = invite['instance'].activity_answers.get(question__event_questions=question)
                            ws.write(rowPos, colPos, u"{question_answer}".format(
                                question_answer=event_question.answer,
                            ))
                        except ObjectDoesNotExist as e:
                            pass
                        except MultipleObjectsReturned as e:
                            pass

                    colPos += 1
                    ws.write(rowPos, colPos, invite['departures'], style)
                    colPos += 1
                    ws.write(rowPos, colPos, invite['returns'], style)


        colPos = 0
        # Presenze
        ws.col(colPos).width = 5000

        # Testate
        colPos += 1
        ws.col(colPos).width = 10000

        # Cognome
        colPos += 1
        ws.col(colPos).width = 5000

        # Nome
        colPos += 1
        ws.col(colPos).width = 5000

        # Telefono
        colPos += 1
        ws.col(colPos).width = 7000

        # Delegato
        colPos += 1
        ws.col(colPos).width = 10000

        # Note
        colPos += 1
        ws.col(colPos).width = 5000

        # Intolleranze
        colPos += 1
        ws.col(colPos).width = 5000

        # ATTIVITÀ
        for question in questions:
            colPos += 1
            ws.col(colPos).width = 2500

        # Andata
        colPos += 1
        ws.col(colPos).width = 5000

        # Ritorno
        colPos += 1
        ws.col(colPos).width = 5000

        response = HttpResponse(mimetype='application/vnd.ms-excel')
        response['Content-Disposition'] = 'attachment; filename={}.xls'.format(slugify(event.name))
        wb.save(response)
        return response

    def join_transports(self, transports):
        return u"\n".join(transports)


class EventLogisticAgencyTriumphExportView(View, EventLogisticAgencyMixin):
    """
    Excel view used to create an excel file containing event partecipations details
    """
    def get(self, request, **kwargs):
        font = xlwt.Font()
        font.bold = True
        green_style = xlwt.easyxf('pattern: pattern solid, fore_colour light_green;')
        gray_style = xlwt.easyxf('pattern: pattern solid, fore_colour gray25;')
        violet_style = xlwt.easyxf('pattern: pattern solid, fore_colour rose;')
        green_style.font = font
        gray_style.font = font
        violet_style.font = font
        event = Event.objects.get(id=kwargs.get('event_id'))
        questions = event.questions.all().order_by('ordering')
        turns = event.turns.all().order_by('turn_start', 'turn_end')
        elements_by_turn = self.get_elements_by_turn(event, turns)
        wb = xlwt.Workbook()
        ws = wb.add_sheet('Travel Agency')
        rowPos, colPos = 0, 0
        columns_count = 15
        ws.write_merge(rowPos, rowPos, colPos, columns_count, event.name, style=green_style)
        for turn_elements in elements_by_turn:
            colPos = 0
            # if turn_elements['turn']:
            #     turn_title = turn_elements['turn'].title
            # else:
            #     turn_title = 'without a turn'
            # rowPos += 1
            # ws.write_merge(rowPos, rowPos, colPos, columns_count, turn_title, style=green_style)

            rowPos += 1
            ws.write(rowPos, colPos, ugettext(_('Registration Date')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('First Name')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Last Name')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Email')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Mobile')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Reservation Code')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Event Place')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Favourite Vehicle Model')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Home Address')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('Birth Info')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('License Number')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('License Types')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('License Emitted By')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('License Issue Date')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext(_('License Expiration Date')), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext('CDU'), style=gray_style)
            colPos += 1
            ws.write(rowPos, colPos, ugettext('Call Result'), style=gray_style)

            for turn_element in turn_elements['elements']:
                for invite in turn_element['invites']:
                    colPos = 0
                    food_note = u"{}".format(invite['food']) if not invite['food_note'] else u"{}, Note: {}".format(
                        invite['food'], invite['food_note']
                    )
                    rowPos += 1
                    style = xlwt.XFStyle()
                    style.alignment.wrap = 1
                    departures_length = len(invite['departures'].split("\n"))
                    returns_length = len(invite['returns'].split("\n"))
                    max_length = max(departures_length, returns_length)
                    if max_length > 1:
                        ws.row(rowPos).height_mismatch = True
                        ws.row(rowPos).height = max_length*280
                    ws.write(rowPos, colPos, invite['instance'].date_create.strftime("%d/%m/%Y %H:%M"))
                    colPos += 1
                    ws.write(rowPos, colPos, invite['instance'].contact.first_name)
                    colPos += 1
                    ws.write(rowPos, colPos, invite['instance'].contact.last_name)
                    colPos += 1
                    ws.write(rowPos, colPos, invite['instance'].contact.email)
                    colPos += 1
                    try:
                        mobile_number = invite['instance'].contact.get_mobile_phone().number
                    except AttributeError:
                        mobile_number = ''
                    ws.write(rowPos, colPos, mobile_number)
                    colPos += 1
                    ws.write(rowPos, colPos, invite['instance'].make_base36())
                    colPos += 1
                    if invite['instance'].assigned_place:
                        ws.write(rowPos, colPos, unicode(invite['instance'].assigned_place))
                    elif invite['instance'].event_place:
                        ws.write(rowPos, colPos, unicode(invite['instance'].event_place))
                    else:
                        ws.write(rowPos, colPos, "-")
                    colPos += 1

                    if invite['instance'].assigned_vehicle:
                        ws.write(rowPos, colPos, unicode(invite['instance'].assigned_vehicle))
                    elif invite['instance'].favourite_vehicle_model:
                        ws.write(rowPos, colPos, unicode(invite['instance'].favourite_vehicle_model))
                    else:
                        ws.write(rowPos, colPos, "-")
                    colPos += 1

                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.get_home_address()))
                    colPos += 1
                    ws.write(rowPos, colPos, invite['instance'].contact.get_birth_info())
                    colPos += 1

                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.license_number) if invite['instance'].contact.license_number else '')
                    colPos += 1
                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.get_license_types_list()) if invite['instance'].contact.get_license_types_list() else '')
                    colPos += 1
                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.license_emitted_by) if invite['instance'].contact.license_emitted_by else '')
                    colPos += 1
                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.license_issue_date) if invite['instance'].contact.license_issue_date else '')
                    colPos += 1
                    ws.write(rowPos, colPos, unicode(invite['instance'].contact.license_expiration_date) if invite['instance'].contact.license_expiration_date else '')
                    colPos += 1
                    ws.write(rowPos, colPos, 'X') if hasattr(invite['instance'], 'cdu') else ''
                    colPos += 1
                    ws.write(rowPos, colPos, unicode(invite['instance'].get_call_status_display()))

        colPos = 0
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 10000

        colPos += 1
        ws.col(colPos).width = 7000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 20000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 10000

        colPos += 1
        ws.col(colPos).width = 7000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 5000

        colPos += 1
        ws.col(colPos).width = 2000

        colPos += 1
        ws.col(colPos).width = 8000

        response = HttpResponse(mimetype='application/vnd.ms-excel')
        response['Content-Disposition'] = 'attachment; filename={}.xls'.format(slugify(event.name))
        wb.save(response)
        return response

    def join_transports(self, transports):
        return u"\n".join(transports)


##################################################################################################
# Import views
##################################################################################################
class ContactImportView(ImportDataView):
    """
    Admin import view for contacts
    """
    template_name = 'admin/event/contact/import.html'
    import_class = ContactImportData
    form_class = ContactImportDataForm

    def get_context_data(self, **kwargs):
        from jmb.i18n.models import Language
        from jmb.l10n.models import Country
        context = super(ContactImportView, self).get_context_data(**kwargs)
        opts = Contact._meta
        context['opts'] = opts
        context['app_label'] = opts.app_label
        context['languages'] = ", ".join([language.code for language in Language.objects.all().order_by('name')])
        context['countries'] = ", ".join([country.iso2_code for country in Country.objects.all().order_by('name')])
        return context

    @property
    def redirect_url(self):
        return reverse('admin:event_contact_changelist')
