# -*- coding: utf-8 -*-

from itertools import chain
import autocomplete_light
from django.utils.translation import ugettext_lazy as _
from jmb.core.utils.autocomplete import DynamicAutocompleteModelBase

from publieditor.event.models import (
    Contact, Event, Invite,
    Flight, Airport, Airline, FlightDetail,
    Train, Station, TrainDetail, Turn,
    EventStartPlace, Stage, TransportDetail, Newspaper, TrainType, AreaCode,
)
from publieditor.survey.models import Question


class TurnAutocompleteDynamic(DynamicAutocompleteModelBase):
    model = Turn
    search_fields = ['title', 'code']
    autocomplete_js_attributes = {'placeholder': _('Turn')}
    choices = Turn.objects
    filters = {'event': '#id_event'}
    autocomplete_js_attributes = {'minimum_characters': 0}


class TrainAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['number', 'code']
    choices = Train.objects
    autocomplete_js_attributes = {'minimum_characters': 2}


class FlightAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['number', 'code']
    choices = Flight.objects
    autocomplete_js_attributes = {'minimum_characters': 2}


class AreaCodeAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['area', 'code']
    choices = AreaCode.objects
    autocomplete_js_attributes = {'minimum_characters': 2}


class EventStartPlaceAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = EventStartPlace.objects
    autocomplete_js_attributes = {'minimum_characters': '0'}


class InviteAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['event__name', 'contact__first_name', 'contact__last_name', 'contact__email']
    choices = Invite.objects


class TransportDetailAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['']
    choices = TransportDetail.objects

    def choices_for_request(self):
        q = self.request.GET.get("q")
        if q:
            trains = TrainDetail.objects.filter(train__number__icontains = q)
            filghts = FlightDetail.objects.filter(flight__number__icontains = q)
            return list(chain(trains, filghts))
        return super(TransportDetailAutocomplete, self).choices_for_request()


class StationAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = Station.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


class TrainTypeAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = TrainType.objects
    autocomplete_js_attributes = {'minimum_characters': '0'}


class StageAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = Stage.objects
    autocomplete_js_attributes = {'minimum_characters': '0'}


class AirportAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name', 'place', 'iata_code']
    choices = Airport.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


class AirlineAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name', 'iata_code']
    choices = Airline.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


class EventAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name', 'event_date_start']
    choices = Event.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


class EventTravelAgencyAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = Event.objects

    def choices_for_request(self):
        qs = super(EventTravelAgencyAutocomplete, self).choices_for_request()
        return qs.filter(travel_agencies=self.request.user)


class EventLogisticAgencyAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = Event.objects

    def choices_for_request(self):
        qs = super(EventLogisticAgencyAutocomplete, self).choices_for_request()
        return qs.filter(logistic_agencies=self.request.user)


class EventHostessAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
    choices = Event.objects

    def choices_for_request(self):
        qs = super(EventHostessAutocomplete, self).choices_for_request()
        return qs.filter(hostess=self.request.user)


class ContactAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['first_name', 'last_name', 'email']
    choices = Contact.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


class ContactTravelAgencyAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['first_name', 'last_name', 'email']
    choices = Contact.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}

    def choices_for_request(self):
        qs = Contact.objects.filter(invites__event__travel_agencies=self.request.user).distinct()
        return qs


class ContactLogisticAgencyAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['first_name', 'last_name', 'email']
    choices = Contact.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}

    def choices_for_request(self):
        qs = Contact.objects.filter(invites__event__logistic_agencies=self.request.user).distinct()
        return qs


class ContactHostessAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['first_name', 'last_name', 'email']
    choices = Contact.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}

    def choices_for_request(self):
        qs = Contact.objects.filter(invites__event__hostess=self.request.user).distinct()
        return qs


class NewspaperAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name', ]
    choices = Newspaper.objects


class QuestionAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['question']
    choices = Question.objects
    autocomplete_js_attributes = {'minimum_characters': '2'}


autocomplete_light.register(AreaCodeAutocomplete)
autocomplete_light.register(NewspaperAutocomplete, add_another_url_name='admin:event_newspaper_add')
autocomplete_light.register(ContactAutocomplete)
autocomplete_light.register(ContactTravelAgencyAutocomplete)
autocomplete_light.register(ContactLogisticAgencyAutocomplete)
autocomplete_light.register(ContactHostessAutocomplete)
autocomplete_light.register(EventAutocomplete)
autocomplete_light.register(EventTravelAgencyAutocomplete)
autocomplete_light.register(EventLogisticAgencyAutocomplete)
autocomplete_light.register(EventHostessAutocomplete)
autocomplete_light.register(AirlineAutocomplete, add_another_url_name='admin:event_airline_add')
autocomplete_light.register(AirportAutocomplete, add_another_url_name='admin:event_airport_add')
autocomplete_light.register(StageAutocomplete, add_another_url_name='admin:event_stage_add')
autocomplete_light.register(TrainTypeAutocomplete, add_another_url_name='admin:event_traintype_add')
autocomplete_light.register(StationAutocomplete, add_another_url_name='admin:event_station_add')
autocomplete_light.register(TransportDetailAutocomplete)
autocomplete_light.register(InviteAutocomplete)
autocomplete_light.register(EventStartPlaceAutocomplete, add_another_url_name='admin:event_eventstartplace_add')
autocomplete_light.register(FlightAutocomplete, add_another_url_name='admin:event_flight_add')
autocomplete_light.register(TrainAutocomplete, add_another_url_name='admin:event_train_add')
autocomplete_light.register(TurnAutocompleteDynamic)
autocomplete_light.register(QuestionAutocomplete)