# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.forms.models import inlineformset_factory
from django.http import HttpResponseRedirect
from django.utils.http import base36_to_int
from django.views.generic import FormView
from django.views.generic import UpdateView, View
from django.utils.translation import ugettext_lazy as _
from django.shortcuts import redirect

from publieditor.event.models import (
    Invite, Contact, Phone, ContactAddress, Card, InviteDecisionAnswer, ContactEditDiff,
    AreaCode)
from publieditor.event.forms import (
    FormsetPhoneForm, ReadonlyFormsetPhoneForm, ReadonlyFormsetCardForm,
    ReadonlyFormsetAddressForm, ContactAddressFormsetForm, CardFormsetForm,
    SetSessionByInviteForm)
from publieditor.event.public_forms import (
    PresentationForm, ScheduleForm, AdhesionForm, RegisterForm,
    TransportForm, ActivityForm, SummaryForm, DocumentsForm,
    DeclineForm, DelegateForm, GreetingsForm, ForwardToFormset
)
from publieditor.event.utils.my_invite import get_context_breadcrumbs


class InviteDispatchView(View):

    def dispatch(self, request, *args, **kwargs):
        enabled_steps = [
            "presentation", "schedule", "adhesion", "register", "transport", "activity",
            "summary", "documents", "decline", "delegate", "greetings", "forward_to"
        ]

        if 'publieditor_event_invite_id' not in request.session:
            return redirect(reverse("event:publieditor_event_session_expired"))

        # Get step and update invite if request is not POST
        invite = Invite.objects.get(pk=request.session['publieditor_event_invite_id'])

        step = kwargs.get("step", invite.step)

        if invite.event.is_deadline_passed() and not step == "greetings":  # greetings is used for poll
            return redirect(reverse(
                'event:publieditor_event_deadline_expired',
                kwargs={'invite_id': invite.id}
            ))

        if not request.POST:
            invite.step = step if step in enabled_steps else invite.event.start_step
            invite.save()

        # Dispatch request to correct step view
        if invite.step == "presentation":
            return PresentationUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "schedule":
            return ScheduleUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "adhesion":
            return AdhesionUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "register":
            return RegisterUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "transport":
            return TransportUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "activity":
            return ActivityUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "summary":
            return SummaryUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "documents":
            return DocumentsUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "decline":
            if invite.event.questions.all():
                return DeclineUpdateView.as_view()(request, *args, **kwargs)
            else:
                invite.step = "greetings"
                invite.save()
                return GreetingsUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "delegate":
            return DelegateUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "forward_to":
            return ForwardUpdateView.as_view()(request, *args, **kwargs)
        elif invite.step == "greetings":
            return GreetingsUpdateView.as_view()(request, *args, **kwargs)

        else:
            invite.step = invite.event.start_step
            invite.save()
            return reverse("event:publieditor_event_my_invite", kwargs={'step': invite.step})


class InviteUpdateView(UpdateView):
    """
    Generic invite view used in not custom steps.
    """

    model = Invite

    def get_object(self):
        invite = Invite.objects.get(pk=self.request.session['publieditor_event_invite_id'])
        self.next_url = self.get_next_url(invite)
        self.previous_url = self.get_previous_url(invite)
        return invite

    def get_next_url(self, invite):
        return reverse("event:publieditor_event_my_invite", kwargs={'step': invite.next_step})

    def get_previous_url(self, invite):
        return reverse("event:publieditor_event_my_invite", kwargs={'step': invite.previous_step})

    def get_template_names(self):
        return [self.object.event.landing_page_template]

    def get(self, request, *args, **kwargs):
        form = self.get_form_instance()
        return self.render_to_response(
            self.get_context_data(
                form=form,
            ))

    def get_step(self):
        return self.object.step

    def get_context_data(self, **kwargs):
        context = super(InviteUpdateView, self).get_context_data(**kwargs)
        context.update({
            "breadcrumbs": get_context_breadcrumbs(self.object),
            "invite": self.object,
            "translation": self.object.event.get_step_translation('translations'),
            "external_images": self.object.get_presentation().get_bmw_external_images(),
            "required_step": self.get_step(),
        })
        context.update(self.get_extra_context_data())
        return context

    def get_extra_context_data(self):
        return dict()

    def is_forward(self):
        if "forward" in self.request.POST:
            return True
        return False

    def post(self, request, *args, **kwargs):
        form = self.get_form_instance(request.POST)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def form_valid(self, form):
        form.save()
        self.get_object()
        return HttpResponseRedirect(self.next_url)

    def form_invalid(self, form):
        return self.render_to_response(
            self.get_context_data(
                form=form,
            ))


class PresentationUpdateView(InviteUpdateView):
    """
    Presentation update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = PresentationForm(post, invite=self.object)
        return form


class ScheduleUpdateView(InviteUpdateView):
    """
    Schedule update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = ScheduleForm(post, invite=self.object)
        return form


class ActionFromEmail(object):
    def __init__(self, **kwargs):
        self.invite = None
        self.contact = None
        self.token_validated = False
        super(ActionFromEmail, self).__init__(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        invite_id = kwargs.get('invite_id', None)
        self.invite = Invite.objects.get(pk=invite_id)

        if self.invite.event.is_deadline_passed():
            return redirect(reverse(
                'event:publieditor_event_deadline_expired',
                kwargs={'invite_id': self.invite.id}
            ))

        uidb36, token = kwargs['uidb36'], kwargs['token']
        self.contact = Contact.objects.get(pk=base36_to_int(uidb36))
        if self.contact.check_token(token) and not self.contact.user and self.invite.event.adhesion_in_email:
            self.token_validated = True
            return super(ActionFromEmail, self).dispatch(request, args, kwargs)
        if not request.user.is_authenticated():
            messages.error(self.request, _("Invalid link"))
        return redirect(reverse('event:login'))


class AcceptFromEmailView(ActionFromEmail, View):
    def get(self, request, *args, **kwargs):
        step = "register"
        if self.token_validated:
            status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
            self.invite.send_on_accept_email()
            self.invite.delete_answers_from_register()
            self.invite.status = status
            self.invite.save()
            request.session['publieditor_event_invite_id'] = self.invite.id
            messages.success(self.request, _('Invitation accepted'))
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )
        else:
            # Non essendoci request.session['publieditor_event_invite_id'] viene dato errore di sessione quindi login
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )


class DeclineFromEmailView(ActionFromEmail, View):
    def get(self, request, *args, **kwargs):
        step = "decline"
        if self.token_validated:
            status = settings.PUBLIEDITOR_EVENT_INVITE_DECLINED
            self.invite.send_on_decline_email()
            self.invite.status = status
            self.invite.save()
            request.session['publieditor_event_invite_id'] = self.invite.id
            messages.success(self.request, _("Invite declined"))
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )
        else:
            # Non essendoci request.session['publieditor_event_invite_id'] viene dato errore di sessione quindi login
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )


class DelegateFromEmailView(ActionFromEmail, View):
    def get(self, request, *args, **kwargs):
        step = "delegate"
        if self.token_validated:
            status = settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
            self.invite.status = status
            self.invite.save()
            request.session['publieditor_event_invite_id'] = self.invite.id
            messages.warning(self.request, "Per favore indica chi intendi delegare per l'evento")
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )
        else:
            # Non essendoci request.session['publieditor_event_invite_id'] viene dato errore di sessione quindi login
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )


class ForwardingFromEmailView(ActionFromEmail, View):

    def get(self, request, *args, **kwargs):
        step = "forward_to"
        if self.token_validated:
            status = settings.PUBLIEDITOR_EVENT_INVITE_FORWARD
            self.invite.status = status
            self.invite.save()
            request.session['publieditor_event_invite_id'] = self.invite.id
            messages.warning(self.request, "Per favore indica a chi intendi inoltrare l'evento")
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )
        else:
            # Non essendoci request.session['publieditor_event_invite_id'] viene dato errore di sessione quindi login
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )


class AdhesionUpdateView(InviteUpdateView):
    """
    Adhesion update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = AdhesionForm(post, invite=self.object)
        return form

    def get_extra_context_data(self):
        extra_context = dict()
        invite = self.get_object()
        adhesion_status_codes = [
            settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
            settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
        ]
        can_edit = True
        if invite.status in adhesion_status_codes:
            can_edit = False
            if self.request.GET.get("edit", None):
                can_edit = True
        if not can_edit:
            status_msgs = {
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED: _("to accept"),
                settings.PUBLIEDITOR_EVENT_INVITE_DECLINED: _("to decline"),
                settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED: _("to delegate"),
            }
            invite_decisions = invite.decision_answers.filter(status=invite.status).order_by('-date_create')
            date_msg = "" if not invite_decisions else _("on") + invite_decisions[0].date_create.strftime(
                " %d/%m/%Y {separator} %H:%M".format(separator=_('at'))
            )
            body = _(u"""
            <p>You already chose %s the invitation %s.<br>
            If you wish to modify your preference click
            <a href="%s?edit=True">here</a>.</p>
            """) % (
                status_msgs[invite.status],
                date_msg,
                reverse("event:publieditor_event_my_invite", kwargs={'step': 'adhesion'})
            )
            extra_context["body"] = body
        extra_context["can_edit"] = can_edit
        return extra_context

    def form_valid(self, form):
        invite = self.get_object()
        form.save()
        status = step = None
        if "accept" in self.request.POST:
            step = "register"
            status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
            invite.send_on_accept_email()
            invite.delete_answers_from_register()
        elif "decline" in self.request.POST:
            status = settings.PUBLIEDITOR_EVENT_INVITE_DECLINED
            step = "decline"
            if not invite.event.show_decline:
                step = 'greetings'
        elif "delegate" in self.request.POST:
            step = "delegate"
            status = settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
        elif "forward_to" in self.request.POST:
            step = "forward_to"
            status = settings.PUBLIEDITOR_EVENT_INVITE_FORWARD

        if self.is_decision_changed() and status and status != invite.status:
            msgs = {
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED: _('Accepted'),
                settings.PUBLIEDITOR_EVENT_INVITE_DECLINED: _('Declined'),
                settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED: _('Delegated'),
                settings.PUBLIEDITOR_EVENT_INVITE_FORWARD: _('Forward'),
            }
            invite.send_on_change_decision_email(msgs[status])
            InviteDecisionAnswer.objects.create(invite=invite, status=status)

        if status and step:
            invite.status = status
            invite.save()
            return HttpResponseRedirect(
                reverse("event:publieditor_event_my_invite", kwargs={'step': step})
            )
        return HttpResponseRedirect(
            reverse("event:publieditor_event_my_invite", kwargs={'step': 'adhesion'})
        )

    def is_decision_changed(self):
        invite = self.get_object()
        if invite.status in [
            settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
            settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
        ]:
            return True
        return False


class RegisterUpdateView(InviteUpdateView):
    """
    Register update view
    """
    def get(self, request, *args, **kwargs):
        form = self.get_form_instance()
        invite = self.get_object()
        # Formset telefono
        phone_formset = None
        if invite.event.ask_phone:
            do_phone_formset = True
            default_phone = invite.contact.default_phone
            phones_queryset = invite.contact.phones.none()
            if invite.event.phone_question_type == "all":
                phones_queryset = invite.contact.phones.all()
            elif invite.event.phone_question_type == "default" and default_phone:
                phones_queryset = invite.contact.phones.filter(id=default_phone.id)
            elif invite.event.phone_question_type == "default" and not default_phone:
                do_phone_formset = False
            if do_phone_formset:
                phone_formset_form = ReadonlyFormsetPhoneForm if \
                    invite.event.phone_question_type == "default" else FormsetPhoneForm
                phone_fields = ('phone_type', 'area_code', 'number')
                phone_formset_class = inlineformset_factory(
                    Contact, Phone, fields=phone_fields, extra=0, can_delete=True, form=phone_formset_form,
                )
                phone_formset = phone_formset_class(instance=self.object.contact, queryset=phones_queryset)
                if invite.event.is_phone_required and len(phone_formset.forms):
                    phone_formset.forms[0].empty_permitted = False
        # Formset indirizzi
        address_formset = None
        if invite.event.ask_address:
            do_address_formset = True
            default_address = invite.contact.default_address
            addresses_queryset = invite.contact.addresses.none()
            if invite.event.address_question_type == "all":
                addresses_queryset = invite.contact.addresses.all()
            elif invite.event.address_question_type == "default" and default_address:
                addresses_queryset = invite.contact.addresses.filter(id=default_address.id)
            elif invite.event.address_question_type == "default" and not default_address:
                do_address_formset = False
            if do_address_formset:
                address_formset_form = ReadonlyFormsetAddressForm if \
                    invite.event.address_question_type == "default" else ContactAddressFormsetForm
                address_fields = (
                    'address_scopes', 'address_type', 'name', 'number', 'zip_code', 'locality', 'province', 'country',
                )
                address_formset_class = inlineformset_factory(
                    Contact, ContactAddress, fields=address_fields, extra=0, can_delete=True, form=address_formset_form,
                    widgets={
                        'address_scopes': forms.CheckboxSelectMultiple,
                        'province': forms.Select(choices=settings.ADDRESS_ADDRESS_PROVINCE_CHOICES),
                    }
                )
                address_formset = address_formset_class(instance=self.object.contact, queryset=addresses_queryset)
                if invite.event.is_address_required and len(address_formset.forms):
                    address_formset.forms[0].empty_permitted = False
        # Formset frequent flyer
        card_formset = None
        if invite.event.ask_frequent_flyer:
            do_card_formset = True
            default_card = invite.contact.default_card
            cards_queryset = invite.contact.cards.none()
            if invite.event.frequent_flyer_question_type == "all":
                cards_queryset = invite.contact.cards.all()
            elif invite.event.frequent_flyer_question_type == "default" and default_card:
                cards_queryset = invite.contact.cards.filter(id=default_card.id)
            elif invite.event.frequent_flyer_question_type == "default" and not default_card:
                do_card_formset = False
            if do_card_formset:
                card_formset_form = ReadonlyFormsetCardForm if \
                    invite.event.frequent_flyer_question_type == "default" else CardFormsetForm
                card_fields = ('airline', 'number', 'expiration_date')
                card_formset_class = inlineformset_factory(
                    Contact, Card, fields=card_fields, extra=0, can_delete=True, form=card_formset_form,
                )
                card_formset = card_formset_class(instance=self.object.contact, queryset=cards_queryset)
                if invite.event.is_frequent_flyer_required and len(card_formset.forms):
                    card_formset.forms[0].empty_permitted = False
        return self.render_to_response(
            self.get_context_data(
                form=form,
                phone_formset=phone_formset,
                address_formset=address_formset,
                card_formset=card_formset,
            ))

    def get_form_instance(self, post=None, files=None):
        self.object = self.get_object()
        form = RegisterForm(post, files, invite=self.object, instance=self.object.contact)
        return form

    def post(self, request, *args, **kwargs):
        form = self.get_form_instance(request.POST, request.FILES)
        invite = self.get_object()
        # Formset telefono
        phone_formset = None
        if invite.event.ask_phone:
            do_phone_formset = True
            default_phone = invite.contact.default_phone
            phones_queryset = invite.contact.phones.none()
            if invite.event.phone_question_type == "all":
                phones_queryset = invite.contact.phones.all()
            elif invite.event.phone_question_type == "default" and default_phone:
                phones_queryset = invite.contact.phones.filter(id=default_phone.id)
            elif invite.event.phone_question_type == "default" and not default_phone:
                do_phone_formset = False
            if do_phone_formset:
                phone_formset_form = ReadonlyFormsetPhoneForm if \
                    invite.event.phone_question_type == "default" else FormsetPhoneForm
                phone_fields = ('phone_type', 'area_code', 'number')
                phone_formset_class = inlineformset_factory(
                    Contact, Phone, fields=phone_fields, extra=0, can_delete=True, form=phone_formset_form,
                )
                phone_formset = phone_formset_class(
                    request.POST, request.FILES, instance=self.object.contact, queryset=phones_queryset
                )
                if invite.event.is_phone_required and len(phone_formset.forms):
                    phone_formset.forms[0].empty_permitted = False
        # Formset indirizzi
        address_formset = None
        if invite.event.ask_address:
            do_address_formset = True
            default_address = invite.contact.default_address
            addresses_queryset = invite.contact.addresses.none()
            if invite.event.address_question_type == "all":
                addresses_queryset = invite.contact.addresses.all()
            elif invite.event.address_question_type == "default" and default_address:
                addresses_queryset = invite.contact.addresses.filter(id=default_address.id)
            elif invite.event.address_question_type == "default" and not default_address:
                do_address_formset = False
            if do_address_formset:
                address_formset_form = ReadonlyFormsetAddressForm if \
                    invite.event.address_question_type == "default" else ContactAddressFormsetForm
                address_fields = (
                    'address_scopes', 'address_type', 'name', 'number', 'zip_code', 'locality', 'province', 'country',
                )
                address_formset_class = inlineformset_factory(
                    Contact, ContactAddress, fields=address_fields, extra=0, can_delete=True, form=address_formset_form,
                    widgets={
                        'address_scopes': forms.CheckboxSelectMultiple,
                        'province': forms.Select(choices=settings.ADDRESS_ADDRESS_PROVINCE_CHOICES),
                    }
                )
                address_formset = address_formset_class(
                    request.POST, request.FILES, instance=self.object.contact, queryset=addresses_queryset
                )
                if invite.event.is_address_required and len(address_formset.forms):
                    address_formset.forms[0].empty_permitted = False
        # Formset frequent flyer
        card_formset = None
        if invite.event.ask_frequent_flyer:
            do_card_formset = True
            default_card = invite.contact.default_card
            cards_queryset = invite.contact.cards.none()
            if invite.event.frequent_flyer_question_type == "all":
                cards_queryset = invite.contact.cards.all()
            if invite.event.frequent_flyer_question_type == "default" and default_card:
                cards_queryset = invite.contact.cards.filter(id=default_card.id)
            elif invite.event.frequent_flyer_question_type == "default" and not default_card:
                do_card_formset = False
            if do_card_formset:
                card_formset_form = ReadonlyFormsetCardForm if \
                    invite.event.frequent_flyer_question_type == "default" else CardFormsetForm
                card_fields = ('airline', 'number', 'expiration_date')
                card_formset_class = inlineformset_factory(
                    Contact, Card, fields=card_fields, extra=0, can_delete=True, form=card_formset_form,
                )
                card_formset = card_formset_class(
                    request.POST, request.FILES, instance=self.object.contact, queryset=cards_queryset
                )
                if invite.event.is_frequent_flyer_required and len(card_formset.forms):
                    card_formset.forms[0].empty_permitted = False
        # Validazione
        formsets_valid = True
        if phone_formset and not phone_formset.is_valid():
            formsets_valid = False
        if address_formset and not address_formset.is_valid():
            formsets_valid = False
        if card_formset and not card_formset.is_valid():
            formsets_valid = False
        if form.is_valid() and formsets_valid:
            return self.form_valid(form, phone_formset, address_formset, card_formset)
        else:
            return self.form_invalid(form, phone_formset, address_formset, card_formset)

    def form_valid(self, form, phone_formset_instance, address_formset_instance, card_formset_instance):
        if self.is_forward():
            contact, contact_diff = form.save()
            contact_edit = ContactEditDiff.objects.create(contact=contact)
            contact_edit.save_contact_edits(contact_diff)
            for formset in [phone_formset_instance, address_formset_instance, card_formset_instance]:
                if formset:
                    formset.instance = contact
                    for form in formset:
                        if form in formset.deleted_forms:
                            form.delete(contact_edit=contact_edit)
                        elif form.has_changed():
                            form.save(contact_edit=contact_edit)
            contact_edit.send()
            contact_edit.delete()
            return HttpResponseRedirect(self.next_url)
        else:
            return HttpResponseRedirect(self.previous_url)

    def form_invalid(self, form, phone_formset_instance, address_formset_instance, card_formset_instance):
        return self.render_to_response(
            self.get_context_data(
                form=form,
                phone_formset=phone_formset_instance,
                address_formset=address_formset_instance,
                card_formset=card_formset_instance,
            ))


class TransportUpdateView(InviteUpdateView):
    """
    Transport update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = TransportForm(post, invite=self.object)
        return form


class ActivityUpdateView(InviteUpdateView):
    """
    Activity update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = ActivityForm(post, invite=self.object)
        return form


class SummaryUpdateView(InviteUpdateView):
    """
    Summary update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = SummaryForm(post, invite=self.object)
        return form

    def form_valid(self, form):
        invite = self.get_object()
        form.save()
        invite.send_on_confirm_email()
        return HttpResponseRedirect(
            reverse("event:publieditor_event_my_invite", kwargs={'step': invite.next_step})
        )


class DocumentsUpdateView(InviteUpdateView):
    """
    Documents update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = DocumentsForm(post, invite=self.object)
        return form

    def get_context_data(self, **kwargs):
        event = self.object.event
        context = super(DocumentsUpdateView, self).get_context_data(**kwargs)
        event_documents = event.event_documents.all()
        context['event_documents'] = event_documents
        return context


class DeclineUpdateView(InviteUpdateView):
    """
    Decline update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = DeclineForm(post, invite=self.object)
        return form

    def form_valid(self, form):
        invite = self.get_object()
        form.save()
        invite.send_on_decline_email()
        return HttpResponseRedirect(
            reverse("event:publieditor_event_my_invite", kwargs={'step': invite.next_step})
        )


class DelegateUpdateView(InviteUpdateView):
    """
    Delegate update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = DelegateForm(post, invite=self.object)
        return form

    def form_valid(self, form):
        invite = self.get_object()
        contact_differs = form.save()
        invite.send_on_delegate_email(contact_differs)
        return HttpResponseRedirect(
            reverse("event:publieditor_event_my_invite", kwargs={'step': invite.next_step})
        )


class ForwardUpdateView(InviteUpdateView):
    """
    Delegate update view
    """

    def __init__(self, **kwargs):
        self.object = None
        self.data = dict()
        self.success_message = _('All indicated contact have already received your forward')
        super(ForwardUpdateView, self).__init__(**kwargs)

    def get_form_instance(self, post=None):
        self.object = self.get_object()

        self.data = {
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            # 'form-MAX_NUM_FORMS': self.object.event.max_limit_forwarders,
        }

        if self.request.method == 'GET':
            formset = ForwardToFormset(
                self.data, initial=[{'area_code': AreaCode.objects.get(area='Italia').pk}], main_invite=self.object
            )
            formset.is_bound = False
            return formset
        else:
            return ForwardToFormset(self.request.POST, main_invite=self.object)

    def form_valid(self, formset):
        for form in formset:
            # form.invite = self.object
            if not form.cleaned_data['email'] in self.object.forwarded_invitations.all():
                self.success_message = _('Our backoffice will receive a notification to check and approve this forward')
                form.save()
        messages.success(self.request, self.success_message)
        self.object.status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
        self.object.save()
        return HttpResponseRedirect(reverse("event:publieditor_event_my_invite", kwargs={'step': 'register'}))

    def get_context_data(self, **kwargs):
        context = super(ForwardUpdateView, self).get_context_data(**kwargs)
        already_forwarded_email = self.object.forwarded_invitations.all().values_list('contact__email', flat=True)
        context['already_forwarded_email'] = already_forwarded_email
        context['MAX_FORM'] = self.object.event.max_limit_forwarders
        return context


class GreetingsUpdateView(InviteUpdateView):
    """
    Greetings update view
    """
    def get_form_instance(self, post=None):
        self.object = self.get_object()
        form = GreetingsForm(post, invite=self.object)
        return form

    def get(self, request, *args, **kwargs):
        response = super(GreetingsUpdateView, self).get(request, *args, **kwargs)
        if self.object.event.slug == settings.EVENT_EVENT_TRIUMPH_SLUG:
            self.object.send_summary_email()

        return response


class SetSessionByInviteFormView(FormView):
    template_name = 'event/set_session_by_invite_form.html'
    form_class = SetSessionByInviteForm

    def get_success_url(self):
        return self.invite.get_url_landingpage() if not self.invite.event.is_privacy else self.invite.get_privacy_landingpage()

    def __init__(self, **kwargs):
        self.invite = None
        self.event = None
        super(SetSessionByInviteFormView, self).__init__(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        invite_id = kwargs.get('invite_id')
        self.invite = Invite.objects.get(pk=invite_id)
        self.event = self.invite.event
        return super(SetSessionByInviteFormView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(SetSessionByInviteFormView, self).get_context_data(**kwargs)
        ctx['landing_page_template'] = self.event.landing_page_template
        ctx['invite_id'] = self.invite.pk
        return ctx
