# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from django.db.models.query import QuerySet

from jmb.i18n.models import Language
from jmb.mailer.sender import Mailer

from publieditor.event.models import (
    TransportAnswer, Invite, RegisterAnswer, ActivityAnswer,
    DeclineAnswer, Turn, OwnAnswer, FreeAnswer, PollAnswer
)
from publieditor.survey.models import Question
from publieditor.event.models import DelegatedContact, Contact
from publieditor.event.utils.mailer import get_invited_description


# PresentationForm
class PresentationForm(forms.Form):
    pass


# ScheduleForm
class ScheduleForm(forms.Form):
    pass


# AdhesionForm
class AdhesionForm(forms.Form):
    pass


class QuestionForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(QuestionForm, self).__init__(*args, **kwargs)

    def set_question_fields(self, typology):
        if self.event:
            event_questions = self.event.questions.filter(
                language_id=self.event.default_language_id,
                enabled=True,
                question__typology=typology,
            ).order_by('ordering')

            for event_question in event_questions:
                key = "question_%d" % event_question.question.id
                field_class, widget_class = self.get_form_question_widget_class(event_question.question)
                self.fields[key] = field_class
                self.fields[key].label = event_question.question.question.capitalize()
                self.fields[key].empty_label = None
                self.fields[key].required = True if event_question.required else False
                self.set_field_initial(key, typology, event_question.question)
                self.fields[key].help_text = None

                if "privacy" in event_question.question.category.name.lower():
                    error_msg = _("Privacy must be accepted.")
                    self.fields[key].error_messages = {"required": error_msg}
                    self.fields[key].help_text = "CIAO"

    def get_form_question_widget_class(self, question):
        """
        Return field class and widget class given qtype field
        """
        fields_widget_tuple_dict = {
            'T': [
                forms.CharField(widget=forms.TextInput), forms.TextInput
            ],
            'A': [
                forms.CharField(widget=forms.Textarea), forms.Textarea
            ],
            'S': [
                forms.ModelChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.Select
                ),
                forms.Select,
            ],
            'R': [
                forms.ModelChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.RadioSelect,
                ),
                forms.RadioSelect,
            ],
            'C': [
                forms.ModelMultipleChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.CheckboxSelectMultiple
                ),
                forms.CheckboxSelectMultiple,
            ]
        }
        return tuple(fields_widget_tuple_dict.get(question.qtype))

    def set_field_initial(self, key, typology, question):
        """
        Set field initial given a key and a question
        """
        if typology == "R":
            question_answers = RegisterAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "A":
            question_answers = ActivityAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "D":
            question_answers = DeclineAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "P":
            question_answers = PollAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )

        if question_answers.count():
            question_answer = question_answers[0]
            if question_answer:
                if question.qtype == "T":
                    self.fields[key].initial = question_answer.answer

                elif question.qtype == "A":
                    self.fields[key].initial = question_answer.answer

                elif question.qtype == "R":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    self.fields[key].initial = previous_answers[0].id

                elif question.qtype == "S":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    self.fields[key].initial = previous_answers[0].id

                elif question.qtype == "C":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    self.fields[key].initial = previous_answers

    def save(self):
        pass


# RegisterForm
class RegisterForm(QuestionForm):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        self.contact = self.invite.contact
        self.session_invite_turn = kwargs.pop('session_invite_turn')
        super(RegisterForm, self).__init__(*args, **kwargs)

        if self.contact and self.event:
            # Set contact fields
            self.fields['title'] = forms.ChoiceField(
                choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES,
                initial=self.contact.title
            )
            self.fields['name'] = forms.CharField(
                _("Name"),
                required=True,
                initial="%s %s" % (
                    self.contact.first_name, self.contact.last_name
                ),
            )
            self.fields['email'] = forms.EmailField(
                _("Email"),
                required=True,
                initial=self.contact.email,
            )
            self.fields['title'].widget.attrs['readonly'] = True
            self.fields['name'].widget.attrs['readonly'] = True
            self.fields['email'].widget.attrs['readonly'] = True

            # Set options field for available event turns
            if self.invite.event.turns.count() > 1:
                queryset = [(x.id, "%s %s"%(x.title, x.description)) for x in self.event.turns.all()]
                self.fields['turn'] = forms.ChoiceField(
                    choices=queryset,
                    widget=forms.RadioSelect,
                    label=_('turns')
                )

            # Set turn field initial getting it from db or session
            previous_transport_answers = TransportAnswer.objects.filter(
                invite__event=self.invite.event,
                invite__contact=self.invite.contact,
            ).distinct('transport_detail__turn').all()
            if self.event.turns.count()>1:
                if len(previous_transport_answers):
                   self.fields['turn'].initial = previous_transport_answers[0].transport_detail.turn.id
                elif isinstance(self.session_invite_turn, Turn):
                   self.fields['turn'].initial = self.session_invite_turn.id

            # Set questions fields of Register typology
            self.set_question_fields("R")

            self.fields['privacy'] = forms.BooleanField(
                _('Privacy'),
            )

    def clean(self):
        super(RegisterForm, self).clean()
        if 'privacy' not in self.cleaned_data:
            return False
        return self.cleaned_data


    def save(self):
        # Update contact data
        self.contact.title = self.cleaned_data["title"]
        self.contact.email = self.cleaned_data["email"]
        # self.contact.phone = self.cleaned_data["phone"]
        # self.contact.company = self.cleaned_data["company"]
        self.contact.save()

        # Delete previous answers
        RegisterAnswer.objects.filter(
            invite=self.invite,
        ).delete()

        # Save new answers
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question = Question.objects.get(pk=int(key.split("question_")[-1]))
            answer = self.cleaned_data[key]
            if answer:
                if isinstance(answer, QuerySet):
                    answer = "|".join(val['choice'] for val in answer.values())
                if answer != "":
                    RegisterAnswer(
                        invite=self.invite,
                        question=question,
                        answer=answer,
                    ).save()


# DelegateForm
class DelegateForm(QuestionForm):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(DelegateForm, self).__init__(*args, **kwargs)

        self.fields['title'] = forms.ChoiceField(
            choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES,
        )
        self.fields['first_name'] = forms.CharField(
            _("First name"),
            required=True,
        )
        self.fields['last_name'] = forms.CharField(
            _("Last name"),
            required=True,
        )
        self.fields['email'] = forms.EmailField(
            _("Email"),
            required=True,
        )

    def save(self):
        event = self.invite.event
        # Recupero contatto se esistente, altrimenti lo creo
        try:
           contact = Contact.objects.get(email=self.cleaned_data["email"],)
        except Contact.DoesNotExist:
            contact = Contact(
                title=self.cleaned_data["title"],
                first_name=self.cleaned_data["first_name"],
                last_name=self.cleaned_data["last_name"],
                language=self.invite.contact.language,
                email=self.cleaned_data["email"],
            )
            contact.save()
        # Creazione ed invio invito
        mailer = Mailer(event)
        invite, contact_created = Invite.objects.get_or_create(
            contact=contact, event=event
        )
        description = get_invited_description(event.translations, invite, self.invite.contact)
        event_translation = event.translations.get(
            language_id=contact.language.id,
        )
        left_signature = event_translation.left_signature
        right_signature = event_translation.right_signature
        site = Site.objects.get_current()
        site_domain = site.domain
        context = dict(
            description=description,
            left_signature=left_signature,
            right_signature=right_signature,
            site_domain=site_domain,
            url_landing_page=invite.get_url_landingpage(),
        )
        mail_created, email_object = mailer.create_message(contact, context)
        if mail_created:
            invite.sent = True
            invite.save()
            mailer.send(email_object)
        # Creazione DelegatedContact
        self.invite.delegated_invites.all().delete()
        delegated_contact = DelegatedContact(invited=contact, invite=self.invite)
        delegated_contact.save()


# DeclineForm
class DeclineForm(QuestionForm):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        self.contact = self.invite.contact
        super(DeclineForm, self).__init__(*args, **kwargs)

        if self.contact and self.event:
            self.set_question_fields("D")

    def save(self):
        # Delete previous answers
        DeclineAnswer.objects.filter(
            invite=self.invite,
        ).delete()
        # Save new answers
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question = Question.objects.get(pk=int(key.split("question_")[-1]))
            answer = self.cleaned_data[key]
            if answer:
                if isinstance(answer, QuerySet):
                    answer = "|".join(val['choice'] for val in answer.values())
                if answer != "":
                    DeclineAnswer(
                        invite=self.invite,
                        question=question,
                        answer=answer,
                    ).save()


# TransportForm
class TransportForm(forms.Form):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        self.session_event_turn = kwargs.pop('session_event_turn')
        super(TransportForm, self).__init__(*args, **kwargs)

        if self.invite:

            # Get available transport details for event and turn
            previous_transport_answers = TransportAnswer.objects.filter(
                invite__event=self.invite.event,
                invite__contact=self.invite.contact,
            ).distinct('transport_detail__turn').all()

            if previous_transport_answers:
                self.session_event_turn = previous_transport_answers[0].transport_detail.turn
            if self.session_event_turn:
                aggregation_keys = self.session_event_turn.get_transport_detail_aggregation_keys()
            else:
                aggregation_keys = self.invite.event.get_transport_detail_aggregation_keys()
            choices = []
            for key, value in aggregation_keys.iteritems():
                choices.append((value, value))

            # Set transport details field
            self.fields["transport_ids"] = forms.ChoiceField(
                choices=choices,
                required=False,
                widget=forms.RadioSelect,
            )

            # If enabled get own questions
            initial = True if self.invite.own_answers.all() else False
            if self.event.enable_own:
                self.fields['own'] = forms.BooleanField(
                    required=False,
                    initial=initial,
                )

            # If enabled get free questions
            initial = None
            free_answers = self.invite.free_answers.all()
            if free_answers:
                initial = free_answers[0].answer
            if self.event.enable_free:
                self.fields['free'] = forms.CharField(
                    widget=forms.Textarea,
                    required=False,
                    initial=initial,
                )

    def clean(self):
        cleaned_data = super(TransportForm, self).clean()
        selected = 0
        if 'own' in cleaned_data and cleaned_data['own']:
            selected += 1
        if 'free' in cleaned_data and cleaned_data['free'].strip() != "":
            selected += 1
        if 'transport_ids' in cleaned_data and cleaned_data['transport_ids'].strip() != "":
            selected += 1

        if selected == 0:
            raise forms.ValidationError(_("Please select a travel option"))
        elif selected > 1:
            raise forms.ValidationError(_("Please select only one travel option"))

        return cleaned_data


    def save(self):
        # Delete previous answers
        for answer in [OwnAnswer, FreeAnswer, TransportAnswer]:
            answer.objects.filter(
                invite=self.invite,
            ).delete()

        # If enabled and checked save OwnAnswer
        if self.event.enable_own and self.cleaned_data["own"]:
            OwnAnswer(
                invite=self.invite,
            ).save()

        # else if enabled and checked save FreeAnswer
        elif self.event.enable_free and len(self.cleaned_data["free"].strip())>0:
            FreeAnswer(
                invite=self.invite,
                answer=self.cleaned_data['free'],
            ).save()

        # else save TransportAnswer
        else:
            for key in self.cleaned_data['transport_ids'].split("_"):
                TransportAnswer(
                    transport_detail_id=int(key),
                    invite=self.invite,
                ).save()


# ActivityForm
class ActivityForm(QuestionForm):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        self.contact = self.invite.contact
        super(ActivityForm, self).__init__(*args, **kwargs)

        if self.contact and self.event:
            self.set_question_fields("A")

    def save(self):
        # Delete previous answers
        ActivityAnswer.objects.filter(
            invite=self.invite,
        ).delete()
        # Save new answers
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question = Question.objects.get(pk=int(key.split("question_")[-1]))
            answer = self.cleaned_data[key]
            if answer:
                if isinstance(answer, QuerySet):
                    answer = "|".join(val['choice'] for val in answer.values())
                if answer != "":
                    ActivityAnswer(
                        invite=self.invite,
                        question=question,
                        answer=answer,
                    ).save()


# SummaryForm
class SummaryForm(forms.Form):
    pass

# DocumentsForm
class DocumentsForm(forms.Form):
    pass

# GreetingsForm
class GreetingsForm(forms.Form):
    pass


# SurveyForm
class PollForm(QuestionForm):

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        super(PollForm, self).__init__(*args, **kwargs)

        if self.invite:
            self.set_question_fields("P")

    def save(self):
        # Delete previous answers
        PollAnswer.objects.filter(
            invite=self.invite,
        ).delete()
        # Save new answers
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question = Question.objects.get(pk=int(key.split("question_")[-1]))
            answer = self.cleaned_data[key]
            if answer:
                if isinstance(answer, QuerySet):
                    answer = "|".join(val['choice'] for val in answer.values())
                if answer != "":
                    PollAnswer(
                        invite=self.invite,
                        question=question,
                        answer=answer,
                    ).save()