# -*- coding: utf-8 -*-

from django.conf import settings
from django.core import serializers
from django.http import HttpResponseRedirect
from django.template import loader
from django.template.base import Context
from django.utils.http import int_to_base36
from django.utils.translation import ugettext as _

from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.functions import import_class

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from publieditor.event.models import(
    EventTranslation, Invite, TransportAnswer,
    RegisterAnswer, ActivityAnswer
)
from publieditor.event.cms.models import (
    ScheduleSettings, HeaderImageSettings, SignatureSettings,
    RegisterSettings,
    AdhesionSettings, TransportSettings,
    ActivitySettings, PresentationSettings,
    SummarySettings, TitleSettings, BreadcrumbSettings,
    DelegateSettings, DeclineSettings,
    DocumentsSettings, GreetingsSettings, PollSettings
)



# Root class to inherit
class AbstractPluginPlugin(CMSPluginBase):
    fields = (
        'status', 'invite_status_to_be_visible', 'template_name', 'form'
    )

    # Imposta se sessione vuota ed il plugin è il primo
    def _set_session_step_first_plugin(self, session, plugin_dict):
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict.get(
                    'plugin'
                ).get_plugin_name().lower()

    def _get_list_plugins(self, plugins):
        list_plugins = []
        for plugin in plugins:
            list_plugins.append(plugin)
        return list_plugins

    def _get_plugin_positions(self, instance):
        plugins = instance.placeholder.cmsplugin_set.all().order_by('position')
        list_plugins = self._get_list_plugins(plugins)
        plugin = instance.cmsplugin_ptr
        position = list_plugins.index(plugin)
        previous_plugin = ""
        next_plugin = ""
        is_first = False
        is_last = False

        if position == 0:
            is_first = True
        else:
            previous_plugin = list_plugins[position-1]

        if position == len(list_plugins)-1:
            is_last = True
        else:
            next_plugin = list_plugins[position+1]

        plugin_dict = {
            'plugins': list_plugins,
            'plugin': plugin,
            'position': position,
            'is_first': is_first,
            'is_last': is_last,
            'previous_plugin': previous_plugin,
            'next_plugin': next_plugin
        }
        return plugin_dict

    def render_template_from_string(self, context_dict, body):
        if 'invite' in context_dict and hasattr(context_dict['invite'], "get_landingpage"):
            context_dict['invite'].get_landingpage = "su avanti"
        context = Context(context_dict)
        template = loader.get_template_from_string(body)
        ret = template.render(context)
        return ret if not "<code></code>" in ret else ret.replace("<code></code>", "")

    def get_invite(self, session):
        try:
            invite = Invite.objects.get(pk=session['publieditor_event_invite_id'])
        except Exception, e:
            invite = Invite.objects.all()[0]
        return invite


# Presentation plugin class
class PresentationPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = PresentationSettings
    name = 'Presentation'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)
        self._set_session_step_first_plugin(session, plugin_dict)

        # inserisco in sesione i plugin presenti in 'Content_Central_Inside'
        session['publieditor_event_plugins'] = serializers.serialize("json", plugin_dict['plugins'])

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            contact = invite.contact
            event = invite.event

            event_translation = event.translations.get(
                language_id=contact.language.id,
            )
            presentation = event_translation
            description = self.render_template_from_string(
                dict(contact=contact, invite=invite),
                presentation.body
            )
            left_signature = event_translation.left_signature
            right_signature = event_translation.right_signature

            # If back button was pressed
            if (
                request.method == 'POST' and
                'presentation_reset' in request.POST and
                not plugin_dict['is_first']
            ):
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'presentation_submit' in request.POST:
                plugin_form = form_class(request.POST)
                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'presentation_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'presentation': presentation,
                'description': description,
                'plugin_dict': plugin_dict,
                'left_signature': left_signature,
                'right_signature': right_signature,
            })

        return context
plugin_pool.register_plugin(PresentationPlugin)


# Schedule plugin class (event schedule step)
class SchedulePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = ScheduleSettings
    name = 'Schedule'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            contact = invite.contact
            event = invite.event
            schedule = event.schedule_translations.get(
                language__id=contact.language.id
            )
            description = self.render_template_from_string(
                dict(contact=contact, invite=invite),
                schedule.body
            )

            # If back button was pressed
            if (
                request.method == 'POST' and
                'schedule_reset' in request.POST and
                not plugin_dict['is_first']
            ):
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If next step was pressed
            if request.method == 'POST' and 'schedule_submit' in request.POST:
                plugin_form = form_class(request.POST)
                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'schedule_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'schedule': schedule,
                'description': description,
                'plugin_dict': plugin_dict
            })
        return context
plugin_pool.register_plugin(SchedulePlugin)


# Adhesion plugin class (event adhesion step)
class AdhesionPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = AdhesionSettings
    name = 'Adhesion'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            contact = invite.contact
            event = invite.event
            adhesion = event.adhesion_translations.get(
                language__id=contact.language.id
            )
            description = self.render_template_from_string(
                dict(contact=contact, invite=invite),
                adhesion.body
            )

            # If accept button was pressed
            if request.method == 'POST' and 'adhesion_accept' in request.POST:
                invite.status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
                invite.save()
                session['publieditor_event_step'] = "register"
                self.render_template = "event/cms_plugins/empty.html"

            # If decline button was pressed
            if request.method == 'POST' and 'adhesion_decline' in request.POST:
                invite.status = settings.PUBLIEDITOR_EVENT_INVITE_DECLINED
                invite.save()
                session['publieditor_event_step'] = "decline"
                self.render_template = "event/cms_plugins/empty.html"

            # If delegate button was pressed
            if request.method == 'POST' and 'adhesion_delegate' in request.POST:
                invite.status = settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
                invite.save()
                session['publieditor_event_step'] = "delegate"
                self.render_template = "event/cms_plugins/empty.html"

            # If back button was pressed
            if request.method == 'POST' and 'adhesion_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            plugin_form = form_class()

            context.update({
                'object': instance,
                'register_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'adhesion': adhesion,
                'description': description,
                'plugin_dict': plugin_dict,
                'event': event,
            })
        return context
plugin_pool.register_plugin(AdhesionPlugin)


# Register plugin class (event register step)
class RegisterPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = RegisterSettings
    name = 'Register'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # imposta se sessione vuota ed il plugin è il primo
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            event = invite.event
            contact = invite.contact

            privacy_translation = event.privacy_translations.get(
                language_id=contact.language.id,
            )
            privacy_translation = self.render_template_from_string(
                dict(contact=contact, invite=invite),
                privacy_translation.body
            )

            # If back button was pressed
            if request.method == 'POST' and 'register_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = "adhesion"
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'register_submit' in request.POST:
                plugin_form = form_class(
                    request.POST,
                    invite=invite,
                    session_invite_turn=session.get('publieditor_invite_turn', None),
                )

                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    plugin_form.save()
                    session['publieditor_invite_turn'] = None
                    # If asked save invite turn
                    if "turn" in plugin_form.cleaned_data:
                        session['publieditor_invite_turn'] = invite.event.turns.get(pk=plugin_form.cleaned_data["turn"])

                    # Go forward
                    session['publieditor_event_step'] = "transport"
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class(
                    invite=invite,
                    session_invite_turn=session.get('publieditor_invite_turn', None),
                )

            print privacy_translation

            context.update({
                'object': instance,
                'register_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict,
                'privacy_translation': privacy_translation
            })
        return context
plugin_pool.register_plugin(RegisterPlugin)


# Delegate plugin class (event delegate step)
class DelegatePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = DelegateSettings
    name = 'Delegate'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            # If back button was pressed
            if request.method == 'POST' and 'delegate_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = "adhesion"
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'delegate_submit' in request.POST:
                plugin_form = form_class(
                    request.POST,
                    invite=invite,
                )
                if plugin_form.is_valid():
                    plugin_form.save()
                    session['publieditor_event_step'] = "greetings"
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class(
                    invite=invite,
                )

            context.update({
                'object': instance,
                'delegate_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict,
            })
        return context
plugin_pool.register_plugin(DelegatePlugin)


# Decline plugin class (event decline step)
class DeclinePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = DeclineSettings
    name = 'Decline'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            # If back button was pressed
            if request.method == 'POST' and 'decline_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = "adhesion"
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'decline_submit' in request.POST:
                plugin_form = form_class(
                    request.POST,
                    invite=invite,
                )
                if plugin_form.is_valid():
                    plugin_form.save()
                    # Go forward
                    session['publieditor_event_step'] = "greetings"
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class(
                    invite=invite,
                )

            context.update({
                'object': instance,
                'decline_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict
            })
        return context
plugin_pool.register_plugin(DeclinePlugin)


# Transport plugin class (event transport step)
class TransportPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = TransportSettings
    name = 'Transport'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)
        error_msg = None

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            # Set session turn
            session_event_turn = None
            if 'publieditor_invite_turn' in session:
                session_event_turn = session['publieditor_invite_turn']

            # Get previous answer details for initial
            transport_answers = TransportAnswer.objects.filter(
                invite=invite
            ).all()
            transport_ids_initial = ""
            for transport_answer in transport_answers:
                transport_ids_initial += "%s_"%transport_answer.transport_detail_id
            transport_ids_initial = transport_ids_initial.rstrip("_")

            # If back button was pressed
            if request.method == 'POST' and 'transport_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = "register"
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if not 'publieditor_invite_turn' in session:
                session['publieditor_invite_turn'] = None
            if request.method == 'POST' and 'transport_submit' in request.POST:
                plugin_form = form_class(
                    request.POST,
                    invite=invite,
                    session_event_turn=session_event_turn,
                )
                if plugin_form.is_valid():
                    plugin_form.save()
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class(
                    invite=invite,
                    session_event_turn=session_event_turn,
                )

            context.update({
                'object': instance,
                'transport_form': plugin_form,
                'event': invite.event,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict,
                'transport_ids_initial': transport_ids_initial,
                'is_transport_answer': True if invite.transport_answers.all().count() else False,
                'is_free_answer': True if invite.free_answers.all().count() else False,
                'is_own_answer': True if invite.own_answers.all().count() else False,
            })
        return context
plugin_pool.register_plugin(TransportPlugin)


# Activity plugin class (event activity step)
class ActivityPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = ActivitySettings
    name = 'Activity'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # Set if session is empty and plugin is first
        if not 'publieditor_event_step' in session:
            session['publieditor_event_step'] = ''
            if plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            activity = invite.event.translations.get(
                language__id=invite.contact.language.id
            )

            # If back button was pressed
            if request.method == 'POST' and 'activity_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'activity_submit' in request.POST:

                plugin_form = form_class(
                    request.POST,
                    invite=invite,
                )

                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    plugin_form.save()

                    # Go forward
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class(
                    invite=invite,
                )

            context.update({
                'object': instance,
                'activity_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'activity': activity,
                'plugin_dict': plugin_dict
            })
        return context
plugin_pool.register_plugin(ActivityPlugin)


# Summary plugin class (event summary step)
class SummaryPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = SummarySettings
    name = 'Summary'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        self._set_session_step_first_plugin(session, plugin_dict)
        invite = self.get_invite(session)

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            contact = invite.contact
            event = invite.event
            summary = event.translations.get(language__id=contact.language.id)
            grouped_invite_answer = invite.get_grouped_answers()

            # If back button was pressed
            if request.method == 'POST' and 'summary_reset' in request.POST and not plugin_dict['is_first']:
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'summary_submit' in request.POST:
                plugin_form = form_class(request.POST)
                if not plugin_dict['is_last']:
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"
            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'placeholder': placeholder,
                'session': session,
                'summary': summary,
                'plugin_dict': plugin_dict,
                'summary_form': plugin_form,
                'contact': contact,
                'grouped_invite_answer': grouped_invite_answer,
                'transport_dict': invite.get_transport_dict(),
                'own_answers': invite.own_answers.all(),
                'free_answers': invite.free_answers.all(),
            })
        return context
plugin_pool.register_plugin(SummaryPlugin)


# Header Image plugin class
class HeaderImagePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = HeaderImageSettings
    name = 'Header Image'
    admin_preview = False
    fields = ('status', 'invite_status_to_be_visible', 'width', 'height')
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        self._set_session_step_first_plugin(session, plugin_dict)

        invite = self.get_invite(session)
        contact = invite.contact
        event = invite.event
        presentation = EventTranslation.objects.get(
            language_id=contact.language.id,
            event_id=event.id
        )
        self.render_template = "event/cms_plugins/header_image.html"

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'session': session,
            'presentation': presentation,
            'plugin_dict': plugin_dict
        })
        return context
plugin_pool.register_plugin(HeaderImagePlugin)


# Signature plugin class
class SignaturePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = SignatureSettings
    name = 'Signature'
    admin_preview = False
    fields = ('status', 'invite_status_to_be_visible',)
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        self._set_session_step_first_plugin(session, plugin_dict)

        invite = Invite.objects.get(pk=session['publieditor_event_invite_id'])
        event = invite.event

        if instance.status != 1:
            self.render_template = "event/cms_plugins/empty.html"

        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:

            # EventTranslation.objects.filter(
            #
            # )

            self.render_template = "event/cms_plugins/signature.html"
            context.update({
                'object': instance,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict
            })

        return context

plugin_pool.register_plugin(SignaturePlugin)


class TitlePlugin(AbstractPluginPlugin):
    module = 'Event'
    model = TitleSettings
    name = 'Title'
    admin_preview = False
    fields = ('status', 'invite_status_to_be_visible',)
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        self._set_session_step_first_plugin(session, plugin_dict)

        invite = self.get_invite(session)
        contact = invite.contact
        event = invite.event

        if instance.status != 1:
            self.render_template = "event/cms_plugins/empty.html"

        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            presentation = EventTranslation.objects.get(
                language_id=contact.language.id,
                event_id=event.id
            )
            self.render_template = "event/cms_plugins/title.html"
            context.update({
                'object': instance,
                'placeholder': placeholder,
                'session': session,
                'presentation': presentation,
                'plugin_dict': plugin_dict
            })

        return context
plugin_pool.register_plugin(TitlePlugin)


# Document plugin class
class DocumentsPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = DocumentsSettings
    name = 'Documents'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # inserisco in sesione i plugin presenti in 'Content_Central_Inside'
        session['publieditor_event_plugins'] = serializers.serialize("json", plugin_dict['plugins'])

        # Check if step is the actual one
        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            self._set_session_step_first_plugin(session, plugin_dict)
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            # Get invite link
            base_36 = invite.make_base36()
            token = invite.make_token()
            event = invite.event
            event_documents = event.event_documents.all()

            # If back button was pressed
            if (
                request.method == 'POST' and
                'documents_reset' in request.POST and
                not plugin_dict['is_first']
            ):
                session['publieditor_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'documents_submit' in request.POST:
                plugin_form = form_class(request.POST)
                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    session['publieditor_event_step'] = "greetings"
                    self.render_template = "event/cms_plugins/empty.html"

            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'presentation_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'plugin_dict': plugin_dict,
                'event': event,
                'event_documents': event_documents,
                'invite': invite,
                'base_36': base_36,
                'token': token
            })

        return context
plugin_pool.register_plugin(DocumentsPlugin)


# Greetings plugin class
class GreetingsPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = GreetingsSettings
    name = 'Greetings'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        # inserisco in sesione i plugin presenti in 'Content_Central_Inside'
        session['publieditor_event_plugins'] = serializers.serialize("json", plugin_dict['plugins'])

        # Check if step is the actual one

        if session['publieditor_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            self._set_session_step_first_plugin(session, plugin_dict)
            form_class = import_class(instance.form)

            # Update invite step
            invite.step = session['publieditor_event_step']
            invite.save()

            delegated_invites = invite.delegated_invites.all()
            if delegated_invites:
                contact = delegated_invites[0].invited
            else:
                contact = invite.contact
            event = invite.event

            greetings_translations = event.greetings_translations.filter(
                language__id=contact.language.id,
                invite_status=invite.status,
            )

            description = ""
            if greetings_translations.count():
                greetings_translation = greetings_translations[0]
                description = self.render_template_from_string(
                    dict(contact=contact, invite=invite, invitant_contact=invite.contact),
                    greetings_translation.body
                )

            # If back button was pressed
            if (
                request.method == 'POST' and
                'greetings_reset' in request.POST and
                not plugin_dict['is_first']
            ):

                list_plugins_instances = []
                for data in serializers.deserialize("json", session['publieditor_event_plugins']):
                    plugin_instance, plugin = data.object.get_plugin_instance()
                    if plugin_instance.invite_status_to_be_visible==invite.status:
                        list_plugins_instances.append(plugin_instance.get_plugin_name().lower())
                session['publieditor_event_step'] = list_plugins_instances[-1]

                self.render_template = "event/cms_plugins/empty.html"

            # If next button was pressed
            if request.method == 'POST' and 'greetings_submit' in request.POST:
                plugin_form = form_class(request.POST)
                if plugin_form.is_valid() and not plugin_dict['is_last']:
                    session['publieditor_event_step'] = plugin_dict['next_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"
                if plugin_form.is_valid() and plugin_dict['is_last']:
                    raise ForceResponse(HttpResponseRedirect('/'))
            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'presentation_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'description': description,
                'plugin_dict': plugin_dict
            })

        return context
plugin_pool.register_plugin(GreetingsPlugin)


# Poll plugin class
class PollPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = PollSettings
    name = 'Poll'
    admin_preview = False
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        invite = self.get_invite(session)

        self.render_template = instance.template_name
        self._set_session_step_first_plugin(session, plugin_dict)
        form_class = import_class(instance.form)

        # If next button was pressed
        if request.method == 'POST' and 'poll_send' in request.POST:
            plugin_form = form_class(
                request.POST,
                invite=invite,
            )

            if plugin_form.is_valid():
                plugin_form.save()
                raise ForceResponse(HttpResponseRedirect('/'))

        else:
            plugin_form = form_class(invite=invite)

        context.update({
            'object': instance,
            'event': invite.event,
            'contact': invite.contact,
            'poll_form': plugin_form,
        })

        return context
plugin_pool.register_plugin(PollPlugin)


class BreadcrumbPlugin(AbstractPluginPlugin):
    module = 'Event'
    model = BreadcrumbSettings
    name = 'Breadcrumbs'
    admin_preview = False
    fields = ('status', 'invite_status_to_be_visible',)
    render_template = "event/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)
        self._set_session_step_first_plugin(session, plugin_dict)

        invite = self.get_invite(session)

        if instance.status != 1:
            self.render_template = "event/cms_plugins/empty.html"

        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            plugin_instances = []
            # prende i plugin serializzati json dalla sessione
            for data in serializers.deserialize("json", session['publieditor_event_plugins']):
                instance, plugin = data.object.get_plugin_instance()
                if (
                    instance and (
                        instance.invite_status_to_be_visible is None or
                        instance.invite_status_to_be_visible == invite.status)
                ):
                    plugin_instances.append(instance)

            invite_base36 = int_to_base36(invite.id)
            invite_token = invite.make_token()

            self.render_template = "event/cms_plugins/breadcrumbs.html"
            context.update({
                'object': instance,
                'placeholder': placeholder,
                'session': session,
                'plugin_instances': plugin_instances,
                'post_url': "%s-%s" % (invite_base36, invite_token)
            })

        return context
plugin_pool.register_plugin(BreadcrumbPlugin)
