# -*- coding: utf-8 -*-
"""
forms module for publieditor.event
"""
import datetime
import autocomplete_light
from django import forms
from django.conf import settings
from django.db.models import Q
from django.utils.http import base36_to_int
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.contrib.auth.hashers import make_password
from django.contrib.auth.forms import AuthenticationForm
from django.core.urlresolvers import reverse
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Field, Div, Submit, HTML
from crispy_forms.bootstrap import TabHolder, Tab
from django.contrib.sites.models import Site

from jmb.core.forms.widgets import DatePicker
from jmb.core.admin import ConstrainedModelForm
from jmb.core.forms import AdminModelForm
from jmb.core.forms.widgets import DateTimePicker, TimePicker
from jmb.core.middleware.thread_local import get_request

from publieditor.event.models import (
    Contact, Event, Invite, InviteProxy,
    TransportAnswer, Flight, Airport, Airline, FlightDetail,
    Train, Station, TrainType, Turn, TransportDetail,
    EventTurnProxy, EventStartPlace, EventTravelOption, Card, Phone, ContactAddress,
    ContactEmailCc, Membership,
    CustomTransportDetail, CustomTransport, EventPlace)
from publieditor.event.crispy_bootstrap import Formset
from publieditor.event.admin_widgets import AdminProportionImageWidget


class ContactImportFileForm(forms.Form):
    import_file = forms.FileField(
        label=_("File"),
        required=True,
    )
    set_in_ml = forms.BooleanField(
        label=_("Set in mailing list"),
        required=False,

    )


class TrainDetailAdminForm(ConstrainedModelForm):
    event_travel_option = forms.ModelChoiceField(
        queryset=EventTravelOption.objects.all(), required=True,
        label=_("Event travel option"),
    )
    train = forms.ModelChoiceField(
        queryset=Train.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('TrainAutocomplete'),
        label=_("Train"),
    )
    departure = forms.DateField(
        widget=DatePicker(attrs={'class': 'hasDatePicker'}),
        label=_("Departure"),
    )
    arrival = forms.DateField(
        widget=DatePicker(attrs={'class': ''}),
        label=_("Arrival"),
    )
    day_after = forms.CharField(
        required=False,
        label=_("Day after"),
    )
    form_train_departure = forms.CharField(
        required=False,
        label=_("Train departure"),
    )
    form_train_arrival = forms.CharField(
        required=False,
        label=_("Train arrival"),
    )

    def __init__(self,  *args, **kwargs):
        super(TrainDetailAdminForm, self).__init__(*args, **kwargs)
        request = get_request()
        self.hidden_fields = [key for key in request.GET.keys() if not key.startswith('_')]
        for field in self.hidden_fields:
            self.fields[field].widget = forms.widgets.HiddenInput()
        self.fields['form_train_departure'].widget.attrs['disabled'] = True
        self.fields['form_train_arrival'].widget.attrs['disabled'] = True


    def clean(self):

        try:
            instance = self.instance
        except ObjectDoesNotExist:
            instance = None

        cleaned_data = super(TrainDetailAdminForm, self).clean()
        event = cleaned_data.get("event")
        turn = cleaned_data.get("turn")
        direction = cleaned_data.get("direction")
        start_place = cleaned_data.get("start_place")
        flight = cleaned_data.get("flight")
        departure = cleaned_data.get("departure")
        arrival = cleaned_data.get("arrival")

        if event and turn and start_place and flight and departure and arrival:

            # We cant have an other flight detail with same turn, start_place, direction and flight
            transport_details = event.transport_details.filter(
                traindetail__turn=turn,
                traindetail__start_place=start_place,
                traindetail__direction=direction,
                traindetail__flight=flight,
            )
            transport_details = transport_details.exclude(id=instance.id) if instance else transport_details
            if transport_details.count():
                raise forms.ValidationError(
                    _("The same flight is already registered with same turn, start place and direction")
                )

            # We cant have transport details for same turn and start place in same time period
            transport_details = event.transport_details.filter(turn=turn, start_place=start_place)
            transport_details = transport_details.exclude(id=instance.id) if instance else transport_details
            if transport_details.filter(
                (Q(departure__range=[departure, arrival]) | Q(arrival__range=[departure, arrival]))
            ).count():
                raise forms.ValidationError(
                    _("An other transport for this turn and start place is already registered in same time period")
                )

        return cleaned_data

    class Meta:
        model = FlightDetail


class FlightDetailAdminForm(ConstrainedModelForm):
    event_travel_option = forms.ModelChoiceField(
        queryset=EventTravelOption.objects.all(), required=True,
        label=_("Event travel option"),
    )
    flight = forms.ModelChoiceField(
        queryset=Flight.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('FlightAutocomplete'),
        label=_("Flight"),
    )
    departure = forms.DateField(
        widget=DatePicker(attrs={'class': 'hasDatePicker',}),
        label=_("Departure"),
    )
    arrival = forms.DateField(
        widget=DatePicker(attrs={'class': '',}),
        label=_("Arrival"),
    )
    day_after = forms.CharField(
        required=False,
        label=_("Day after"),
    )
    form_flight_departure = forms.CharField(
        required=False,
        label=_("Flight departure"),
    )
    form_flight_arrival = forms.CharField(
        required=False,
        label=_("Flight arrival"),
    )

    def __init__(self,  *args, **kwargs):
        super(FlightDetailAdminForm, self).__init__(*args, **kwargs)
        request = get_request()
        self.hidden_fields = [key for key in request.GET.keys() if not key.startswith('_')]
        for field in self.hidden_fields:
            self.fields[field].widget = forms.widgets.HiddenInput()
        self.fields['form_flight_departure'].widget.attrs['disabled'] = True
        self.fields['form_flight_arrival'].widget.attrs['disabled'] = True

    def clean(self):

        try:
            instance = self.instance
        except ObjectDoesNotExist:
            instance = None

        cleaned_data = super(FlightDetailAdminForm, self).clean()
        event = cleaned_data.get("event")
        turn = cleaned_data.get("turn")
        direction = cleaned_data.get("direction")
        start_place = cleaned_data.get("start_place")
        flight = cleaned_data.get("flight")
        departure = cleaned_data.get("departure")
        arrival = cleaned_data.get("arrival")

        if event and turn and start_place and flight and departure and arrival:

            # We cant have an other flight detail with same turn, start_place, direction and flight
            transport_details = event.transport_details.filter(
                flightdetail__turn=turn,
                flightdetail__start_place=start_place,
                flightdetail__direction=direction,
                flightdetail__flight=flight,
            )
            transport_details = transport_details.exclude(id=instance.id) if instance else transport_details
            if transport_details.count():
                raise forms.ValidationError(
                    _("The same flight is already registered with same turn, start place and direction")
                )

            # We cant have transport details for same turn and start place in same time period
            transport_details = event.transport_details.filter(turn=turn, start_place=start_place)
            transport_details = transport_details.exclude(id=instance.id) if instance else transport_details
            if transport_details.filter(
                (Q(departure__range=[departure, arrival]) | Q(arrival__range=[departure, arrival]))
            ).count():
                raise forms.ValidationError(
                    _("An other transport for this turn and start place is already registered in same time period")
                )

        return cleaned_data

    class Meta:
        model = FlightDetail


class CustomTransportDetailAdminForm(ConstrainedModelForm):
    event_travel_option = forms.ModelChoiceField(
        queryset=EventTravelOption.objects.all(), required=True,
        label=_("Event travel option"),
    )
    customtransport = forms.CharField(
        required=True,
        label=_("description"),
    )
    departure = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker'}),
        label=_("Departure date"),
    )
    arrival = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker'}),
        label=_("Arrival date"),
    )
    day_after = forms.BooleanField(
        required=False,
        label=_("Day after"),
    )
    form_custom_transport_departure = forms.CharField(
        required=False,
        label=_("Departure place"),
    )
    form_custom_transport_arrival = forms.CharField(
        required=False,
        label=_("Arrival place"),
    )

    class Meta:
        model = CustomTransportDetail

    def clean(self):
        # CUSTOM TRANSPORT INIT
        customtransport_description = self.cleaned_data.get('customtransport')
        customtransport_instance = CustomTransport.objects.create(
            description=customtransport_description,
            departure=self.cleaned_data.get('departure'),
            arrival=self.cleaned_data.get('arrival'),
            day_after=self.cleaned_data.get('day_after'),
            departure_place=self.cleaned_data.get('form_custom_transport_departure'),
            arrival_place=self.cleaned_data.get('form_custom_transport_arrival'),
        )
        self.cleaned_data['customtransport'] = customtransport_instance

        # DATE CHECK
        departure = self.cleaned_data.get('departure')
        arrival = self.cleaned_data.get('arrival')

        if departure > arrival:
            raise forms.ValidationError(_("The arrival date must be less greater then the departure date"))

        return self.cleaned_data

    def __init__(self,  *args, **kwargs):
        instance = kwargs.get('instance', None)
        if instance:
            kwargs.update(initial={
                # 'field': 'value'
                'customtransport': instance.customtransport.description,
                'form_custom_transport_departure': instance.customtransport.departure_place,
                'form_custom_transport_arrival': instance.customtransport.arrival_place,
            })

        super(CustomTransportDetailAdminForm, self).__init__(*args, **kwargs)

        request = get_request()
        self.hidden_fields = [key for key in request.GET.keys() if not key.startswith('_')]
        for field in self.hidden_fields:
            self.fields[field].widget = forms.widgets.HiddenInput()
        # self.fields['form_custom_transport_departure'].widget.attrs['disabled'] = True
        # self.fields['form_custom_transport_arrival'].widget.attrs['disabled'] = True


class TransportAnswerAdminForm(AdminModelForm):

    invite = forms.ModelChoiceField(
            queryset=Invite.objects.all(), required=True,
            widget=autocomplete_light.ChoiceWidget('InviteAutocomplete')
        )
    transport_detail = forms.ModelChoiceField(
            queryset=TransportDetail.objects.all(), required=True,
            widget=autocomplete_light.ChoiceWidget('TransportDetailAutocomplete')
        )

    class Meta:
        model = TransportAnswer


class TrainAdminForm(AdminModelForm):
    dep_station = forms.ModelChoiceField(
        queryset=Station.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('StationAutocomplete')
    )
    arr_station = forms.ModelChoiceField(
        queryset=Station.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('StationAutocomplete')
    )
    train_type = forms.ModelChoiceField(
        queryset=TrainType.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('TrainTypeAutocomplete')
    )
    departure = forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1})
    )
    arrival= forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1})
    )
    duration= forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1}),
        required=False,
    )

    class Meta:
        model = Train


class FlightAdminForm(AdminModelForm):
    airline = forms.ModelChoiceField(
        queryset=Airline.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('AirlineAutocomplete'),
        label=_("Airline"),
    )
    dep_airport = forms.ModelChoiceField(
        queryset=Airport.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('AirportAutocomplete'),
        label=_('Departure airport'),
    )
    arr_airport = forms.ModelChoiceField(
        queryset=Airport.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('AirportAutocomplete'),
        label=_('Arrival airport'),
    )
    departure= forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1})
    )
    arrival= forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1})
    )
    duration= forms.TimeField(
        widget=TimePicker(attrs={'class': 'hasTimePicker', 'stepMinute': 1}),
        required=False,
    )

    class Meta:
        model = Flight

    def clean_number(self):
        number = self.cleaned_data.get('number')
        if not number.isdigit():
            raise forms.ValidationError(_("Number can't contains letters"))
        return number


class EventAdminForm(AdminModelForm):
    event_date_start = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )
    event_date_end = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )
    event_adhesion_deadline = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )

    def clean_name(self):
        name = self.cleaned_data['name']
        if Event.objects.filter(name=name).exclude(pk=self.instance.pk).count():
            raise forms.ValidationError(_('Event with this name already exist'))
        return name

    def clean(self):
        cleaned_data = self.cleaned_data
        if cleaned_data.get("ask_phone", False) and not cleaned_data.get("phone_question_type", False):
            raise forms.ValidationError(_("Please select a phone question type"))
        if cleaned_data.get("ask_address", False) and not cleaned_data.get("address_question_type", False):
            raise forms.ValidationError(_("Please select an address question type"))
        if cleaned_data.get("ask_frequent_flyer", False) and not cleaned_data.get("frequent_flyer_question_type", False):
            raise forms.ValidationError(_("Please select a frequent flyer question type"))
        if cleaned_data.get("enable_forwarding", False):
            if not cleaned_data.get("max_limit_forwarders", False):
                raise forms.ValidationError(_("If forwarding is enabled, you must set max limit forwarding"))
        return cleaned_data

    def clean_slug(self):
        event_slug = self.cleaned_data['slug']
        qs = Event.objects.filter(slug=event_slug)
        if self.instance:
            qs = qs.exclude(pk=self.instance.pk)

        if qs:
            raise forms.ValidationError(_('slug already present in database'))
        else:
            return event_slug

    class Meta:
        model = Event


class TurnAdminForm(AdminModelForm):

    event = forms.ModelChoiceField(
            queryset=Event.objects.all(), required=True,
            widget=autocomplete_light.ChoiceWidget('EventAutocomplete')
        )

    class Meta:
        model = Turn


class EventPlaceAdminForm(ConstrainedModelForm):
    hidden_fields = ('event',)

    # def clean_date(self):
    #     date = self.cleaned_data['date']
    #     event = self.cleaned_data['event']
    #     if not event.event_date_start.date() <= date <= event.event_date_end.date():
    #         raise forms.ValidationError('This date must be between {} and {}'.format(
    #             event.event_date_start.date(), event.event_date_end.date()
    #         ))
    #     return date

    class Meta:
        model = EventPlace


class EventTurnProxyAdminForm(ConstrainedModelForm):
    hidden_fields = ('event',)
    turn_start = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )
    turn_end = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )

    class Meta:
        model = EventTurnProxy


class EventStartPlaceAdminForm(ConstrainedModelForm):

    hidden_fields = ('event',)

    class Meta:
        model = EventStartPlace


class EventTravelOptionAdminForm(ConstrainedModelForm):

    hidden_fields = ('event',)

    class Meta:
        model = EventTravelOption


class InviteHiddenEventForm(AdminModelForm, ConstrainedModelForm):

    event = forms.ModelChoiceField(
            queryset=Event.objects.all(), required=True,
            widget=autocomplete_light.ChoiceWidget('EventAutocomplete')
        )

    contact = forms.ModelChoiceField(
            queryset=Contact.objects.all(), required=True,
            widget=autocomplete_light.ChoiceWidget('ContactAutocomplete')
        )

    class Meta:
        model = InviteProxy

    hidden_fields = ('event',)


class InviteAdminForm(forms.ModelForm):
    contact = forms.ModelChoiceField(
        queryset=Contact.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('ContactAutocomplete')
    )

    class Meta:
        model = Invite


########################################################################################################################
# CONTACTS #############################################################################################################
########################################################################################################################
class LinkToEventForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class CopyMailingListsForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class SendInviteForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class ResendInviteForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class SendFollowUpForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class SendDocumentsForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class ContactAddressFormsetForm(forms.ModelForm):

    class Meta:
        model = ContactAddress
        widgets = {
            'province': forms.Select(choices=settings.ADDRESS_ADDRESS_PROVINCE_CHOICES),
        }

    def __init__(self, *args, **kwargs):
        super(ContactAddressFormsetForm, self).__init__(*args, **kwargs)
        self.fields['address_scopes'].label = _("Address types")
        self.fields['name'].label = _("Address")
        self.fields['number'].label = _("Civic num.")
        self.fields['zip_code'].label = _("C.A.P.")
        for field in self.fields:
            self.fields[field].required = True

    def clean_zip_code(self):
        zip_code = self.cleaned_data['zip_code']
        if not all(char.isdigit() for char in zip_code):
            raise forms.ValidationError(_('Invalid zip code'))
        return zip_code

    def save(self, contact_edit, commit=True):
        from publieditor.event.utils.generic import check_differences
        m2m_fields = ["address_scopes"]
        related_map = dict(
            single_ch=['address_type'],
            single_fk={'country': 'name'},
        )
        try:
            old_instance = ContactAddress.objects.get(id=self.instance.id)
            changed_data = self.changed_data
            old_m2m_items = {
                m2m_field: ",".join(
                    sorted([related.__unicode__() for related in getattr(old_instance, m2m_field).all()])
                ) for m2m_field in m2m_fields
            }
        except ContactAddress.DoesNotExist:
            old_instance = None
            changed_data = [field for field in self.fields if field != "DELETE"]
            old_m2m_items = {m2m_field: [] for m2m_field in m2m_fields}
        instance = super(ContactAddressFormsetForm, self).save(commit=commit)
        if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
            differences = check_differences(
                old_instance, instance, changed_data, related_map, m2m_fields, old_m2m_items
            )
            contact_edit.save_address_edits(instance, differences, is_created=not bool(old_instance))
        return instance

    def delete(self, contact_edit, commit=True):
        try:
            old_instance = ContactAddress.objects.get(id=self.instance.id)
            differences = dict(
                id=old_instance.id,
                address_scopes=", ".join(sorted([scope.name for scope in old_instance.address_scopes.all()])),
                address_type=old_instance.get_address_type_display(),
                name=old_instance.name,
                number=old_instance.number,
                zip_code=old_instance.zip_code,
                country=old_instance.country.name,
                province=old_instance.province,
                locality=old_instance.locality,
            )
            if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
                contact_edit.save_address_edits(old_instance, differences=differences, is_deleted=True)
            self.instance.delete()
        except ContactAddress.DoesNotExist:
            pass


class FormsetPhoneForm(forms.ModelForm):

    class Meta:
        model = Phone

    def __init__(self, *args, **kwargs):
        super(FormsetPhoneForm, self).__init__(*args, **kwargs)
        self.fields["area_code"].required = True

    def clean_number(self):
        number = self.cleaned_data.get('number')
        if not number.isdigit():
            raise forms.ValidationError(_("Number can't contains letters"))
        return number

    def save(self, contact_edit, commit=True):
        from publieditor.event.utils.generic import check_differences
        related_map = dict(
            single_ch=['phone_type'],
            single_fk={'area_code': 'code'},
        )
        try:
            old_instance = Phone.objects.get(id=self.instance.id)
            changed_data = self.changed_data
        except Phone.DoesNotExist:
            old_instance = None
            changed_data = [field for field in self.fields if field != "DELETE"]
        instance = super(FormsetPhoneForm, self).save(commit=commit)
        if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
            differences = check_differences(old_instance, instance, changed_data, related_map=related_map)
            contact_edit.save_phone_edits(instance, differences, is_created=not bool(old_instance))
        return instance

    def delete(self, contact_edit, commit=True):
        try:
            old_instance = Phone.objects.get(id=self.instance.id)
            differences = dict(
                id=old_instance.id,
                area_code=old_instance.area_code.code,
                number=old_instance.number,
                phone_type=old_instance.get_phone_type_display()
            )
            if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
                contact_edit.save_phone_edits(old_instance, differences=differences, is_deleted=True)
            self.instance.delete()
        except Phone.DoesNotExist:
            pass


class EmailCcFormsetForm(forms.ModelForm):

    class Meta:
        model = ContactEmailCc
        fields = ["email"]

    def __init__(self, *args, **kwargs):
        super(EmailCcFormsetForm, self).__init__(*args, **kwargs)
        self.fields['email'].required = True

    def save(self, contact_edit, commit=True):
        from publieditor.event.utils.generic import check_differences
        try:
            old_instance = ContactEmailCc.objects.get(id=self.instance.id)
            changed_data = self.changed_data
        except ContactEmailCc.DoesNotExist:
            old_instance = None
            changed_data = [field for field in self.fields if field != "DELETE"]
        instance = super(EmailCcFormsetForm, self).save(commit=commit)
        differences = check_differences(old_instance, instance, changed_data)
        contact_edit.save_email_cc_edits(instance, differences, is_created=not bool(old_instance))
        return instance

    def delete(self, contact_edit, commit=True):
        try:
            old_instance = ContactEmailCc.objects.get(id=self.instance.id)
            differences = dict(
                id=old_instance.id,
                email=old_instance.email_details(),
            )
            contact_edit.save_email_cc_edits(old_instance, differences=differences, is_deleted=True)
            self.instance.delete()
        except ContactEmailCc.DoesNotExist:
            pass


class MembershipFormsetForm(forms.ModelForm):

    class Meta:
        model = Membership

    def __init__(self, *args, **kwargs):
        super(MembershipFormsetForm, self).__init__(*args, **kwargs)
        self.fields['role'].required = True

    def save(self, contact_edit, commit=True):
        from publieditor.event.utils.generic import check_differences
        related_map = dict(single_fk={'newspaper': 'name', 'role': 'name'})
        try:
            old_instance = Membership.objects.get(id=self.instance.id)
            changed_data = self.changed_data
        except Membership.DoesNotExist:
            old_instance = None
            changed_data = [field for field in self.fields if field != "DELETE"]
        instance = super(MembershipFormsetForm, self).save(commit=commit)
        differences = check_differences(old_instance, instance, changed_data, related_map=related_map)
        contact_edit.save_membership_edits(instance, differences, is_created=not bool(old_instance))
        return instance

    def delete(self, contact_edit, commit=True):
        try:
            old_instance = Membership.objects.get(id=self.instance.id)
            differences = dict(
                id=old_instance.id,
                newspaper=old_instance.newspaper.name,
            )
            if old_instance.role:
                differences['role'] = old_instance.role
            contact_edit.save_membership_edits(old_instance, differences=differences, is_deleted=True)
            self.instance.delete()
        except Membership.DoesNotExist:
            pass


class CardFormsetForm(forms.ModelForm):

    class Meta:
        model = Card

    def clean_number(self):
        number = self.cleaned_data.get('number')
        if not number.isdigit():
            raise forms.ValidationError(_("Number can't contains letters"))
        return number

    def save(self, contact_edit, commit=True):
        from publieditor.event.utils.generic import check_differences
        try:
            old_instance = Card.objects.get(id=self.instance.id)
            changed_data = self.changed_data
        except Card.DoesNotExist:
            old_instance = None
            changed_data = [field for field in self.fields if field != "DELETE"]
        related_map = dict(single_fk={'airline': 'name'})
        instance = super(CardFormsetForm, self).save(commit=commit)
        if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
            differences = check_differences(old_instance, instance, changed_data, related_map=related_map)
            contact_edit.save_card_edits(instance, differences, is_created=not bool(old_instance))
        return instance

    def delete(self, contact_edit, commit=True):
        try:
            old_instance = Card.objects.get(id=self.instance.id)
            differences = dict(
                id=old_instance.id,
                airline=old_instance.airline.name,
                number=old_instance.number,
            )
            if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
                contact_edit.save_card_edits(old_instance, differences=differences, is_deleted=True)
            self.instance.delete()
        except Card.DoesNotExist:
            pass


class ReadonlyFormsetPhoneForm(FormsetPhoneForm):

    def __init__(self, *args, **kwargs):
        super(ReadonlyFormsetPhoneForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].widget.attrs['readonly'] = True


class ReadonlyFormsetAddressForm(ContactAddressFormsetForm):

    def __init__(self, *args, **kwargs):
        super(ReadonlyFormsetAddressForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].widget.attrs['readonly'] = True


class ReadonlyFormsetCardForm(CardFormsetForm):

    def __init__(self, *args, **kwargs):
        super(ReadonlyFormsetCardForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].widget.attrs['readonly'] = True


class LoginForm(AuthenticationForm):
    username = forms.CharField(
        max_length=254, label=_("Email"), widget=forms.EmailInput
    )


class RegisterForm(forms.Form):
    email = forms.EmailField(
        required=True,
        label=_("Email")
    )
    password = forms.CharField(
        required=True, max_length=32,
        widget=forms.PasswordInput,
        label=_("Choose a password (minimum 8 chars)")
    )

    def __init__(self, *args, **kwargs):
        self.contact = kwargs.pop('contact')
        super(RegisterForm, self).__init__(*args, **kwargs)
        self.fields['email'].initial = self.contact.email
        self.fields['email'].widget.attrs['readonly'] = True
        # Crispy helper
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            Div(
                Div('email', css_class='col-lg-12 col-md-12',),
                css_class='row',
            ),
            Div(
                Div('password', css_class='col-lg-12 col-md-12',),
                css_class='row',
            ),
        )

    def clean_email(self):
        email = self.cleaned_data['email']
        if bool(User.objects.filter(Q(email__iexact=email) | Q(username__iexact=email)).count()):
            raise forms.ValidationError(_("User with provided email already exists"))
        return email

    def clean_password(self):
        password = self.cleaned_data['password']
        if len(password) < 8:
            raise forms.ValidationError(_("Password must be at least 8 characters"))
        if len(password) > 30:
            raise forms.ValidationError(_("Password must be less than 30 characters"))
        return password

    def save(self):
        email = self.cleaned_data['email']
        user = User.objects.create(
            first_name=self.contact.first_name,
            last_name=self.contact.last_name,
            username=email,
            email=email,
            password=make_password(self.cleaned_data['password']),
        )
        self.contact.user = user
        self.contact.save()
        return email, self.cleaned_data['password']


class ResetPasswordRequestForm(forms.Form):
    email = forms.EmailField(
        required=True,
        label=_("Email")
    )

    def clean_email(self):
        email = self.cleaned_data['email']
        if not bool(Contact.objects.filter(email__iexact=email).count()):
            raise forms.ValidationError(
                _("That e-mail address doesn't match any user accounts. Are you sure you've registered?")
            )
        else:
            contact = Contact.objects.filter(email__iexact=email)[0]
            if not contact.user:
                raise forms.ValidationError(
                    _("That e-mail address doesn't match any user accounts. Are you sure you've registered?")
                )
        return email

    def save(self):
        email = self.cleaned_data['email']
        contact = Contact.objects.get(email__iexact=email)
        current_site = Site.objects.get_current()
        current_site.event_password_recovery_settings.send_email(contact)


class ResetPasswordForm(forms.Form):
    new_password1 = forms.CharField(
        required=True, max_length=32,
        widget=forms.PasswordInput,
        label=_("New password")
    )
    new_password2 = forms.CharField(
        required=True, max_length=32,
        widget=forms.PasswordInput,
        label=_("New password confirmation")
    )

    def __init__(self, *args, **kwargs):
        self.contact = kwargs.pop('contact')
        super(ResetPasswordForm, self).__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super(ResetPasswordForm, self).clean()
        new_password1 = cleaned_data.get("new_password1")
        new_password2 = cleaned_data.get("new_password2")
        if new_password1 != new_password2:
            raise forms.ValidationError(_("The two password fields didn't match."))
        return cleaned_data

    def save(self):
        self.contact.user.password = make_password(self.cleaned_data['new_password1'])
        self.contact.user.save()


class ProfileForm(forms.ModelForm):
    privacy = forms.BooleanField(
        label=_(
            'Acconsento al trattamento dei miei dati personali sensibili qualora essi siano stati inseriti nel form.'
        ),
        required=True,
    )

    class Meta:
        model = Contact
        fields = settings.PUBLIEDITOR_EVENT_CONTACT_PUBLIC_EDITABLE_FIELDS
        excludes = [
            "privacy_acceptance_date", "is_test_contact", "groups", "user", "company", "fca_privacy_acceptance_date"
            "date_create", "creator"
        ]
        widgets = {
            'food_note': forms.Textarea(attrs={'rows': 3}),
            'note': forms.Textarea(attrs={'rows': 3}),
            'photo': AdminProportionImageWidget,
            'id_card_scan': AdminProportionImageWidget,
            'passport_scan': AdminProportionImageWidget,
            'license_scan': AdminProportionImageWidget,
        }

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        for field in settings.PUBLIEDITOR_EVENT_CONTACT_PUBLIC_REQUIRED_FIELDS:
            self.fields[field].required = True
        if getattr(self.instance, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD):
            self.fields['privacy'].initial = True
            self.fields['privacy'].widget.attrs['disabled'] = True
            self.fields['privacy'].required = False
        for field in ["photo", "id_card_scan", "passport_scan", "license_scan"]:
            if field in self.fields:
                self.fields[field].error_messages['invalid_image'] = _(
                    "Upload a valid image (jpeg or png). The file you uploaded was either not an image or a corrupted image."
                )
        # Crispy helper
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            Div(
                Div(
                    HTML(
                    u"<p class='pull-right' style='margin-bottom: 0'>{}</p>".format(
                        ugettext("All fields marked with an asterisk (*) are required")
                    )
                ), css_class='row',
            ), css_class='col-lg-12 col-md-12'),
            Div(
                Div(
                    HTML(
                    u"<p class='pull-right' style='margin-bottom: 0'>{}</p>".format(
                        ugettext("Uploads must be smaller than 10MB")
                    )
                ), css_class='row',
            ), css_class='col-lg-12 col-md-12'),
            Div(
                Div(
                    HTML(
                    u"<p class='pull-right' style='margin-bottom: 0'>{}</p>".format(
                        ugettext("Data will not be saved until the end of the procedure")
                    )
                ), css_class='row',
            ), css_class='col-lg-12 col-md-12'),
            Div(
                Div(
                    HTML(
                    u"<p id='date-format' class='pull-right' style='margin-bottom: 0'></p>"
                ), css_class='row',
            ), css_class='col-lg-12 col-md-12'),

            TabHolder(
                Tab(
                    _('General notes'),
                    Div(
                        Div('email', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('first_name', css_class='col-lg-6 col-md-6',),
                        Div('last_name', css_class='col-lg-6 col-md-6',),
                        css_class='row',
                    ),
                    Div(
                        Div('language', css_class='col-lg-6 col-md-6',),
                        Div('title', css_class='col-lg-6 col-md-6',),
                        css_class='row',
                    ),
                    Div(
                        Div('birth_place', css_class='col-lg-6 col-md-6',),
                        Div(Field('birthday', css_class="datepicker"), css_class='col-lg-6 col-md-6',),
                        css_class='row',
                    ),
                    Div(
                        Div('press_card_number', css_class='col-lg-6 col-md-6',),
                        Div('elenco', css_class='col-lg-6 col-md-6',),
                        css_class='row',
                    ),
                    Div(
                        Div('photo', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('smoking', css_class='col-lg-6 col-md-6',),
                        Div('food', css_class='col-lg-6 col-md-6',),
                        css_class='row',
                    ),
                    Div(
                        Div('food_note', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('height', css_class='col-lg-2 col-md-2',),
                        Div('weight', css_class='col-lg-2 col-md-2',),
                        Div('shirt_size', css_class='col-lg-2 col-md-2',),
                        Div('sweater_size', css_class='col-lg-2 col-md-2',),
                        Div('shoes_size', css_class='col-lg-2 col-md-2',),
                        css_class='row',
                    ),
                    Div(
                        Div('id_card_number', css_class='col-lg-4 col-md-4',),
                        Div(Field('id_card_issue_date', css_class="datepicker"), css_class='col-lg-4 col-md-4',),
                        Div(Field('id_card_expiration_date', css_class="datepicker"), css_class='col-lg-4 col-md-4',),
                        css_class='row',
                    ),
                    Div(
                        Div('id_card_scan', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('passport_number', css_class='col-lg-4 col-md-4',),
                        Div(Field('passport_issue_date', css_class="datepicker"), css_class='col-lg-4 col-md-4',),
                        Div(Field('passport_expiration_date', css_class="datepicker"), css_class='col-lg-4 col-md-4',),
                        css_class='row',
                    ),
                    Div(
                        Div('passport_scan', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('license_number', css_class='col-lg-3 col-md-3',),
                        Div(Field('license_issue_date', css_class="datepicker"), css_class='col-lg-3 col-md-3',),
                        Div(Field('license_expiration_date', css_class="datepicker"), css_class='col-lg-3 col-md-3',),
                        Div('license_emitted_by', css_class='col-lg-3 col-md-3',),
                        css_class='row',
                    ),
                    Div(
                        Div(Field('license_types', css_class="select2"), css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('license_scan', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(
                        Div('note', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(HTML(
                        u"""
                            <br>
                            <b>CONSENSO EX ART. 23 DEL D.LGS. N.196/2003</b><br>
                            Letta l’<a class='privacy_link'>informativa</a>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    Div(
                        Div('privacy', css_class='col-lg-12 col-md-12',),
                        css_class='row',
                    ),
                    Div(HTML(
                        u"""
                            Cliccando sul pulsante 'Continua' acconsento a che i dati personali conferiti siano
                            trattati per le finalità del Servizio secondo le modalità ivi indicate, incluso l’eventuale
                            trattamento in Paesi membri dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    Div(HTML(
                        u"<div class='ajax-formset' data-url='{}'></div>".format(reverse("event:profile_form_submit"))
                    )),
                    css_id="tab-general-info"
                ),
                Tab(
                    _("Memberships"),
                    Div(
                        Div(
                            HTML(
                                u"<p class='pull-left'><b>{}</b></p>".format(
                                    ugettext(
                                        "Please enter at least 1 publication for which you work or cooperate, \
                                         specifying your role (max. 5)"
                                    )
                                )
                            ), css_class='row',
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Formset(
                        'membership_formset', 'membership_form_helper', 'membership_tab_data_url',
                        template="bootstrap3/formset_membership.html"
                    ),
                    Div(
                        Div(
                            HTML(u"<a id='add-membership' class='btn btn-link pull-right'>{}</a>".format(ugettext("Add")))
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Div(HTML(
                        u"""
                            Letta l’<a class='privacy_link'>informativa</a>, cliccando sul pulsante 'Continua' acconsento
                            a che i dati personali conferiti siano trattati per le finalità del Servizio secondo le
                            modalità ivi indicate, incluso l’eventuale trattamento in Paesi membri
                            dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    css_id="tab-memberships"
                ),
                Tab(
                    _("Addresses"),
                    Div(
                        Div(
                            HTML(
                                u"<p class='pull-left'><b>{}</b></p>".format(
                                    ugettext("Insert at least 1 address to proceed (max. 5)")
                                )
                            ), css_class='row',
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Formset(
                        'address_formset', 'address_form_helper', 'addresses_tab_data_url',
                        template="bootstrap3/formset_address.html"
                    ),
                    Div(
                        Div(
                            HTML(u"<a id='add-address' class='btn btn-link pull-right'>{}</a>".format(ugettext("Add")))
                        ), css_class=''
                    ),
                    Div(HTML(
                        u"""
                            Letta l’<a class='privacy_link'>informativa</a>, cliccando sul pulsante 'Continua' acconsento
                            a che i dati personali conferiti siano trattati per le finalità del Servizio secondo le
                            modalità ivi indicate, incluso l’eventuale trattamento in Paesi membri
                            dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    css_id="tab-addresses",
                    css_class='col-lg-12 col-md-12'
                ),
                Tab(
                    _("Telephone numbers"),
                    Div(
                        Div(
                            HTML(
                                u"<p class='pull-left'><b>{}</b></p>".format(
                                    ugettext("Insert at least 1 phone to proceed (max. 5)")
                                )
                            ), css_class='row',
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Formset(
                        'phone_formset', 'phones_form_helper', 'phones_tab_data_url',
                        template="bootstrap3/formset_phone.html"
                    ),
                    Div(
                        Div(
                            HTML(u"<a id='add-phone' class='btn btn-link pull-right'>{}</a>".format(ugettext("Add")))
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Div(HTML(
                        u"""
                            Letta l’<a class='privacy_link'>informativa</a>, cliccando sul pulsante 'Continua' acconsento
                            a che i dati personali conferiti siano trattati per le finalità del Servizio secondo le
                            modalità ivi indicate, incluso l’eventuale trattamento in Paesi membri
                            dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    css_id="tab-phones"
                ),
                Tab(
                    _("Email CC"),
                    Div(
                        Div(
                            HTML(
                                u"<p class='pull-left'><b>{}</b></p>".format(
                                    ugettext(
                                        "If desired, you may have to receive all your communications  to \
                                        other email addresses (max. 5)"
                                    )
                                )
                            ), css_class='row',
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Formset('email_cc_formset', 'email_cc_form_helper', 'email_cc_tab_data_url'),
                    Div(
                        Div(
                            HTML(u"<a id='add-email-cc' class='btn btn-link pull-right'>{}</a>".format(ugettext("Add")))
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Div(HTML(
                        u"""
                            Letta l’<a class='privacy_link'>informativa</a>, cliccando sul pulsante 'Continua' acconsento
                            a che i dati personali conferiti siano trattati per le finalità del Servizio secondo le
                            modalità ivi indicate, incluso l’eventuale trattamento in Paesi membri
                            dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    css_id="tab-email-cc"
                ),
                Tab(
                    _("Frequent flyer"),
                    Div(
                        Div(
                            HTML(
                                u"<p class='pull-left'><b>{}</b></p>".format(
                                    ugettext("Enter the data of your airlines frequent flyer cards (max. 5)")
                                )
                            ), css_class='row',
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Formset('card_formset', 'card_formset_helper', 'cards_tab_data_url'),
                    Div(
                        Div(
                            HTML(u"<a id='add-frequent-flyer' class='btn btn-link pull-right'>{}</a>".format(ugettext("Add")))
                        ), css_class='col-lg-12 col-md-12'
                    ),
                    Div(HTML(
                        u"""
                            Letta l’<a class='privacy_link'>informativa</a>, cliccando sul pulsante 'Salva modifiche' acconsento
                            a che i dati personali conferiti siano trattati per le finalità del Servizio secondo le
                            modalità ivi indicate, incluso l’eventuale trattamento in Paesi membri
                            dell’UE o in Paesi extra UE.<br/><br/>
                        """
                    ), css_class='row col-lg-12 col-md-12'),
                    css_id="tab-frequent-flyer"
                )
            ),
        )

    def save(self, commit=True):
        from publieditor.event.utils.generic import check_differences
        m2m_fields = ["license_types"]
        related_map = dict(
            single_ch=['title', 'smoking', 'food', 'shirt_size', 'sweater_size', 'elenco'],
            single_fk={'language': 'name'},
        )
        old_instance = Contact.objects.get(id=self.instance.id)
        old_m2m_items = {
            m2m_field: ",".join(
                sorted([related.__unicode__() for related in getattr(old_instance, m2m_field).all()])
            ) for m2m_field in m2m_fields
        }
        if self.cleaned_data.get("privacy", None):
            setattr(self.instance, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD, datetime.date.today())
        instance = super(ProfileForm, self).save(commit=commit)
        changed_data = [field for field in self.changed_data if field != "privacy"]
        differences = check_differences(
            old_instance, instance, changed_data, related_map, m2m_fields, old_m2m_items
        )
        return instance, differences


class SetSessionByInviteForm(forms.Form):
    token = forms.CharField()

    def __init__(self, *args, **kwargs):
        super(SetSessionByInviteForm, self).__init__(*args, **kwargs)
        self.fields['token'].required = True
        self.fields['token'].widget.attrs['class'] = 'form-control'
        self.fields['token'].widget.attrs['placeholder'] = 'XXXXX-XXXXXXXXXXXXXXXXXXXX'

    def clean(self):
        try:
            uidb36, token = self.cleaned_data['token'].split('-')
            invite_id = base36_to_int(uidb36)
            invite = Invite.objects.get(pk=invite_id)
            if not invite.check_token(token):
                raise Exception
        except:
            raise forms.ValidationError(_('Invalid token'))
