#! -*- coding: utf-8 -*-

import re
import datetime
from django import forms
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models.query import QuerySet
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Submit, HTML, Div
from crispy_forms.bootstrap import AccordionGroup

from publieditor.event.models import (
    RegisterAnswer, ActivityAnswer, DeclineAnswer, PollAnswer, Contact, DelegatedContact,
    OwnAnswer, FreeAnswer, Phone, Turn, AreaCode,
    EventTravelOption, InviteTravelOptionAnswer
)
from publieditor.event.crispy_bootstrap import Accordion
from publieditor.event.admin_widgets import AdminProportionImageWidget
from publieditor.survey.models import Question


class QuestionForm(forms.Form):
    """
    Form which will be subclassed in questions step forms
    """
    def __init__(self, *args, **kwargs):
        super(QuestionForm, self).__init__(*args, **kwargs)
        self.question_fields = []

    def set_question_fields(self, typology):
        del self.question_fields[:]
        event = self.invite.event
        event_questions = event.questions.filter(
            enabled=True,
            question__typology=typology,
        ).order_by('ordering')
        for event_question in event_questions:
            key = "question_%d" % event_question.question.id
            field_class, widget_class = self.get_form_question_widget_class(event_question.question)
            self.fields[key] = field_class
            self.fields[key].label = event_question.question.question
            self.fields[key].empty_label = None
            self.fields[key].required = True if event_question.required else False
            self.set_field_initial(key, typology, event_question.question)
            self.fields[key].help_text = None

            if "privacy" in event_question.question.category.name.lower():
                error_msg = _("Privacy must be accepted.")
                self.fields[key].error_messages = {"required": error_msg}

            self.question_fields.append(key)

    def get_form_question_widget_class(self, question):
        """
        Return field class and widget class given qtype field
        """
        fields_widget_tuple_dict = {
            'T': [
                forms.CharField(widget=forms.TextInput), forms.TextInput
            ],
            'A': [
                forms.CharField(widget=forms.Textarea), forms.Textarea
            ],
            'S': [
                forms.ModelChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.Select
                ),
                forms.Select,
            ],
            'R': [
                forms.ModelChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.RadioSelect,
                ),
                forms.RadioSelect,
            ],
            'C': [
                forms.ModelMultipleChoiceField(
                    queryset=question.choices.order_by('ordering'),
                    widget=forms.CheckboxSelectMultiple
                ),
                forms.CheckboxSelectMultiple,
            ]
        }
        return tuple(fields_widget_tuple_dict.get(question.qtype))

    def set_field_initial(self, key, typology, question):
        """
        Set field initial given a key and a question
        """
        if typology == "R":
            question_answers = RegisterAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "A":
            question_answers = ActivityAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "D":
            question_answers = DeclineAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )
        elif typology == "P":
            question_answers = PollAnswer.objects.filter(
                invite=self.invite,
                question=question,
            )

        if question_answers.count():
            question_answer = question_answers[0]
            if question_answer:
                if question.qtype == "T":
                    self.fields[key].initial = question_answer.answer

                elif question.qtype == "A":
                    self.fields[key].initial = question_answer.answer

                elif question.qtype == "R":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    if previous_answers.count():
                        self.fields[key].initial = previous_answers[0].id

                elif question.qtype == "S":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    if previous_answers.count():
                        self.fields[key].initial = previous_answers[0].id

                elif question.qtype == "C":
                    previous_answers = question.choices.filter(choice__in=question_answer.answer.split("|"))
                    if previous_answers.count():
                        self.fields[key].initial = previous_answers

    def save(self):
        pass

    def clean_answer(self, answer):
        if isinstance(answer, QuerySet):
            answer = "|".join(val['choice'] for val in answer.values())
        return answer


class PresentationForm(forms.Form):
    """
    Form used in presentation step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop("invite")
        super(PresentationForm, self).__init__(*args, **kwargs)
        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        if not self.invite.event.is_newsletter:
            self.helper.layout = Layout(
                Div(
                    Submit('forward', _('Forward'), css_class='pull-right'),
                )
            )

    def save(self):
        if self.invite and (not self.invite.event.show_schedule and not self.invite.event.show_adhesion):
            self.invite.status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
            self.invite.save()


class ScheduleForm(forms.Form):
    """
    Form used in schedule step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(ScheduleForm, self).__init__(*args, **kwargs)
        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        if not self.invite.event.is_newsletter:
            self.helper.layout = Layout(
                Div(
                    HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                        reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                        _("Backward")
                    )),
                    Submit('forward', _('Forward'), css_class='pull-right'),
                )
            )

    def save(self):
        if not self.invite.event.show_adhesion:
            self.invite.status = settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED
            self.invite.save()


class AdhesionForm(forms.Form):
    """
    Form used in adhesion step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(AdhesionForm, self).__init__(*args, **kwargs)
        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'

        button_number = 2
        if self.invite.event.enable_delegate:
            button_number += 1

        if self.invite.event.enable_forwarding:
            button_number += 1

        bootstrap_column = str(12/button_number)

        div_list = [
            Div(
                Submit('accept', _('Accept'), css_class='adhesion-submit'),
                HTML(u"""<p>%s</p>""" % _("I intend to participate")),
                css_class="col-lg-" + bootstrap_column
            ),
            Div(
                Submit('decline', _('Decline'), css_class='adhesion-submit'),
                HTML(u"""<p>%s</p>""" % _("I do not intend to participate")),
                css_class="col-lg-" + bootstrap_column
            ),
        ]

        if self.invite.event.enable_delegate:
            div_list.append(
                Div(
                    Submit('delegate', _('Delegate'), css_class='adhesion-submit'),
                    HTML(u"""<p>%s</p>""" % _("I can not participate, but I delegate a substitute")),
                    css_class="col-lg-" + bootstrap_column
                ),
            )

        if self.invite.event.enable_forwarding:
            div_list.append(
                Div(
                    Submit('forward_to', _('Extend to'), css_class='adhesion-submit'),
                    HTML(u"""<p>%s</p>""" % _("I want forward my invite (is possible to decide your participation)")),
                    css_class="col-lg-" + bootstrap_column
                ),
            )

        self.helper.layout = Layout(*div_list)

    def save(self):
        pass


class RegisterForm(forms.ModelForm):
    """
    Form used in register step
    """
    class Meta:
        model = Contact
        exclude = [
            'date_create', 'creator', 'user',
            'title', 'first_name', 'last_name', 'email', 'status', 'ordering', 'is_test_contact',
            'language', 'privacy_acceptance_date', 'fca_privacy_acceptance_date',
        ]
        widgets = {
            'id_card_scan': AdminProportionImageWidget,
            'passport_scan': AdminProportionImageWidget,
            'license_types': forms.CheckboxSelectMultiple,
            'license_scan': AdminProportionImageWidget,
            'food_note': forms.Textarea(attrs={'rows': 3, 'cols': 15}),
        }

    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(RegisterForm, self).__init__(*args, **kwargs)
        event = self.invite.event

        # Aggiungo i campi necessari
        self.fields['privacy'] = forms.BooleanField(
            label=_('Autorizzo privacy'),
        )
        if getattr(self.invite.contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD):
            self.fields['privacy'].initial = True
            self.fields['privacy'].widget.attrs['disabled'] = True
            self.fields['privacy'].required = False

        # Invite note
        self.fields["invite_note"] = forms.CharField(
            label=_("General note"),
            widget=forms.Textarea(attrs={'rows': 3, 'cols': 15}),
            initial=self.invite.note,
            required=False,
        )

        # Set options field for available event turns
        if bool(self.invite.event.turns.count()):
            if event.turns.all().count() == 1:
                turn = event.turns.all()[0]
                self.invite.turn_preference = turn
                self.invite.save()
            else:
                available_turns = []
                for available_turn in event.turns.filter(status=1):
                    if available_turn.maximum_participants:
                        if available_turn.available_seats or available_turn == self.invite.turn_preference:
                            available_turns.append(available_turn)
                    else:
                        available_turns.append(available_turn)
                queryset = [(turn.id, turn.extended_unicode()) for turn in available_turns]
                self.fields['turn'] = forms.ChoiceField(
                    choices=queryset,
                    widget=forms.Select(),
                    label=_('Turn preference')
                )

        # Rimuovo i campi in base alla configurazione dell'evento
        # id_card
        self.id_card_fields = ["id_card_number", "id_card_issue_date", "id_card_expiration_date", "id_card_scan"]
        self.id_card_fields_required = ["id_card_number", "id_card_issue_date", "id_card_expiration_date"]
        if not event.ask_id_card:
            self.remove_fields(self.id_card_fields)
        elif event.is_id_card_required:
            for field in self.id_card_fields:
                self.fields[field]. required = True
        # passport
        self.passport_fields = ["passport_number", "passport_issue_date", "passport_expiration_date", "passport_scan"]
        self.passport_fields_required = ["passport_number", "passport_issue_date", "passport_expiration_date"]
        if not event.ask_passport:
            self.remove_fields(self.passport_fields)
        elif event.is_passport_required:
            for field in self.passport_fields:
                self.fields[field]. required = True
        # license
        license_fields = [
            "license_number", "license_issue_date", "license_expiration_date",
            "license_emitted_by", "license_types", "license_scan"
        ]
        license_fields_required = [
            "license_number", "license_issue_date", "license_expiration_date",
            "license_emitted_by", "license_types",
        ]
        if not event.ask_license:
            self.remove_fields(license_fields)
        elif event.is_license_required:
            for field in license_fields_required:
                self.fields[field]. required = True
        # press card number
        if not event.ask_press_card_number:
            self.remove_fields(["press_card_number"])
        elif event.is_press_card_number_required:
            self.fields["press_card_number"]. required = True
        # clothing
        clothing_fields = ["shirt_size", "sweater_size", "shoes_size"]
        if not event.ask_clothing:
            self.remove_fields(clothing_fields)
        elif event.is_clothing_required:
            for field in clothing_fields:
                self.fields[field]. required = True
        # body measurements
        body_measurements_fields = ["height", "weight"]
        if not event.ask_body_measurements:
            self.remove_fields(body_measurements_fields)
        elif event.is_clothing_required:
            for field in body_measurements_fields:
                self.fields[field]. required = True
        # ski
        ski_fields = ["ski", "ski_level", "ski_equipment", "ski_boots_size"]
        if not event.ask_ski:
            self.remove_fields(ski_fields)
        elif event.is_ski_required:
            for field in ski_fields:
                self.fields[field]. required = True
        # ask_food_hotel_preferences
        food_hotel_fields = ["smoking", "food", "food_note"]
        if not event.ask_food_hotel_preferences:
            self.remove_fields(food_hotel_fields)
        elif event.is_food_hotel_preferences_required:
            for field in food_hotel_fields:
                self.fields[field]. required = True

        # ask_birthday
        birthday_fields = ["birthday"]
        if not event.ask_birthday:
            self.remove_fields(birthday_fields)
        elif event.is_birthday_required:
            for field in birthday_fields:
                self.fields[field].required = True

        # ask_birth_place
        birth_place_fields = ["birth_place"]
        if not event.ask_birth_place:
            self.remove_fields(birth_place_fields)
        elif event.is_birth_place_required:
            for field in birth_place_fields:
                self.fields[field].required = True

        # ask_ssn
        ssn_fields = ["ssn"]
        if not event.ask_ssn:
            self.remove_fields(ssn_fields)
        elif event.is_ssn_required:
            for field in ssn_fields:
                self.fields[field].required = True

    def remove_fields(self, fields):
        for field in fields:
            del self.fields[field]

    def save(self, commit=True):
        from publieditor.event.utils.generic import check_differences
        contact = self.invite.contact
        m2m_fields = dict()
        if "license_types" in self.cleaned_data:
            m2m_fields["license_types"] = self.cleaned_data["license_types"]

        related_map = dict(
            single_ch=['title', 'smoking', 'food', 'shirt_size', 'sweater_size', 'elenco'],
            single_fk={'language': 'name'},
        )
        old_instance = Contact.objects.get(id=self.instance.id)
        old_m2m_items = {
            m2m_field: ",".join(
                sorted([related.__unicode__() for related in getattr(old_instance, m2m_field).all()])
            ) for m2m_field in m2m_fields.keys()
        }
        instance = super(RegisterForm, self).save(commit=False)
        if self.cleaned_data.get("privacy", None) and not \
                getattr(self.invite.contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD):
            setattr(instance, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD, datetime.date.today())
        if self.cleaned_data.get("turn", "") != "":
            self.invite.turn_preference = Turn.objects.get(id=self.cleaned_data["turn"].strip())
        if self.cleaned_data.get("invite_note", None) != "":
            self.invite.note = self.cleaned_data["invite_note"].strip()
        self.invite.save()
        if commit:
            instance.save()

            # M2M adjustment
            for m2m_name, m2m_values in m2m_fields.items():
                getattr(instance, m2m_name).clear()
                for m2m_item in m2m_values:
                    getattr(instance, m2m_name).add(m2m_item)

        model_field_names = [field.name for field in Contact._meta.fields]
        changed_data = [field for field in self.changed_data if field in model_field_names]
        differences = check_differences(
            old_instance, instance, changed_data, related_map, m2m_fields, old_m2m_items
        )
        return instance, differences

    def clean_ssn(self):
        ssn = self.cleaned_data['ssn']
        if not self.invite.event.is_ssn_required:
            if len(ssn) == 0:
                return ssn
        if len(ssn) != 16:
            raise forms.ValidationError(_('Insert a valid ssn'))
        return ssn

    def clean(self):
        cleaned_data = super(RegisterForm, self).clean()

        if self.invite.event.is_id_card_or_passport_required:
            # id or card required
            id_card_fields_filled = []
            for key in self.id_card_fields_required:
                if key in cleaned_data:
                    id_card_fields_filled.append(bool(self.cleaned_data[key]))
                else:
                    id_card_fields_filled.append(False)

            passport_fields_filled = []
            for key in self.passport_fields_required:
                if key in cleaned_data:
                    passport_fields_filled.append(bool(self.cleaned_data[key]))
                else:
                    passport_fields_filled.append(False)

            if not(all(id_card_fields_filled) or all(passport_fields_filled)):
                raise forms.ValidationError(_('You must enter the complete data of at least one of identification document, document scanning is optional'))
        return cleaned_data


class TransportForm(forms.Form):
    """
    Form used in transport step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(TransportForm, self).__init__(*args, **kwargs)
        event = self.invite.event
        accordion = Accordion()

        # Gestione trasporti turno
        if bool(self.invite.event.turns.count()):
            invite_travel_option_preference = self.invite.get_event_travel_option_preference()
            qs = event.travel_options.all()
            if self.invite.turn_preference:
                qs = qs.filter(turn=self.invite.turn_preference)
            for event_travel_option in qs:
                field_key = "%s" % event_travel_option.id
                initial = False
                active = False
                if invite_travel_option_preference and invite_travel_option_preference.event_travel_option == event_travel_option:
                    active = True
                    initial = field_key
                self.fields[field_key] = forms.ChoiceField(
                    choices=[(event_travel_option.id, mark_safe(event_travel_option.get_render_transport_group()))],
                    required=False,
                    widget=forms.RadioSelect(),
                    label="",
                    initial=initial
                )
                accordion.fields.append(
                    AccordionGroup(
                        event_travel_option.name, field_key, active=active
                    )
                )

        # Gestione mezzi propri
        if event.enable_own:
            initial = bool(self.invite.own_answers.all().count())
            active = True if initial else False
            self.fields["own"] = forms.BooleanField(
                label=_("I'll come by myself"),
                required=False,
                initial=initial,
            )
            accordion.fields.append(
                AccordionGroup(_("Own transport"), "own", active=active)
            )

        if event.enable_free:
            initial = None
            free_answers = self.invite.free_answers.all()
            active = bool(free_answers.count())
            if free_answers.count():
                initial = free_answers[0].answer
            self.fields["free"] = forms.CharField(
                label="",
                widget=forms.Textarea,
                required=False,
                initial=initial,
            )
            accordion.fields.append(
                AccordionGroup(_("Other, please specify"), "free", active=active)
            )

        # HELPER
        self.helper = FormHelper()
        self.helper.layout = Layout(accordion)
        self.helper.layout.fields.append(
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def clean(self):
        cleaned_data = super(TransportForm, self).clean()
        has_values = False
        for key, value in cleaned_data.iteritems():
            if isinstance(value, bool) and value:
                has_values = True
                break
            elif isinstance(value, (str, unicode)) and value.strip() != "":
                has_values = True
                break
        if not has_values:
            raise forms.ValidationError(
                _("Please fill a travel option")
            )
        return cleaned_data

    def save(self):
        event = self.invite.event

        # Delete previous answers
        self.invite.own_answers.all().delete()
        self.invite.free_answers.all().delete()
        self.invite.transport_answers.all().delete()
        self.invite.travel_option_answers.all().delete()

        # If enabled and checked save OwnAnswer
        if event.enable_own and self.cleaned_data["own"]:
            own_answer = OwnAnswer(invite=self.invite)
            own_answer.save()

        # else if enabled and checked save FreeAnswer
        elif event.enable_free and self.cleaned_data["free"]:
            free_answer = FreeAnswer(
                invite=self.invite,
                answer=self.cleaned_data["free"].strip(),
            )
            free_answer.save()

        else:
            travel_option_fields = [key for key in self.cleaned_data.keys() if key not in ["own", "free"]]
            for field in travel_option_fields:
                if self.cleaned_data[field]:
                    event_travel_option = EventTravelOption.objects.get(id=self.cleaned_data[field])
                    invite_travel_option_answer = InviteTravelOptionAnswer(
                        invite=self.invite, event_travel_option=event_travel_option
                    )
                    invite_travel_option_answer.save()
                    break


class ActivityForm(QuestionForm):
    """
    Form used in activity step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(ActivityForm, self).__init__(*args, **kwargs)
        # Altre informazioni
        fieldset = Fieldset(
            _("Activities information"),

        )
        self.set_question_fields("A")
        fieldset.fields.extend(self.question_fields)
        # Helper
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-5'
        self.helper.field_class = 'col-lg-5'
        self.helper.layout = Layout(
            fieldset,
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def save(self):
        self.invite.activity_answers.all().delete()
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question_id = int(key.split("_")[-1])
            question = Question.objects.get(id=question_id)
            event_question = self.invite.event.questions.get(question=question)
            answer = self.clean_answer(self.cleaned_data[key])
            if answer and answer != "":
                ActivityAnswer(
                    invite=self.invite, question=question, answer=answer, ordering=event_question.ordering
                ).save()


class SummaryForm(forms.Form):
    """
    Form used in summary step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(SummaryForm, self).__init__(*args, **kwargs)
        # Helper
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        self.helper.layout = Layout(
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def save(self):
        pass


class DocumentsForm(forms.Form):
    """
    Form used in documents step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(DocumentsForm, self).__init__(*args, **kwargs)
        # Helper
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        self.helper.layout = Layout(
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def save(self):
        pass


class DeclineForm(QuestionForm):
    """
    Form used in decline step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(DeclineForm, self).__init__(*args, **kwargs)
        self.set_question_fields("D")
        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-vertical'
        decline_fieldset = Fieldset(
            _("Feedback"),
        )
        decline_fieldset.fields.extend(self.fields.keys())
        self.helper.layout = Layout(
            decline_fieldset,
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def save(self):
        # Delete all old answers
        self.invite.delete_all_answers()
        # Save new answers
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question_id = int(key.split("_")[-1])
            question = Question.objects.get(id=question_id)
            event_question = self.invite.event.questions.get(question=question)
            answer = self.clean_answer(self.cleaned_data[key])
            if answer and answer != "":
                DeclineAnswer(
                    invite=self.invite, question=question, answer=answer, ordering=event_question.ordering
                ).save()


class DelegateForm(forms.Form):
    """
    Form used in delegate step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(DelegateForm, self).__init__(*args, **kwargs)
        self.fields['title'] = forms.ChoiceField(
            label=_("Title"),
            choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES,
        )
        self.fields['first_name'] = forms.CharField(
            label=_("First name"),
            required=True,
        )
        self.fields['last_name'] = forms.CharField(
            label=_("Last name"),
            required=True,
        )
        self.fields['email'] = forms.EmailField(
            label=_("Email"),
            required=True,
        )
        self.fields['phone_number'] = forms.CharField(
            label=_("Phone number"),
            required=True,
        )

        # Recupero informazioni già compilate se esistenti
        delegated_invites = self.invite.delegated_invites.all()
        if delegated_invites.exists():
            delegated_invite = delegated_invites[0]
            self.fields["title"].initial = delegated_invite.title
            self.fields["first_name"].initial = delegated_invite.first_name
            self.fields["last_name"].initial = delegated_invite.last_name
            self.fields["email"].initial = delegated_invite.email
            self.fields["phone_number"].initial = delegated_invite.phone_number

        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        self.helper.layout = Layout(
            Fieldset(
                _("Delegated informations"), 'title', 'first_name', 'last_name', 'email', 'phone_number',
            ),
            Div(
                HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                    reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                    _("Backward")
                )),
                Submit('forward', _('Forward'), css_class='pull-right text-right'),
            )
        )

    def clean_email(self):
        email = self.cleaned_data.get('email').strip()
        contact = self.invite.contact
        if email == contact.email:
            raise forms.ValidationError(_("You can't delegate yourself"))
        return email

    def save(self):
        title = self.cleaned_data["title"]
        email = self.cleaned_data["email"].strip()
        first_name = self.cleaned_data["first_name"].strip()
        last_name = self.cleaned_data["last_name"].strip()
        phone_number = self.cleaned_data["phone_number"].strip()
        # Delete all old answers
        self.invite.delete_all_answers()
        # Recupero contatto se esistente, altrimenti lo creo
        contact, contact_created = Contact.objects.get_or_create(
            email=email,
            defaults={
                'title': title,
                'first_name': first_name,
                'last_name': last_name,
                'language': self.invite.contact.language,
            }
        )
        # Creazione telefono contatto se non già esistente
        match_phones = contact.phones.filter(number__iexact=phone_number)
        if not match_phones.exists():
            new_contact_phone = Phone(contact=contact, number=phone_number)
            new_contact_phone.save()
        # Creazione DelegatedContact
        self.invite.delegated_invites.all().delete()
        delegated_contact = DelegatedContact(
            invited=contact,
            invite=self.invite,
            email=email,
            title=title,
            first_name=first_name,
            last_name=last_name,
            phone_number=phone_number,
        )
        delegated_contact.save()
        # Checks if delegated contact data differs from already contact ones
        contact_differs = False
        if not contact_created:
            for field in ["title", "first_name", "last_name"]:
                delegated_contact_attr = getattr(delegated_contact, field)
                contact_attr = getattr(contact, field)
                if delegated_contact_attr != contact_attr:
                    contact_differs = True
        return contact_differs


class ForwardToForm(forms.Form):
    """
    Form used in forward_to step
    """

    title = forms.ChoiceField(
        choices=settings.PUBLIEDITOR_CONTACT_TITLE_CHOICES,
    )
    first_name = forms.CharField(
        label=_('First name'),
    )
    last_name = forms.CharField(
        label=_('Last name'),
    )
    email = forms.EmailField(
        label=_('email'),
    )
    phone_number = forms.CharField(
        label=_('Number'),
    )
    area_code = forms.ModelChoiceField(
        label=_('Area Code'),
        queryset=AreaCode.objects.all(),
    )
    phone_type = forms.ChoiceField(
        label=_('Type'),
        choices=settings.CONTACT_PHONE_PHONE_TYPE_CHOICES,
    )

    def clean_email(self):
        email = self.cleaned_data['email']
        if email in self.main_invite.event.invites.all().values_list('contact__email', flat=True):
            raise forms.ValidationError(_('This contact is already present in the invited list'))
        return email

    def clean_phone_number(self):
        phone_number = self.cleaned_data['phone_number']
        if len(phone_number) < 6:
            raise forms.ValidationError(_('Insert a valid phone number'))
        return phone_number

    def save(self):
        from publieditor.event.util import forward_invite

        invite_forwarded, created = forward_invite(
            main_invite=self.main_invite,
            fw_email=self.cleaned_data['email'],
            fw_first_name=self.cleaned_data['first_name'],
            fw_last_name=self.cleaned_data['last_name'],
        )
        phone, created = Phone.objects.get_or_create(
            contact=invite_forwarded.contact,
            area_code=self.cleaned_data['area_code'],
            phone_type=self.cleaned_data['phone_type'],
            number=self.cleaned_data['phone_number'],
            default=True,
        )
        invite_forwarded.send_forward_approval()

BaseForwardToFormset = formset_factory(ForwardToForm, extra=0)


class ForwardToFormset(BaseForwardToFormset):
    """
    self.main_invite is initialized with this class because in django 1.6 BaseFormSet.get_form_kwargs doesn't exists
    """

    def __init__(self, *args, **kwargs):
        self.main_invite = kwargs.pop('main_invite')
        super(ForwardToFormset, self).__init__(*args, **kwargs)

    def _construct_form(self, *args, **kwargs):
        form = super(ForwardToFormset, self)._construct_form(*args, **kwargs)
        setattr(form, 'main_invite', self.main_invite)
        return form


class GreetingsForm(forms.Form):
    """
    Form used in greetings step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        super(GreetingsForm, self).__init__(*args, **kwargs)
        # HELPER
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-2'
        self.helper.field_class = 'col-lg-8'
        if not self.invite.event.is_expired():
            self.helper.layout = Layout(
                Div(
                    HTML(u"<a href='%s' class='btn btn-primary pull-left'>%s</a>" % (
                        reverse("event:publieditor_event_my_invite", kwargs={'step': self.invite.previous_step}),
                        _("Backward")
                    )),
                    # HTML(u"<a href='javascript:window.close()' class='btn btn-primary pull-right text-right'>%s</a>" % (
                    #     _("Close browser")
                    # )),
                    # Submit('forward', _('Forward'), css_class='pull-right text-right'),
                )
        )

    def save(self):
        pass


class PollForm(QuestionForm):
    """
    Form used in poll step
    """
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.request = kwargs.pop('request')
        super(PollForm, self).__init__(*args, **kwargs)
        # Altre informazioni
        fieldset = Fieldset(
            _("Poll"),
        )
        self.set_question_fields("P")
        fieldset.fields.extend(self.question_fields)
        # Helper
        self.helper = FormHelper()
        # self.helper.form_class = 'form-horizontal'
        # self.helper.label_class = 'col-lg-4'
        # self.helper.field_class = 'col-lg-4'
        self.helper.layout = Layout(
            fieldset,
            Div(
                Submit('submit', _('Submit'), css_class='pull-right text-right'),
            )
        )

    def save(self):
        self.invite.poll_answers.all().delete()
        for key in [k for k in self.cleaned_data if k.startswith("question_")]:
            question_id = int(key.split("_")[-1])
            question = Question.objects.get(id=question_id)
            answer = self.clean_answer(self.cleaned_data[key])
            if answer and answer != "":
                PollAnswer(
                    invite=self.invite, question=question, answer=answer
                ).save()
        self.invite.status = settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE
        self.invite.save()
