# coding: utf-8

import datetime

from django import forms
from django.conf import settings
from django.contrib.auth import authenticate, models as auth_models
from django.utils.translation import ugettext_lazy as _
from jmb.address.models import Address
from jmb.core.forms import DatePicker
from jmb.l10n.models import Country
from publieditor.event.models import VehicleModel, Invite, EventPlace, License, Vehicle, ContactAddress


class TriumphRegistrationFormStep1(forms.Form):
    first_name = forms.CharField(max_length=255, required=True, label=_('first name'))
    last_name = forms.CharField(max_length=255, required=True, label=_('last_name'))
    email = forms.EmailField(required=True, label=_('email'), error_messages={'invalid': _('Enter a valid email address')})
    mobile = forms.RegexField(
        regex=r'^\+?1?\d{9,15}$',
        error_message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
        label=_('mobile')
    )
    favourite_vehicle_model = forms.ModelChoiceField(
        queryset=VehicleModel.objects.none(),
        required=False,
    )
    favourite_event_place = forms.ModelChoiceField(
        queryset=EventPlace.objects.none(),
        required=True,
    )
    privacy = forms.BooleanField(required=False)

    def __init__(self, *args, **kwargs):
        self.event = kwargs.pop('event')
        super(TriumphRegistrationFormStep1, self).__init__(*args, **kwargs)
        self.fields['favourite_vehicle_model'].queryset = self.event.vehicle_models.all()
        self.fields['favourite_event_place'].queryset = self.event.places.filter(date__gte=datetime.date.today())

    def clean_privacy(self):
        privacy = self.cleaned_data['privacy']
        if not privacy:
            raise forms.ValidationError(_('To proceed with registration you must accept the privacy policy'))
        return privacy


class SearchInviteForm(forms.Form):
    invite_id = forms.CharField(max_length=255, required=False, label=_('reservation code'))
    first_name = forms.CharField(max_length=255, required=False, label=_('first name'))
    last_name = forms.CharField(max_length=255, required=False, label=_('last_name'))
    email = forms.EmailField(required=False, label=_('email'),
                             error_messages={'invalid': _('Enter a valid email address')})

    def clean(self):
        keys = self.cleaned_data.keys()
        if len(self.cleaned_data.keys()) > 0:
            for key in keys:
                if self.cleaned_data[key]:
                    return self.cleaned_data
        raise forms.ValidationError(_('Insert at least one field in search form'))


class TriumphCustomerRegistration(forms.Form):
    first_name = forms.CharField(max_length=255, required=True, label=_('first name'))
    last_name = forms.CharField(max_length=255, required=True, label=_('last_name'))
    email = forms.EmailField(required=True, label=_('email'),
                             error_messages={'invalid': _('Enter a valid email address')})
    mobile = forms.RegexField(
        regex=r'^\+?1?\d{9,15}$',
        error_message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
        label=_('mobile')
    )
    privacy = forms.BooleanField(required=False)
    favourite_vehicle_model = forms.ModelChoiceField(
        queryset=VehicleModel.objects.none(),
        required=False,
    )
    favourite_event_place = forms.ModelChoiceField(
        queryset=EventPlace.objects.none(),
        required=False,
    )
    address_type = forms.ChoiceField(
        choices=settings.ADDRESS_ADDRESS_TYPE_CHOICES,
        required=True,
    )
    address_name = forms.CharField(max_length=255, required=True, label=_('address name'))
    address_number = forms.CharField(max_length=10, required=True, label=_('address number'))
    address_locality = forms.CharField(max_length=100, required=True, label=_('address locality'))
    address_zip_code = forms.IntegerField(required=True, label=_('address cap'))
    address_province = forms.ChoiceField(
        choices=settings.ADDRESS_ADDRESS_PROVINCE_CHOICES,
        required=True,
    )
    address_country = forms.ModelChoiceField(
        queryset=Country.objects.all(),
        required=True,
    )
    birthday = forms.DateField(required=True, widget=DatePicker(attrs={'class': 'hasDatePicker'}))
    birth_place = forms.CharField(max_length=255, required=True, label=_('birthday place'))
    license_number = forms.CharField(max_length=255, required=True, label=_('license number'))
    license_emitted_by = forms.CharField(max_length=255, required=True, label=_('license emitted by'))
    license_issue_date = forms.DateField(required=True, widget=DatePicker(attrs={'class': 'hasDatePicker'}))
    license_expiration_date = forms.DateField(required=True, widget=DatePicker(attrs={'class': 'hasDatePicker'}))

    license_type = forms.ModelMultipleChoiceField(
        queryset=License.objects.all().values_list('pk', 'name'),
        required=True,
    )

    assigned_vehicle = forms.ModelChoiceField(
        queryset=Vehicle.objects.all(),
        required=True,
    )
    event_real_place = forms.ModelChoiceField(
        queryset=EventPlace.objects.none(),
        required=True,
    )

    def __init__(self, *args, **kwargs):
        self.invite_id = kwargs.pop('invite_id')
        self.event = kwargs.pop('event')
        self.hostess_event_place_ids = kwargs.pop('hostess_event_place_ids')
        super(TriumphCustomerRegistration, self).__init__(*args, **kwargs)

        self.fields['favourite_vehicle_model'].queryset = self.event.vehicle_models.all()
        self.fields['favourite_event_place'].queryset = self.event.places.filter(date__gte=datetime.date.today())
        self.fields['assigned_vehicle'].queryset = Vehicle.objects.filter(
            vehicle_model__vehicle_models_event_list__event=self.event,
            event_place_vehicles__id__in=self.hostess_event_place_ids
        ).order_by('vehicle_model__name', 'licence_plate').distinct()
        self.fields['event_real_place'].queryset = EventPlace.objects.filter(
            event=self.event,
            id__in=self.hostess_event_place_ids
        )
        self.fields['license_type'].choices = License.objects.all().values_list('pk', 'name')
        self.fields['address_country'].initial = 108
        if self.invite_id:
            invite = Invite.objects.get(pk=self.invite_id)

            self.fields['first_name'].initial = invite.contact.first_name
            self.fields['last_name'].initial = invite.contact.last_name
            self.fields['email'].initial = invite.contact.email
            self.fields['mobile'].initial = invite.contact.get_mobile_phone().number
            if invite.favourite_vehicle_model:
                self.fields['favourite_vehicle_model'].initial = invite.favourite_vehicle_model.id
            if invite.event_place:
                self.fields['favourite_event_place'].initial = invite.event_place.id
            self.fields['privacy'].initial = True

            address = ContactAddress.objects.filter(contact=invite.contact, address_scopes__id=5)
            if address:
                self.fields['address_type'].initial = address[0].address_type
                self.fields['address_name'].initial = address[0].name
                self.fields['address_number'].initial = address[0].number
                self.fields['address_locality'].initial = address[0].locality
                self.fields['address_zip_code'].initial = address[0].zip_code
                self.fields['address_province'].initial = address[0].province
                self.fields['address_country'].initial = address[0].country.id
            if invite.contact.birthday:
                self.fields['birthday'].initial = invite.contact.birthday.strftime("%d/%m/%Y")
            if invite.contact.birth_place:
                self.fields['birth_place'].initial = invite.contact.birth_place
            if invite.contact.license_number:
                self.fields['license_number'].initial = invite.contact.license_number
            if invite.contact.license_emitted_by:
                self.fields['license_emitted_by'].initial = invite.contact.license_emitted_by
            if invite.contact.license_issue_date:
                self.fields['license_issue_date'].initial = invite.contact.license_issue_date.strftime("%d/%m/%Y")
            if invite.contact.license_expiration_date:
                self.fields['license_expiration_date'].initial = invite.contact.license_expiration_date.strftime("%d/%m/%Y")
            if invite.contact.license_types.all():
                self.fields['license_type'].initial = invite.contact.license_types.all().values_list('id', flat=True)
            if invite.assigned_vehicle:
                self.fields['assigned_vehicle'].initial = invite.assigned_vehicle.id
            if invite.assigned_place:
                self.fields['event_real_place'].initial = invite.assigned_place.id


class TrumphNewVehicleForm(forms.ModelForm):

    class Meta:
        model = Vehicle
        fields = ['vehicle_model', 'vin', 'licence_plate', 'color']

    def __init__(self, *args, **kwargs):
        self.hostess_event_place_ids = kwargs.pop('hostess_event_place_ids')
        super(TrumphNewVehicleForm, self).__init__(*args, **kwargs)

    def clean(self):
        vehicles = Vehicle.objects.filter(licence_plate__iexact=self.cleaned_data['licence_plate'])
        if vehicles:
            vehicle = vehicles[0]
            if vehicle.vin != self.cleaned_data['vin']:
                raise forms.ValidationError(_('License Plate already assigned (vin {})').format(vehicle.vin))
            if vehicle.vehicle_model.id != self.cleaned_data['vehicle_model'].id:
                raise forms.ValidationError(_('License Plate already assigned (model {})').format(vehicle.vehicle_model.name))
            if vehicle.color != self.cleaned_data['color']:
                raise forms.ValidationError(_('License Plate already assigned (color {})').format(vehicle.color))
            if set(vehicle.event_place_vehicles.values_list('id', flat=True)) == set(self.hostess_event_place_ids):
                raise forms.ValidationError(_('Vehicle already assigned to event place'))
        return self.cleaned_data


class HostessLoginForm(forms.Form):
    username = forms.CharField(max_length=255, required=True, label=_('username'))
    password = forms.CharField(widget=forms.PasswordInput, max_length=255, required=True, label=_('password'))

    def __init__(self, *args, **kwargs):
        self.user = None
        super(HostessLoginForm, self).__init__(*args, **kwargs)

    def clean(self):
        username = self.cleaned_data['username']
        password = self.cleaned_data['password']
        user = authenticate(username=username, password=password)

        if not user:
            raise forms.ValidationError(_('Login failure, unknown username or bad password'))
        self.user = user

        if not user.is_active:
            raise forms.ValidationError(_('User not yet active'))

        hostess_group = auth_models.Group.objects.get(name=settings.PUBLIEDITOR_AUTH_GROUPS_NAME['hostess'])
        if not user.groups.filter(name=hostess_group.name).exists():
            raise forms.ValidationError(_('User does not have permission to access this section'))

        if not user.event_place_hostesses.all():
            raise forms.ValidationError(_('User has no assigned event. Please contact the site administration.'))
        return super(HostessLoginForm, self).clean()
