#! -*- coding: utf-8 -*-

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from publieditor.event.public_forms import (
    PresentationForm, ScheduleForm, AdhesionForm, RegisterForm, TransportForm, ActivityForm, SummaryForm, DocumentsForm,
    DeclineForm, DelegateForm, GreetingsForm
)


def get_form(invite, step, request, post=None, files=None):
    forms_dict = dict(
        presentation=PresentationForm,
        schedule=ScheduleForm,
        adhesion=AdhesionForm,
        register=RegisterForm,
        transport=TransportForm,
        activity=ActivityForm,
        summary=SummaryForm,
        documents=DocumentsForm,
        decline=DeclineForm,
        delegate=DelegateForm,
        greetings=GreetingsForm,
    )
    if step == "register":
        form = forms_dict[step](post, files, invite=invite, request=request, instance=invite.contact)
    else:
        form = forms_dict[step](post, files, invite=invite, request=request)
    return form


def get_extra_context(invite, step, request, post=None, files=None):
    from django.forms.models import inlineformset_factory
    from publieditor.event.models import Contact, Phone
    extra_context = dict()
    if step == "register":
        contact = invite.contact
        extra_context["contact_details"] = "%s %s (%s)" % (
            contact.get_title_display(), contact.full_name, contact.email
        )
        phone_formset  = inlineformset_factory(Contact, Phone, fields=('phone_type','number'), extra=0, can_delete=False)
        extra_context["phone_formset"] = phone_formset(post, files, instance=invite.contact)
    elif step == "adhesion":
        adhesion_status_codes = [
            settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
            settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
            settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
        ]
        can_edit = True
        if invite.status in adhesion_status_codes:
            can_edit = False
            if request.GET.get("edit", None):
                can_edit = True
        if not can_edit:
            status_msgs = {
                settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED: _("to accept"),
                settings.PUBLIEDITOR_EVENT_INVITE_DECLINED: _("to decline"),
                settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED: _("to delegate"),
            }
            invite_decisions = invite.decision_answers.filter(status=invite.status).order_by('-date_create')
            date_msg = "" if not invite_decisions else "in data %s" % invite_decisions[0].date_create.strftime(
                " %d/%m/%Y alle %H:%M"
            )
            body = _(u"""
            <p>You already chose %s the invitation %s.<br>
            If you wish to modify your preference click
            <a href="%s?edit=True">here</a>.</p>
            """) % (
                status_msgs[invite.status],
                date_msg,
                reverse("event:publieditor_event_my_invite", kwargs={'step': 'adhesion'})
            )
            extra_context["body"] = body
        extra_context["can_edit"] = can_edit
    return extra_context


def get_context_breadcrumbs(invite):
    breadcrumbs = []
    event = invite.event
    # Presentation
    if event.show_presentation:
        breadcrumbs.append({"step": "presentation", "description": _("presentation")})
    # Schedule
    if event.show_schedule:
        breadcrumbs.append({"step": "schedule", "description": _("schedule")})
    # Adhesion
    if event.show_adhesion:
        breadcrumbs.append({"step": "adhesion", "description": _("adhesion")})
    # Register
    if event.show_register and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
        breadcrumbs.append({"step": "register", "description": _("register")})
    # Transport
    if event.show_transport and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
        breadcrumbs.append({"step": "transport", "description": _("transport")})
    # Activity
    if event.show_activity and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
        breadcrumbs.append({"step": "activity", "description": _("activity")})
    # Summary
    if event.show_summary and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
        breadcrumbs.append({"step": "summary", "description": _("summary")})
    # Documents
    if event.show_documents and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED:
        breadcrumbs.append({"step": "documents", "description": _("documents")})
    # Decline
    if event.show_decline and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_DECLINED:
        breadcrumbs.append({"step": "decline", "description": _("decline")})
    # Delegate
    if event.show_delegate and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED:
        breadcrumbs.append({"step": "delegate", "description": _("delegate")})
    # Forward to
    if event.show_forwarding and invite.status == settings.PUBLIEDITOR_EVENT_INVITE_FORWARD:
        breadcrumbs.append({"step": "forward_to", "description": _("extend to")})
    # Greetings
    greetings_enabled_status = [
        settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
        settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
        settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED,
    ]
    if event.show_greetings and invite.status in greetings_enabled_status:
        breadcrumbs.append({"step": "greetings", "description": _("greetings")})
    return breadcrumbs


def get_span_form_field(label_text, span_text):
    return """
        <div class="form-group">
            <label class="control-label col-lg-2">%s</label>
            <div class="controls col-lg-8">
                <span>%s</span>
            </div>
        </div>
    """ % (label_text, span_text)
