# -*- coding: utf-8 -*-
"""
helper module for jmb.mailerstat
"""


import logging
from datetime import datetime, timedelta

from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.db.models import F
from django.conf import settings

from publieditor.event import models
from jmb.mailer.models import MailingStatus, Link, LinkStatus


logger = logging.getLogger('eventstat')


def update_statistics(duration=None):
    start_script = datetime.now()

    try:
        last_st = models.LastMailingStatusProcessed.objects.get()
        last_mailingstatus = last_st.last_processed_id
        is_running = last_st.is_script_running
    except ObjectDoesNotExist:
        last_st = LastMailingStatusProcessed.objects.create(is_running=False, last_processed_id=0)
        is_running = False
        last_mailingstatus = 0

    if is_running:
        return
    else:
        is_running = True
        last_st.is_script_running = True
        last_st.save()

        offset = 0
        c_rows_to_process = MailingStatus.objects.filter(
            id__gt=last_mailingstatus
        ).prefetch_related('email_object').order_by('id').count()

        if c_rows_to_process > 0:
            while offset < c_rows_to_process:
                try:
                    last_mailingstatus = models.LastMailingStatusProcessed.objects.get().last_processed_id
                except ObjectDoesNotExist:
                    last_mailingstatus = 0

                rows_to_process = MailingStatus.objects.filter(
                    id__gt=last_mailingstatus
                ).prefetch_related('email_object').order_by('id')[0:5000]

                status_order = [
                    settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND_IN_TEST,
                    settings.MAILER_MAILINGSTATUS_STATUS_SENT_TEST,
                    settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND,
                    settings.MAILER_MAILINGSTATUS_STATUS_SENT,
                    settings.MAILER_MAILINGSTATUS_STATUS_BOUNCED,
                    # settings.MAILER_MAILINGSTATUS_STATUS_ERROR,
                    # settings.MAILER_MAILINGSTATUS_STATUS_INVALID,
                    settings.MAILER_MAILINGSTATUS_STATUS_OPENED,
                    settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE,
                    settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED,
                ]
                for row in rows_to_process:
                    email = row.email_object.email_address
                    status = row.status
                    date_invite = row.email_object.date_create
                    try:
                        event = row.email_object.inviteemailobject.invite.event
                    except Exception, e:
                        logger.warning("{} MailingStatus:{}, EmailObject:{}. {}".format(
                            datetime.now().strftime("%d-%m-%Y %H:%M"),
                            row.id,
                            row.email_object.id,
                            e
                        ))
                        event = None
                    if event:
                        start_date = date_invite.replace(hour=0, minute=0, second=0, microsecond=0)
                        end_date = date_invite.replace(hour=23, minute=59, second=59, microsecond=999999)

                        with transaction.atomic():
                            init_event_statistics(status_order, event, status, start_date)
                            update_event_status_date_statistic(row.id, event, status, start_date, end_date, email)
                            update_event_status_statistic(row.id, event, status, email)
                            if row.status == settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED:
                                link_status_objects = LinkStatus.objects.filter(
                                    mailing_status__id=row.id
                                ).prefetch_related('link').order_by('id')
                                for link_status in link_status_objects:
                                    link_url = link_status.link.url
                                    link_title = link_status.link.title
                                    link_email = link_status.link.email_object.email_address
                                    update_event_link_date_statistic(row.id, event, start_date, end_date,
                                                                     link_title, link_url, link_email, link_status.id)
                                    update_event_link_statistic(row.id, event, link_title,
                                                                link_url, link_email, link_status.id)

                    try:
                        last_p = models.LastMailingStatusProcessed.objects.get()
                        logger.debug("{}  Update table {}".format(
                            datetime.now().strftime("%d-%m-%Y %H:%M"),
                            models.LastMailingStatusProcessed._meta.db_table
                        ))
                        last_p.last_processed_id = row.id
                        last_p.save()
                    except ObjectDoesNotExist:
                        logger.debug("{}  Insert new entry in table {}".format(
                            datetime.now().strftime("%d-%m-%Y %H:%M"),
                            models.LastMailingStatusProcessed._meta.db_table
                        ))
                        rec = models.LastMailingStatusProcessed(last_processed_id=row.id)
                        rec.save()
                    if duration:
                        if datetime.now() >= start_script + timedelta(minutes=int(duration)):
                            break
                offset += 5000
        last_st = models.LastMailingStatusProcessed.objects.get()
        last_st.is_script_running = False
        last_st.save()
        logger.info("{}  End process".format(
            datetime.now().strftime("%d-%m-%Y %H:%M"))
        )
    return


@transaction.atomic
def init_event_statistics(status_order, event, status, start_date):
    for st in status_order:
        models.EventStatusDateStatistic.objects.get_or_create(
            event=event,
            status=status,
            date_invite=start_date,
            defaults={'total': 0, 'unique_email_address': 0}
        )
        models.EventStatusStatistic.objects.get_or_create(
            event=event,
            status=st,
            defaults={'total': 0, 'unique_email_address': 0}
        )


@transaction.atomic
def update_event_status_date_statistic(mailingstatus_id, event, status, start_date, end_date, email):
    try:
        row_statistic = models.EventStatusDateStatistic.objects.get(
            event=event,
            status=status,
            date_invite=start_date,
        )
        row_statistic.total = F('total') + 1

        """verify email unique """
        rows_processed = MailingStatus.objects.filter(
            id__lt=mailingstatus_id,
            status=status,
            email_object__inviteemailobject__invite__event=event,
            email_object__email_address=email,
            email_object__date_create__range=(start_date, end_date)
        ).count()
        if rows_processed == 0:
            row_statistic.unique_email_address = F('unique_email_address') + 1

    except ObjectDoesNotExist:
        row_statistic = models.EventStatusDateStatistic(
            event=event,
            status=status,
            date_invite=start_date,
            total=1,
            unique_email_address=1)

    try:
        row_statistic.save()
    except Exception, e:
        logger.critical("{}  Error when insert data. {} ".format(
            datetime.now().strftime("%d-%m-%Y %H:%M"), e
        ))
        raise Exception


@transaction.atomic
def update_event_status_statistic(mailingstatus_id, event, status, email):
    try:
        row_statistic = models.EventStatusStatistic.objects.get(
            event=event,
            status=status,
        )
        row_statistic.total = F('total') + 1

        """verify email unique """
        rows_processed = MailingStatus.objects.filter(
            id__lt=mailingstatus_id,
            status=status,
            email_object__inviteemailobject__invite__event_id=event,
            email_object__email_address=email,
        ).count()
        if rows_processed == 0:
            row_statistic.unique_email_address = F('unique_email_address') + 1

    except ObjectDoesNotExist:
        row_statistic = models.EventStatusStatistic(
            event=event,
            status=status,
            total=1,
            unique_email_address=1)

    try:
        row_statistic.save()
    except Exception, e:
        logger.critical("{}  Error when insert data ".format(
            datetime.now().strftime("%d-%m-%Y %H:%M"),
        ))
        raise Exception


@transaction.atomic
def update_event_link_date_statistic(mailingstatus_id, event, start_date, end_date,
                                     link_title, link_url, link_email, link_status_id):
    try:
        row_statistic = models.EventLinkDateStatistic.objects.get(
            event=event,
            link_title=link_title,
            link_url=link_url,
            date_invite=start_date,
        )
        row_statistic.total = F('total') + 1

        """verify email unique """

        rows_processed = LinkStatus.objects.filter(
            id__lt=link_status_id,
            mailing_status__id__lt=mailingstatus_id,
            mailing_status__email_object__email_address=link_email,
            link__url=link_url,
            link__title=link_title,
            mailing_status__email_object__date_create__range=(start_date, end_date)
        ).count()
        if rows_processed == 0:
            row_statistic.unique_email_address = F('unique_email_address') + 1

    except ObjectDoesNotExist:
        row_statistic = models.EventLinkDateStatistic(
            event=event,
            link_title=link_title,
            link_url=link_url,
            date_invite=start_date,
            total=1,
            unique_email_address=1)

    try:
        row_statistic.save()
    except Exception, e:
        logger.critical("{}  Error when insert data ".format(
            datetime.now().strftime("%d-%m-%Y %H:%M"),
        ))
        raise Exception


@transaction.atomic
def update_event_link_statistic(mailingstatus_id, event, link_title, link_url, link_email, link_status_id):
    try:
        row_statistic = models.EventLinkStatistic.objects.get(
            event=event,
            link_title=link_title,
            link_url=link_url,
        )
        row_statistic.total = F('total') + 1

        """verify email unique """

        rows_processed = LinkStatus.objects.filter(
            id__lt=link_status_id,
            mailing_status__id__lt=mailingstatus_id,
            mailing_status__email_object__email_address=link_email,
            link__url=link_url,
            link__title=link_title,
        ).count()
        if rows_processed == 0:
            row_statistic.unique_email_address = F('unique_email_address') + 1

    except ObjectDoesNotExist:
        row_statistic = models.EventLinkStatistic(
            event=event,
            link_title=link_title,
            link_url=link_url,
            total=1,
            unique_email_address=1)

    try:
        row_statistic.save()
    except Exception, e:
        logger.critical("{}  Error when insert data ".format(
            datetime.now().strftime("%d-%m-%Y %H:%M"),
        ))
        raise Exception


def deleteContents():
    models.EventStatusStatistic.objects.all().delete()
    models.EventStatusDateStatistic.objects.all().delete()
    models.EventLinkDateStatistic.objects.all().delete()
    models.EventLinkStatistic.objects.all().delete()
    models.LastMailingStatusProcessed.objects.all().delete()