#! -*- coding: utf-8 -*-

import base64, json
import datetime

import os
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.contrib.auth import login, logout, authenticate
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import PasswordChangeForm
from django.contrib.sites.models import Site
from django.forms.models import inlineformset_factory
from django.http import HttpResponse, Http404
from django.http import HttpResponseRedirect
from django.views.generic import TemplateView, ListView
from django.views.generic.edit import FormView
from django.views.generic import View
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext, Context, Template
from django.template.defaultfilters import slugify
from django.utils.http import base36_to_int
from django.utils.translation import ugettext_lazy as _
from django.utils import translation as django_translation
from django.utils.decorators import method_decorator

from jmb.core.utils.appypod import AppyPodTemplate

from publieditor.event.models import (
    Invite, Contact, Turn, Phone, ContactEmailCc, Membership, ContactAddress, Card, Event,
    ContactGroup, EventPlace, License, Vehicle)
from publieditor.event.triumph.forms import HostessLoginForm, TrumphNewVehicleForm
from publieditor.event.utils.my_invite import get_form, get_extra_context, get_context_breadcrumbs
from publieditor.event.forms import (
    RegisterForm, LoginForm, ProfileForm, FormsetPhoneForm, EmailCcFormsetForm,
    MembershipFormsetForm, ContactAddressFormsetForm, ResetPasswordRequestForm, ResetPasswordForm,
    CardFormsetForm
)
from publieditor.event.form_helpers import (
    FormsetPhoneFormHelper, FormsetEmailCcFormHelper, MembershipFormHelper, FormsetContactAddressFormHelper,
    CardFormHelper
)
from publieditor.event.formsets import RequiredFormSet


def set_session_by_invite(request, step, uidb36, token):
    landing_page = '/'
    try:
        invite_id = base36_to_int(uidb36)
        invite = Invite.objects.get(pk=invite_id)
        event, contact = invite.event, invite.contact
        contact_privacy_acceptance_date = getattr(contact, settings.PUBLIEDITOR_EVENT_CONTACT_PRIVACY_FIELD)

        if settings.PUBLIEDITOR_EVENT_CONTACT_USER_ENABLED:
            if not event.is_newsletter and not event.is_privacy and not contact_privacy_acceptance_date:
                return redirect(invite.get_register_and_redirect_url())

        if not invite.check_token(token):
            return redirect(landing_page)
        elif invite.managed is True:
            return redirect(landing_page)
        elif invite.status == settings.PUBLIEDITOR_EVENT_INVITE_POLL_DONE:
            return redirect(landing_page)
        request.session['publieditor_event_invite_id'] = invite.id
        request.session['publieditor_event_invite_dispatched'] = True
        return redirect(reverse("event:publieditor_event_my_invite", kwargs={'step': step}))
    except Exception, e:
        pass
    return redirect(landing_page)


def my_invite(request, step):
    landing_page = '/'
    if 'publieditor_event_invite_id' not in request.session:
        messages.add_message(request, messages.INFO, _(''))
        return redirect(landing_page)
    # Recupero invito
    invite_id = request.session['publieditor_event_invite_id']
    invite = Invite.objects.get(pk=invite_id)
    event = invite.event
    # Se l'invito non è ancora stato accettato, declinato o delegato faccio in modo che vada nel primo step
    if invite.status not in [
        settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
        settings.PUBLIEDITOR_EVENT_INVITE_DECLINED,
        settings.PUBLIEDITOR_EVENT_INVITE_DELEGATED
    ] and getattr(request.session, 'publieditor_event_invite_dispatched', None):
        del request.session['publieditor_event_invite_dispatched']
        step = invite.steps[0]["step"]
    # Imposto nuovo step
    if "old_step" not in request.session:
        request.session["old_step"] = step
    required_step = step if step != request.session["old_step"] else request.session["old_step"]
    invite.update_step(required_step)
    # Gestione form
    form = get_form(invite, required_step, request)
    if request.method == 'POST':
        if step != request.session["old_step"]:
            request.session["old_step"] = step
        form = get_form(invite, required_step, request, post=request.POST, files=request.FILES)
        if form.is_forward():
            if form.is_valid():
                form.save()
                return redirect(reverse("event:publieditor_event_my_invite", kwargs={'step': invite.next_step}))
        else:
            return redirect(reverse("event:publieditor_event_my_invite", kwargs={'step': invite.previous_step}))
    # Creazione context
    context = dict(
        invite=invite,
        form=form,
        required_step=required_step,
        breadcrumbs=get_context_breadcrumbs(invite),
    )
    extra_context = get_extra_context(invite, required_step, request)
    context.update(extra_context)

    return render_to_response(
        event.landing_page_template,
        context,
        context_instance=RequestContext(request)
    )


def session_expired(request):
    context = dict()
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_SESSION_EXPIRED_TEMPLATE,
        context,
        context_instance=RequestContext(request)
    )


def event_deadline_expired(request, invite_id):
    event = None
    invite = None
    deadline_expired_message = ''
    landing_page_template = 'empty.html'
    try:
        invite = Invite.objects.get(pk=invite_id)
        event = invite.event
        deadline_expired_message = event.dissuasion_message_translations.get(
            language__code=event.default_language.code
        ).deadline_expired_message
        deadline_expired_message = Template(deadline_expired_message).render(
            Context({
                'event': event
            })
        )
        landing_page_template = event.landing_page_template
    except Exception as e:
        pass
    context = {
        'deadline_expired_message': deadline_expired_message,
        'event': event,
        'invite': invite,
        'landing_page_template': landing_page_template
    }
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_DEADLINE_EXPIRED_TEMPLATE,
        context,
        context_instance=RequestContext(request)
    )


def get_invite_pdf(request, uidb36, token):
    from django.templatetags.static import static
    from publieditor.event.utils.appy import get_photo_appy, convert_to_pdf

    invite_id = base36_to_int(uidb36)
    invite = Invite.objects.get(pk=invite_id)
    if invite.check_token(token):
        invite = Invite.objects.get(pk=invite_id)
        event = invite.event
        contact = invite.contact
        translation = invite.get_step_translation("translations")
        schedule_translation = invite.get_step_translation("schedule_translations")
        template_file = settings.PUBLIEDITOR_EVENT_PERSONALIZED_PROGRAM_TEMPLATES[event.landing_page_template]
        header_image = get_photo_appy(translation.header_image_pdf.path) if translation.header_image else None
        skin_image = get_photo_appy(os.path.join(settings.STATIC_ROOT, settings.PUBLIEDITOR_EVENT_SKIN_IMAGES[event.landing_page_template])) if \
            settings.PUBLIEDITOR_EVENT_SKIN_IMAGES[event.landing_page_template] else None

        if translation.disclaimer_image and translation.disclaimer_in_personalized_program:
            disclaimer_image = get_photo_appy(translation.disclaimer_image.path)
        else:
            disclaimer_image = None

        if translation.disclaimer_text and translation.disclaimer_in_personalized_program:
            disclaimer_text = translation.disclaimer_text
        else:
            disclaimer_text = None

        django_translation.activate(schedule_translation.language.code)
        schedule_title = _('Schedule')

        context = dict(
            title=translation.title,
            invite=invite,
            answers_tables=invite.get_all_answers(is_html=False),
            event=event,
            contact=contact,
            header_image=header_image,
            schedule_translation=schedule_translation,
            skin_image=skin_image,
            disclaimer_image=disclaimer_image,
            disclaimer_text=disclaimer_text,
            schedule_title=unicode(schedule_title),
        )

        output = AppyPodTemplate(template_file, context)

        if settings.POD_CONVERTER:
            filename = '/tmp/{}.odt'.format(slugify(translation.title))
            output.save_as(filename, file_type='odt')
            file_to_convert = open(filename, 'rb')
            result = convert_to_pdf(file_to_convert, output_format_orig='pdf')
        else:
            result = output.render()

        response = HttpResponse(result, mimetype='application/pdf')
        response['Content-Disposition'] = u'inline;filename={}.pdf'.format(slugify(translation.title))
        return response


def poll(request, uidb36, token):
    from publieditor.event.public_forms import PollForm

    landing_page = '/'

    invite_id = base36_to_int(uidb36)
    invite = Invite.objects.get(pk=invite_id)
    event = invite.event
    if not invite.check_token(token):
        return redirect(landing_page)
    request.session['publieditor_event_invite_id'] = invite.id
    form = PollForm(invite=invite, request=request)
    if request.method == 'POST':
        form = PollForm(request.POST, invite=invite, request=request)
        if form.is_valid():
            form.save()
            return redirect(reverse("event:publieditor_event_my_invite", kwargs={'step': 'greetings'}))
    context = dict(
        invite=invite, form=form,
        required_step="poll",
        breadcrumbs=None,
    )
    return render_to_response(
        settings.PUBLIEDITOR_EVENT_POLL_PAGE_TEMPLATES[event.landing_page_template],
        context,
        context_instance=RequestContext(request)
    )
    return redirect(landing_page)


def event_ics_export(request, uidb36, token):
    invite_id = base36_to_int(uidb36)
    invite = Invite.objects.get(pk=invite_id)
    if invite.check_token(token):
        invite = Invite.objects.get(pk=invite_id)
        cal = invite.get_ics_calendar()
        response = HttpResponse(cal.to_ical(), mimetype="text/calendar")
        response['Content-Disposition'] = 'attachment; filename={}.ics'.format(invite.event.slug)
        return response


def set_test_contact(request, contact_id):
    try:
        contact = Contact.objects.get(id=contact_id)
        contact.toggle_test()
    except Exception:
        pass
    return redirect(reverse("admin:event_contact_changelist"))


def set_turn_status(request, turn_id):
    try:
        turn = Turn.objects.get(id=turn_id)
        turn.toggle_status()
    except Exception:
        pass
    return redirect(reverse("admin:event_turn_changelist"))


@login_required
def read_invite_pass(request, uidb36, token):
    invite_id = base36_to_int(uidb36)
    try:
        invite = Invite.objects.get(pk=invite_id)
        if invite.check_token(token):
            invite.has_partecipated = True
            invite.save()
            return HttpResponse(_('Partecipation saved'))
        return HttpResponse(_('Token is invalid!'))
    except Invite.DoesNotExist:
        return HttpResponse(_('Matching invite not found'))


def get_invite_pass(request, uidb36, token):
    from reportlab.lib.units import cm
    from reportlab.pdfgen import canvas
    from django.http import HttpResponse
    from reportlab.graphics.shapes import Drawing
    from reportlab.graphics.barcode.qr import QrCodeWidget
    from reportlab.graphics import renderPDF
    from django.template.defaultfilters import date as _date
    from django.utils.translation import ugettext as _

    invite_id = base36_to_int(uidb36)
    invite = Invite.objects.get(pk=invite_id)
    if invite.check_token(token):
        response = HttpResponse(mimetype='application/pdf')
        painting = canvas.Canvas(response)
        url = "%s://%s%s" % (settings.SITE_PROTOCOL, settings.SITE_DOMAIN, invite.get_absolute_url())
        drawing = Drawing()
        name_string = "%s: %s" % (
            _("Invited contact"),
            invite.contact.full_name
        )
        event_string = "%s: %s" % (
            _("Event"),
            invite.event.name,
        )
        place_string = "%s: %s" % (
            _("Place"),
            invite.event.place,
        )
        date_string = "%s: %s - %s" % (
            _("Date"),
            _date(invite.event.event_date_start, "d/m/Y"),
            _date(invite.event.event_date_end, "d/m/Y"),
        ) if invite.event.event_date_start.date() != invite.event.event_date_end.date() else \
            "%s: %s" % (
                _("Date"),
                _date(invite.event.event_date_start, "d/m/Y"),
            )
        # Divido il foglio in 4 rettangoli
        painting.rect(1*cm, 1*cm, 19*cm, 27.7*cm, fill=0)
        painting.line(10.5*cm, 1*cm, 10.5*cm, 28.7*cm)
        painting.line(1*cm, 14.85*cm, 20*cm, 14.85*cm)
        # Inserisco il testo da visualizzare
        painting.setFont("Courier-BoldOblique", 30)
        painting.drawString(2.8*cm, 26*cm, "RSVP PASS")
        painting.setFont("Helvetica", 12)
        painting.drawString(3*cm, 24*cm, name_string)
        painting.drawString(3*cm, 23*cm, event_string)
        painting.drawString(3*cm, 22*cm, place_string)
        painting.drawString(3*cm, 21*cm, date_string)
        # Inserisco qr code
        qrcode_widget = QrCodeWidget(url)
        drawing.add(qrcode_widget)
        renderPDF.draw(drawing, painting, 4*cm, 16.5*cm)
        # Renderizzo
        painting.showPage()
        painting.save()
        response['Content-Disposition'] = 'attachment; filename="RSVP-PASS.pdf"'
        return response
    return HttpResponse(_('Matching invite not found'))


##################################################
# User views #####################################
##################################################
class LoginView(FormView):
    template_name = 'event/user/login.html'
    form_class = LoginForm

    def get(self, request, *args, **kwargs):
        user = self.request.user
        if user.is_authenticated() and hasattr(user, 'contact'):
            return redirect(reverse("event:profile"))
        return super(LoginView, self).get(request, args, kwargs)

    def get_form(self, form_class):
        return form_class(data=self.request.POST or None)

    def get_success_url(self):
        return reverse("event:profile")

    def get_context_data(self, **kwargs):
        context = super(LoginView, self).get_context_data(**kwargs)
        current_site = Site.objects.get_current()
        context['is_reset_password_enabled'] = hasattr(current_site, 'event_password_recovery_settings')
        return context

    def form_valid(self, form):
        user = form.get_user()
        login(self.request, user)
        return super(LoginView, self).form_valid(form)


class LoginRedirectView(LoginView):

    def dispatch(self, request, *args, **kwargs):
        self.next_page = kwargs['next_page']
        self.invite = Invite.objects.get(pk=base36_to_int(kwargs['uidb36']))
        return super(LoginRedirectView, self).dispatch(request, args, kwargs)

    def get(self, request, *args, **kwargs):
        return super(LoginRedirectView, self).get(request, args, kwargs)

    def get_context_data(self, **kwargs):
        context = super(LoginView, self).get_context_data(**kwargs)
        current_site = Site.objects.get_current()
        context['is_reset_password_enabled'] = hasattr(current_site, 'event_password_recovery_settings')
        return context

    def form_valid(self, form):
        user = form.get_user()
        if user == self.invite.contact.user:
            login(self.request, user)
            return redirect(base64.b64decode(self.next_page))
        messages.error(self.request, _('Access denied, did you register?'))
        return redirect(self.request.path)


class RegisterView(FormView):
    template_name = 'event/user/register.html'
    form_class = RegisterForm
    contact = None

    def dispatch(self, request, *args, **kwargs):
        invite_id = kwargs.get('invite_id', None)
        self.invite_id = invite_id
        uidb36, token = kwargs['uidb36'], kwargs['token']
        contact = Contact.objects.get(pk=base36_to_int(uidb36))
        if contact.check_token(token) and not contact.user:
            self.contact = contact
            return super(RegisterView, self).dispatch(request, args, kwargs)
        if not request.user.is_authenticated():
            messages.error(self.request, _("Invalid link"))
        return redirect(reverse('event:login'))

    def get_form(self, form_class):
        return form_class(self.request.POST or None, contact=self.contact)

    def form_valid(self, form):
        username, password = form.save()
        user = authenticate(username=username, password=password)
        login(self.request, user)
        messages.success(self.request, _("Password saved"))
        self.request.session["registration_redirect_invite_id"] = self.invite_id
        return redirect(reverse('event:profile'))


class ChangePasswordView(FormView):
    template_name = 'event/user/change_password.html'
    form_class = PasswordChangeForm

    def get(self, request, *args, **kwargs):
        user = self.request.user
        if not user.is_authenticated() or not hasattr(user, 'contact'):
            return redirect(reverse("event:login"))
        return super(ChangePasswordView, self).get(request, args, kwargs)

    def get_form(self, form_class):
        user = self.request.user
        return form_class(user, self.request.POST or None)

    def get_success_url(self):
        return reverse("event:profile")

    def form_valid(self, form):
        form.save()
        messages.success(self.request, _('Password changed'))
        return super(ChangePasswordView, self).form_valid(form)


class ResetPasswordRequestView(FormView):
    template_name = 'event/user/reset_password_request.html'
    form_class = ResetPasswordRequestForm

    def get_context_data(self, **kwargs):
        context = super(ResetPasswordRequestView, self).get_context_data(**kwargs)
        context['email_to'] = "pressoffice@fcagroup.com"
        context['email_subject'] = _("Richiesta registrazione R.S.V.P.")
        return context

    def get(self, request, *args, **kwargs):
        current_site = Site.objects.get_current()
        if not hasattr(current_site, 'event_password_recovery_settings'):
            return redirect(reverse("event:login"))
        return super(ResetPasswordRequestView, self).get(request, args, kwargs)

    def get_success_url(self):
        return reverse("event:login")

    def form_valid(self, form):
        form.save()
        messages.success(
            self.request,
            _('An email was sent to your email address, follow instructions to reset your password.')
        )
        return super(ResetPasswordRequestView, self).form_valid(form)


class ResetPasswordView(FormView):
    template_name = 'event/user/reset_password.html'
    form_class = ResetPasswordForm

    def dispatch(self, request, *args, **kwargs):
        uidb36, token = kwargs['uidb36'], kwargs['token']
        contact = Contact.objects.get(pk=base36_to_int(uidb36))
        if contact.check_token(token) and contact.user:
            self.contact = contact
            return super(ResetPasswordView, self).dispatch(request, args, kwargs)
        messages.error(self.request, _("Invalid link"))
        return redirect(reverse('event:login'))

    def get_form(self, form_class):
        return form_class(self.request.POST or None, contact=self.contact)

    def get_context_data(self, **kwargs):
        context = super(ResetPasswordView, self).get_context_data(**kwargs)
        context['contact'] = self.contact
        return context

    def get_success_url(self):
        return reverse("event:login")

    def form_valid(self, form):
        form.save()
        messages.success(self.request, _('Password changed'))
        return super(ResetPasswordView, self).form_valid(form)


class ProfileView(FormView):
    template_name = 'event/user/profile.html'
    form_class = ProfileForm

    def get_context_data(self, **kwargs):
        contact = self.request.user.contact
        context = super(ProfileView, self).get_context_data(**kwargs)
        context['available_fields'] = settings.PUBLIEDITOR_EVENT_CONTACT_PUBLIC_EDITABLE_FIELDS
        # Formset address
        address_formset = inlineformset_factory(
            Contact, ContactAddress,
            form=ContactAddressFormsetForm,
            formset=RequiredFormSet,
            fields=(
                'address_scopes', 'address_type', 'name', 'number', 'zip_code', 'locality', 'province', 'country'
            ),
            extra=5, can_delete=True, max_num=5,
        )
        context['address_formset'] = address_formset(
            self.request.POST or None,
            self.request.FILES or None,
            instance=contact
        )
        context['address_form_helper'] = FormsetContactAddressFormHelper
        context["addresses_tab_data_url"] = reverse("event:profile_addresses_formset_submit")
        # Formset phone
        phone_formset = inlineformset_factory(
            Contact, Phone,
            form=FormsetPhoneForm,
            formset=RequiredFormSet,
            fields=('phone_type', 'area_code', 'number'),
            extra=5, can_delete=True, max_num=5,
        )
        context['phone_formset'] = phone_formset(
            self.request.POST or None,
            self.request.FILES or None,
            instance=contact
        )
        context['phones_form_helper'] = FormsetPhoneFormHelper
        context["phones_tab_data_url"] = reverse("event:profile_phones_formset_submit")
        # Formset email_cc
        email_cc_formset = inlineformset_factory(
            Contact, ContactEmailCc,
            form=EmailCcFormsetForm,
            fields=('email',),
            extra=5, can_delete=True, max_num=5,
            fk_name="contact",
        )
        context['email_cc_formset'] = email_cc_formset(
            self.request.POST or None,
            self.request.FILES or None,
            instance=contact
        )
        context['email_cc_form_helper'] = FormsetEmailCcFormHelper
        context["email_cc_tab_data_url"] = reverse("event:profile_emails_formset_submit")
        # Formset membership
        membership_formset = inlineformset_factory(
            Contact, Membership,
            form=MembershipFormsetForm,
            formset=RequiredFormSet,
            fields=('newspaper', 'role'),
            extra=5, can_delete=True, max_num=5
        )
        context['membership_formset'] = membership_formset(
            self.request.POST or None,
            self.request.FILES or None,
            instance=contact
        )
        context['membership_form_helper'] = MembershipFormHelper
        context["membership_tab_data_url"] = reverse("event:profile_memberships_formset_submit")
        # Formset card
        card_formset = inlineformset_factory(
            Contact, Card,
            form=CardFormsetForm,
            fields=('airline', 'number', 'expiration_date'),
            extra=5, can_delete=True, max_num=5
        )
        context['card_formset'] = card_formset(
            self.request.POST or None,
            self.request.FILES or None,
            instance=contact
        )
        context['card_formset_helper'] = CardFormHelper
        context["cards_tab_data_url"] = reverse("event:profile_cards_formset_submit")
        context['language'] = self.request.LANGUAGE_CODE
        context['language_format'] = u"{} '{}'".format(
            _("Dates must be in format"), "mm/dd/yyyy" if self.request.LANGUAGE_CODE == 'en' else "gg/mm/aaaa"
        )
        return context

    def get(self, request, *args, **kwargs):
        user = self.request.user
        if not user.is_authenticated() or not hasattr(user, 'contact'):
            return redirect(reverse("event:login"))
        return super(ProfileView, self).get(request, args, kwargs)

    def get_form(self, form_class):
        user = self.request.user
        return form_class(
            self.request.POST or None,
            self.request.FILES or None,
            instance=user.contact
        )

    def form_valid(self, form):
        from publieditor.event.models import ContactEditDiff
        context = self.get_context_data()
        address_formset = context['address_formset']
        phone_formset = context['phone_formset']
        email_cc_formset = context['email_cc_formset']
        membership_formset = context['membership_formset']
        card_formset = context['card_formset']
        if address_formset.is_valid() and \
                phone_formset.is_valid() and \
                email_cc_formset.is_valid() and \
                membership_formset.is_valid() and \
                card_formset.is_valid():
            instance, contact_diff = form.save()
            contact_edit = ContactEditDiff.objects.create(contact=instance)
            contact_edit.save_contact_edits(contact_diff)
            for formset in [address_formset, phone_formset, membership_formset, email_cc_formset, card_formset]:
                formset.instance = instance
                for form in formset:
                    if form in formset.deleted_forms:
                        form.delete(contact_edit=contact_edit)
                    elif form.has_changed():
                        form.save(contact_edit=contact_edit)
            contact_edit.send()
            contact_edit.delete()
            current_site = Site.objects.get_current()
            if hasattr(current_site, "event_profile_done_email_settings"):
                current_site.event_profile_done_email_settings.send_email(instance)
            if self.request.session.get("registration_redirect_invite_id", None):
                invite = Invite.objects.get(id=self.request.session['registration_redirect_invite_id'])
                del self.request.session['registration_redirect_invite_id']
                messages.success(self.request, _('Registration successfully completed!'))
                return redirect(
                    reverse(
                        "event:publieditor_event_set_session_by_invite",
                        kwargs={'step': invite.step, 'uidb36': invite.make_base36(), 'token': invite.make_token()}
                    )
                )
            messages.success(self.request, _('Profile updated'))
            return redirect(reverse("event:profile_done"))
        messages.error(self.request, _('Please correct form errors'))
        return self.render_to_response(self.get_context_data(form=form))


class ProfileFormAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        multiple_fields = ["license_types"]
        post = dict()
        for field in ProfileForm.base_fields:
            post[field] = request.POST.getlist(field) if field in multiple_fields else request.POST.get(field)
        form = ProfileForm(post, instance=contact)
        data = dict(success=form.is_valid(), errors=form.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class ProfileMembershipsFormsetAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        formset_class = inlineformset_factory(
            Contact, Membership,
            form=MembershipFormsetForm,
            formset=RequiredFormSet,
            fields=('newspaper', 'role'),
            extra=5, can_delete=True, max_num=5
        )
        formset = formset_class(request.POST, instance=contact)
        data = dict(success=formset.is_valid(), errors=formset.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class ProfileAddressesFormsetAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        formset_class = inlineformset_factory(
            Contact, ContactAddress,
            form=ContactAddressFormsetForm,
            formset=RequiredFormSet,
            fields=(
                'address_scopes', 'address_type', 'name', 'number', 'zip_code', 'locality', 'province', 'country'
            ),
            extra=5, can_delete=True, max_num=5,
        )
        formset = formset_class(request.POST, instance=contact)
        data = dict(success=formset.is_valid(), errors=formset.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class ProfilePhonesFormsetAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        formset_class = inlineformset_factory(
            Contact, Phone,
            form=FormsetPhoneForm,
            formset=RequiredFormSet,
            fields=('phone_type', 'area_code', 'number'),
            extra=5, can_delete=True, max_num=5,
        )
        formset = formset_class(request.POST, instance=contact)
        data = dict(success=formset.is_valid(), errors=formset.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class ProfileEmailsFormsetAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        formset_class = inlineformset_factory(
            Contact, ContactEmailCc,
            form=EmailCcFormsetForm,
            fields=('email',),
            extra=5, can_delete=True, max_num=5,
            fk_name="contact",
        )
        formset = formset_class(request.POST, instance=contact)
        data = dict(success=formset.is_valid(), errors=formset.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class ProfileCardsFormsetAjaxSubmit(View):

    def post(self, request, *args, **kwargs):
        contact = request.user.contact
        formset_class = inlineformset_factory(
            Contact, Card,
            form=CardFormsetForm,
            fields=('airline', 'number', 'expiration_date'),
            extra=5, can_delete=True, max_num=5
        )
        formset = formset_class(request.POST, instance=contact)
        data = dict(success=formset.is_valid(), errors=formset.errors)
        return HttpResponse(json.dumps(data), mimetype='application/json')


class EventHomePageView(FormView):

    def __init__(self, **kwargs):
        self.event = None
        self.contact = None
        self.invite = None
        self.contact_already_exist = None
        super(EventHomePageView, self).__init__(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        event_slug = kwargs.get('slug')
        try:
            self.event = Event.objects.get(slug=event_slug)
        except Event.DoesNotExist:
            raise Http404
        return super(EventHomePageView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        # return self.event.get_home_page_template_name()
        # return settings.PUBLIEDITOR_EVENT_TRIUMPH_TEMPLATE
        return settings.PUBLIEDITOR_EVENT_TRIUMPH_HOME_PAGE_TEMPLATE

    def get_form_class(self):
        from publieditor.event.triumph.forms import TriumphRegistrationFormStep1

        return TriumphRegistrationFormStep1

    def get_form_kwargs(self):
        kwargs = super(EventHomePageView, self).get_form_kwargs()
        kwargs['event'] = self.event
        return kwargs

    def form_valid(self, form):
        # CONTACT/PHONE CREATION
        self.contact, created = Contact.objects.get_or_create(
            email__iexact=form.cleaned_data['email'],
            defaults={
                'email': form.cleaned_data['email'],
                'first_name': form.cleaned_data['first_name'],
                'last_name': form.cleaned_data['last_name'],
                'language_id': 1,  # Ita
                'privacy_acceptance_date': datetime.datetime.now(),
                'is_lead': True,
            }
        )
        django_translation.activate(self.contact.language.code)
        self.contact_already_exist = not created
        if created:
            triumph_group = ContactGroup.objects.get(name=settings.EVENT_CONTACTGROUP_TRIUMPH_NAME)
            self.contact.groups.add(triumph_group)
            Phone.objects.create(
                default=True,
                number=form.cleaned_data['mobile'],
                phone_type='M',  # Mobile
                contact=self.contact
            )
        else:
            Phone.objects.get_or_create(
                contact=self.contact,
                number=form.cleaned_data['mobile'],
                defaults={
                    'phone_type': 'M',  # Mobile
                }
            )

        self.invite, created = Invite.objects.get_or_create(
            contact=self.contact,
            event=self.event,
            defaults={
                'event_place': form.cleaned_data['favourite_event_place'],
                'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                'favourite_vehicle_model': form.cleaned_data['favourite_vehicle_model'],
                'step': 'register',
            },
        )
        self.invite.step = "register"
        self.invite.save()
        self.invite.send_confirmation_email()
        return super(EventHomePageView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        ctx = super(EventHomePageView, self).get_context_data(**kwargs)
        ctx['event'] = self.event
        ctx['places'] = self.event.places.filter(
            date__gte=datetime.date.today()
        )
        return ctx

    def get_success_url(self):
        # messages.warning(self.request, 'To proceed with the registration please click on link in the email that we have sent to you')
        messages.warning(self.request, 'Per procedere con la registrazione clicca sul link nell\'email che ti abbiamo inviato')
        return reverse('triumph:event_home_greetings', kwargs={'slug': self.invite.event.slug})


class EventHomePageThanksView(TemplateView):
    def get_template_names(self):
        return settings.PUBLIEDITOR_EVENT_TRIUMPH_HOME_PAGE_THANKS_TEMPLATE


class EventHostessHomePageView(ListView):

    search_filters = {}

    @method_decorator(login_required(login_url='/hostess/login/'))
    def dispatch(self, request, *args, **kwargs):
        self.event_slug = kwargs.get('slug')
        if self.request.method == 'POST':
            self.search_filters = {'event__slug': self.event_slug}
            hostess_event_place_ids = request.user.event_place_hostesses.values_list('id', flat=True)
            if hostess_event_place_ids:
                self.search_filters = {'event_place__id__in': hostess_event_place_ids}
            for key, value in self.request.POST.iteritems():
                if not key.startswith('csrf'):
                    if value:
                        if key == 'invite_id':
                            filter_value = base36_to_int(value)
                            self.search_filters['id'] = filter_value
                        else:
                            self.search_filters['contact__' + key + '__icontains'] = value
        return super(EventHostessHomePageView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        return "event/home_page/hostess.html"

    def get_context_data(self, **kwargs):
        ctx = super(EventHostessHomePageView, self).get_context_data(**kwargs)
        ctx['event_slug'] = self.event_slug
        from publieditor.event.triumph.forms import SearchInviteForm
        ctx['form'] = SearchInviteForm
        return ctx

    def get_queryset(self):
        if len(self.search_filters) == 0:
            return Invite.objects.none()
        if 'event_place__id__in' not in self.search_filters:
            return Invite.objects.none()
        return Invite.objects.filter(**self.search_filters)

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)


class EventHostessRegistrationCustomerView(FormView):
    invite_id = None

    @method_decorator(login_required(login_url='/hostess/login/'))
    def dispatch(self, request, *args, **kwargs):
        event_slug = kwargs.get('slug')
        try:
            self.event = Event.objects.get(slug=event_slug)
        except Event.DoesNotExist:
            raise Http404
        try:
            self.invite_id = base36_to_int(kwargs.get('invite'))
        except:
            self.invite_id = None
        self.hostess_event_place_ids = request.user.event_place_hostesses.values_list('id', flat=True)
        return super(EventHostessRegistrationCustomerView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(EventHostessRegistrationCustomerView, self).get_context_data(**kwargs)
        ctx['event_slug'] = self.event.slug
        return ctx

    def get_template_names(self):
        return "event/home_page/hostess_register_customer.html"

    def get_form_class(self):
        from publieditor.event.triumph.forms import TriumphCustomerRegistration
        return TriumphCustomerRegistration

    def get_form_kwargs(self):
        kwargs = super(EventHostessRegistrationCustomerView, self).get_form_kwargs()
        kwargs['invite_id'] = self.invite_id
        kwargs['event'] = self.event
        kwargs['hostess_event_place_ids'] = self.hostess_event_place_ids
        return kwargs

    def form_valid(self, form):
        # CONTACT/PHONE CREATION
        from jmb.i18n.models import Language
        from jmb.address.models import AddressScope

        license_types_qs = License.objects.filter(id__in=form.cleaned_data['license_type'])
        language = Language.objects.get(id=1)
        address_scope = AddressScope.objects.get(pk=5)

        contact_defaults = {
            'email': form.cleaned_data['email'],
            'first_name': form.cleaned_data['first_name'],
            'last_name': form.cleaned_data['last_name'],
            'language': language,  # Ita
            'privacy_acceptance_date': datetime.datetime.now(),
            'birth_place': form.cleaned_data['birth_place'],
            'birthday': form.cleaned_data['birthday'],
            'license_number': form.cleaned_data['license_number'],
            'license_issue_date': form.cleaned_data['license_issue_date'],
            'license_expiration_date': form.cleaned_data['license_expiration_date'],
            'license_emitted_by': form.cleaned_data['license_emitted_by'],
        }
        address_defaults = {
            'address_type': form.cleaned_data['address_type'],
            'name': form.cleaned_data['address_name'],
            'number': form.cleaned_data['address_number'],
            'zip_code': form.cleaned_data['address_zip_code'],
            'locality': form.cleaned_data['address_locality'],
            'province': form.cleaned_data['address_province'],
            'country': form.cleaned_data['address_country']
        }

        self.contact, created = Contact.objects.get_or_create(
            email__iexact=form.cleaned_data['email'],
            defaults=contact_defaults
        )
        if not created:
            Contact.objects.filter(id=self.contact.id).update(**contact_defaults)

        self.contact.license_types.add(*license_types_qs)

        django_translation.activate(self.contact.language.code)

        if created:
            triumph_group = ContactGroup.objects.get(name=settings.EVENT_CONTACTGROUP_TRIUMPH_NAME)
            self.contact.groups.add(triumph_group)
            Phone.objects.create(
                default=True,
                number=form.cleaned_data['mobile'],
                phone_type='M',  # Mobile
                contact=self.contact
            )
            address = ContactAddress.objects.create(
                contact=self.contact,
                ** address_defaults
            )
            address.address_scopes.add(address_scope)
            invite = Invite.objects.create(
                status=settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                contact=self.contact,
                event=self.event,
                assigned_vehicle=form.cleaned_data['assigned_vehicle'],
                assigned_place=form.cleaned_data['event_real_place']
            )
        else:
            Phone.objects.get_or_create(
                contact=self.contact,
                number=form.cleaned_data['mobile'],
                defaults={
                    'phone_type': 'M',  # Mobile
                }
            )

            home_address = ContactAddress.objects.filter(contact=self.contact, address_scopes__id=5)

            if home_address:
                ContactAddress.objects.filter(id=home_address[0].id).update(**address_defaults)
            else:
                address = ContactAddress.objects.create(
                    contact=self.contact,
                    **address_defaults
                )
                address.address_scopes.add(address_scope)

            invite, new_invite = Invite.objects.get_or_create(
                contact=self.contact,
                event=self.event,
                defaults={
                    'status': settings.PUBLIEDITOR_EVENT_INVITE_ACCEPTED,
                    'assigned_vehicle': form.cleaned_data['assigned_vehicle'],
                    'assigned_place': form.cleaned_data['event_real_place']
                }
            )
            if not new_invite:
                Invite.objects.filter(id=invite.id).update(
                    assigned_vehicle=form.cleaned_data['assigned_vehicle'],
                    assigned_place=form.cleaned_data['event_real_place']
                )
        self.invite = Invite.objects.get(pk=invite.id)

        return super(EventHostessRegistrationCustomerView, self).form_valid(form)

    def get_success_url(self):
        return reverse('triumph:hostess_print_cdu', kwargs={'slug': self.event.slug, 'invite': self.invite.make_base36()})


class EventHostessPrintCDUView(TemplateView):
    template_name = "event/home_page/hostess_thanks.html"

    @method_decorator(login_required(login_url='/hostess/login/'))
    def dispatch(self, request, *args, **kwargs):
        self.event_slug = kwargs.get('slug')
        invite_id = base36_to_int(kwargs.get('invite'))
        try:
            self.invite = Invite.objects.get(pk=invite_id)
        except:
            raise Http404
        if 'download' in request.path:
            file_name = self.invite.cdu.cdu_file.path
            with open(file_name, 'rb') as fh:
                response = HttpResponse(fh.read(), content_type="application/pdf")
                response['Content-Disposition'] = 'inline; filename=' + os.path.basename(file_name)
                return response
        return super(EventHostessPrintCDUView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        file_name = self.invite.generate_cdu_file()
        self.invite.send_cdu_email(file_name, delete_attachment_from_tmp=True)
        return super(EventHostessPrintCDUView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(EventHostessPrintCDUView, self).get_context_data(**kwargs)
        ctx['invite'] = self.invite
        ctx['event_slug'] = self.event_slug
        return ctx


class HostessLoginView(FormView):
    form_class = HostessLoginForm

    template_name = "event/home_page/hostess_login.html"

    def get_success_url(self):
        return reverse('triumph:hostess_event_home', kwargs={'slug': 'triumph-tour-2018'})

    def form_valid(self, form):
        login(self.request, form.user)
        return super(HostessLoginView, self).form_valid(form)

    def form_invalid(self, form):
        return super(HostessLoginView, self).form_invalid(form)


class HostessLogoutView(View):

    def get(self, request, *args, **kwargs):
        logout(request)
        return HttpResponseRedirect(reverse('triumph:hostess_login'))


class AjaxableResponseMixin(object):
    def render_to_json_response(self, context, **response_kwargs):
        data = json.dumps(context)
        response_kwargs['content_type'] = 'application/json'
        return HttpResponse(data, **response_kwargs)

    def form_invalid(self, form):
        response = super(AjaxableResponseMixin, self).form_invalid(form)
        if self.request.is_ajax():
            return self.render_to_json_response(form.errors, status=400)
        else:
            return response

    def form_valid(self, form):
        vehicle, created = Vehicle.objects.get_or_create(
            licence_plate=form.cleaned_data['licence_plate'],
            defaults={
                'vin': form.cleaned_data['vin'],
                'vehicle_model': form.cleaned_data['vehicle_model'],
                'color': form.cleaned_data['color'],
            }
        )
        event_places = self.request.user.event_place_hostesses.all()
        for ep in event_places:
            ep.vehicles.add(vehicle)
        response = super(AjaxableResponseMixin, self).form_valid(form)
        if self.request.is_ajax():
            data = {
                'vehicle_id': vehicle.id,
                'vehicle_name': vehicle.__unicode__(),
                'created': created
            }
            return self.render_to_json_response(data)
        else:
            return response


class AddVehicleView(AjaxableResponseMixin, FormView):
    form_class = TrumphNewVehicleForm

    def dispatch(self, request, *args, **kwargs):
        self.event_slug = kwargs.get('slug')
        return super(AddVehicleView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(AddVehicleView, self).get_context_data(**kwargs)
        ctx['event_slug'] = self.event_slug
        if not self.request.is_ajax():
            hostess_event_place_ids = self.request.user.event_place_hostesses.values_list('id', flat=True)
            ctx['vehicle_list'] = Vehicle.objects.filter(
                vehicle_model__vehicle_models_event_list__event__slug=self.event_slug,
                event_place_vehicles__id__in=hostess_event_place_ids
            ).order_by('vehicle_model__name', 'licence_plate').distinct()
        return ctx
        
    def get_form_class(self):
        return TrumphNewVehicleForm

    def get_form_kwargs(self):
        kwargs = super(AddVehicleView, self).get_form_kwargs()
        kwargs['hostess_event_place_ids'] = self.request.user.event_place_hostesses.values_list('id', flat=True)
        return kwargs

    def get_template_names(self):
        if self.request.is_ajax():
            return "event/home_page/add_vehicle_form.html"
        return "event/home_page/hostess_add_vehicle_page.html"

    def get_success_url(self):
        if self.request.is_ajax():
           return "/"
        return reverse("triumph:hostess_event_add_vehicle", kwargs={'slug': self.event_slug})
